/*        File: parint.h
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: All
   Description: Top level header file, to be #include'd by users
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/include/parint.h,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:27 $

                $Log: parint.h,v $
                Revision 1.1.1.1  2000/06/06 03:13:27  parint
                Initial checkin


       Version: 01/27/98  rrz  Original
		06/18/98  rrz  Added proto pi_setopt_ms()
		10/22/98  rrz  For keeping heap: Added PI_OPT_KILLHEAP bit to
				pi_setopt()
		12/18/98  rrz  Add bit option to pi_setopt() for forcing simple
				initial region selection
		05/12/99  rrz  Add fcn_evals_per_rule to pi_status_t
		05/25/99  rrz  Remove pi_setopt_ms()
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
		03/25/00  rrz  Add proto for pi_setotp_max_heap_size()
*/

/* This header file provides basic definitions and prototypes across all versions of ParInt,
 * for those items that the user needs to be aware of.  (Basic definitions the user does not
 * need to see are in pi-defs.h)
 */

#ifndef PARINT_H
#define PARINT_H

#include <pi-version.h>     /* (this include provides version-specific definitions) */

#ifndef FORCE_NO_MPI
#include <mpi.h>            /* (for MPI_Comm type)                                  */
#endif

#ifdef USING_LONG_DOUBLES
typedef long double   pi_base_t;
#else
typedef double        pi_base_t;
#endif

typedef int (* pi_ifcn_t) (int *, pi_base_t *, int *, pi_base_t *);

/* For script file paths, etc: */
#define PI_MAX_PATH    100

/* Max argument to pi_setopt_nruns(): */
#define PI_MAX_NRUNS   100

typedef struct {
    pi_base_t *a;
    pi_base_t *b;
    int       ndims;
} pi_hregion_t;

/* Note that fcn_limit_flag holds the # of times we hit the flag, in a multi-run run, so 
 * even though I might be tempted to present a bool here, the 'int' is really correct.
 * In addition, I do not want to worry about naming conflicts between my 'bool' type
 * and any the user may define.
 */
typedef struct {
    double total_time;
    int    fcn_eval_count;
    int    fcn_limit_flag;
    int    fcn_evals_per_rule;
} pi_status_t;


/* The rank of the parint controller: */
#define pi_controller(rank)    ((rank) == 0)

/* These are user-callable functions, and may also be called internally from ParInt code.
 * The 'ifdef' used here is so that the seq version can call some of these functions (the
 * seq version is *not* an MPI program).  Later, this may switch between different versions
 * of ParInt (MPI, PVM, etc.). */
extern pi_hregion_t *pi_allocate_hregion(int ndims);
extern void pi_free_hregion(pi_hregion_t *hregion_p);
#ifndef FORCE_NO_MPI
extern int pi_init(MPI_Comm comm);
extern int pi_init_mpi(int *argc_ptr, char **argv_ptr[], int *rank, int *comm_size);
#endif
extern int pi_finalize(void);
extern int pi_setopt(int flags);
extern int pi_setopt_help_ratio(double help_ratio);
extern int pi_setopt_nruns(int nruns);
extern int pi_setopt_ns(int ns);
extern int pi_setopt_max_heap_size(int max_heap_size);
extern void pi_print_results(pi_base_t result[], pi_base_t estabs[], int nfcns, 
			     pi_status_t *status_p);
extern int pi_integrate(pi_ifcn_t ifcn_p, int nfcns, int intg_rule, int fcn_eval_limit,
			pi_base_t eps_a, pi_base_t eps_r, void *region_p, int region_type,
			pi_base_t result[], pi_base_t estabs[], pi_status_t *status_p);


/* Flags for pi_setopt() (defaults are always all-flags-false): */
#define PI_OPT_DEFAULTS    0x0000   /* Use to reset to defaults     */
#define PI_OPT_NOLOADBAL   0x0001   /* Turns off load balancing     */
#define PI_OPT_NOICWORKS   0x0002   /* Turns off IC working as IW   */
#define PI_OPT_SIMPRGNS    0x0008   /* Turns on forcing of simple initial region selection */

/* Values for region type */
#define PI_RGN_HRECT    1

/* Values for integration rules: */
#define PI_IRULE_DIM2_DEG13     1
#define PI_IRULE_DIM3_DEG11     2
#define PI_IRULE_DEG9_OSCIL     3
#define PI_IRULE_DEG7           4
#define PI_IRULE_DQK15          5
#define PI_IRULE_DQK21          6
#define PI_IRULE_DQK31          7
#define PI_IRULE_DQK41          8
#define PI_IRULE_DQK51          9
#define PI_IRULE_DQK61         10

/* Error constants: */
#define PI_ERR_FN_ORDER     1
#define PI_ERR_NUM_PE       2
#define PI_ERR_PARMS        3
#define PI_ERR_NO_SUPPORT   4  /* option not yet supported in ParInt */
#define PI_ERR_INTERNAL     5  /* eg: ENOMEM, ... */

#endif /* PARINT_H */
