/*        File: sample1.c
     Copyright: (C) 1998, 1999,2000 by Elise de Doncker, Ajay Gupta, 
                       Alan Genz and Rodger Zanny. All rights reserved. 
		       See Copyright.txt in top-level directory.
  ParInt Versn: MPI versions
   Description: A sample user application using ParInt as a library of routines
       Version: 02/21/98  rrz  Original
*/

/* Note that this simple example only works when ParInt is compiled to
   use doubles (the default setup) */

#include <mpi.h>
#include <parint.h>

#define DIMS  3

static int fcn7(int *ndims, double x[], int *nfcns, double funvls[])
{
    double z = x[0] + x[1] + x[2];
    z *= z;
    funvls[0] = (z != 0.0 ? 1.0 / z : 0.0);
    return 0;
} /* fcn7() */


int main(int argc, char *argv[])
{
    double result, estabs;
    pi_status_t status;
    pi_hregion_t *hrgn_p;
    int nPE, rank, i;

    pi_init_mpi(&argc, &argv, &rank, &nPE);
    hrgn_p = pi_allocate_hregion(DIMS);
    for (i = 0; i < DIMS; i++)
    {
	hrgn_p->a[i] = 0.0;
	hrgn_p->b[i] = 1.0;
    }
    pi_integrate(fcn7, 1, PI_IRULE_DEG7, 400000, 1.0E-06, 1.0E-06, hrgn_p,
		 PI_RGN_HRECT, &result, &estabs, &status);
    pi_finalize();
    
    if (pi_controller(rank))
	pi_print_results(&result, &estabs, 1, &status);

    pi_free_hregion(hrgn_p);
    exit(0);

} /* main() */
