/*        File: ifcn-ctrl.h
     Copyright: (C) 1998, 1999 by Elise de Doncker, Ajay Gupta, Alan Genz and Rodger Zanny. 
                       All rights reserved. See Copyright.txt in top-level directory.
  ParInt Versn: All versions
   Description: Integrand function library control header file
       Version: 05/11/98  rrz  Original
		07/23/98  rrz  Add entries for fcn18 and fcn19 (good for
				extrapolation)
		08/07/98  rrz  Change default rule for fcn7 & fcn20 to be Rule
				#4
		08/13/98  rrz  Add fcn22
		08/30/98  rrz  Added fcn23
		11/17/98  rrz  Add default answers for fcn19 & fcn21 (albeit
				from ParInt!)
		05/12/99  rrz  By default, remove (comment-out) fcn20 (the
				FORTRAN function! ;-) )
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
		11/17/99  rrz  Add fcn4x functions; add fcn31b
		03/19/00  rrz  Add fcn24 (sample CERN function)*/

/* This file contains the list of all integrand library functions, specifically, the list of 
 * prototypes and the constant array-of-structures that holds all the needed info about
 * each function.  This file contains info about the sample list of functions provided with
 * the ParInt release.  Users will want to modify this or create their own for their own 
 * integrand functions
 */
#include <pi-defs.h>
#include <pi-ifcn.h>
#include <parint.h>
#include <math-misc.h>


/*** Default default values: */
#define DEF_NFCN      1
#define DEF_EA        1.0E-06
#define DEF_ER        1.0E-06
#define DEF_IR        PI_IRULE_DEG7
#define DEF_FLIM      400000


/*** Prototypes for ifcn functions: */
extern int fcn1(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn2(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn3(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn4(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn5(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn6(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn7(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn8(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn9(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn10(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn14(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn15(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn16(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn17(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn18(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn19(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn21(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn22(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn23(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn24(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn25(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn31(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn31b(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn32(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn34(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn35(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn41a(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn41b(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn42a(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn42b(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn42c(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn43a(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn43b(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn43c(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn43d(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn44a(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn44b(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn44c(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn44d(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn44e(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn44f(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int fcn44g(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
extern int finance1(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);

#if 0
extern int fcn20_(int *ndims, pi_base_t *x, int *nfcns, pi_base_t *funvls);
#endif

/* The list of integral functions. The functions are In no particular order.
 * Each entry is ordered according to the pi_ifcn_info_t type: 
 *   < name, desc, 
 *     ndims, nfcns, eps_a, eps_r, intg_rule, fcn_count_limit, ifcn_ptr, 
 *                init_fcn_ptr, NULL, NULL, def_answer,
 *     a[], b[] >
 * (The extra NULL's are for future expansion)
 *
 */
static pi_ifcn_info_t pi_ifcn_info_table[] = 
{
    { "fcn1", "f(x) = x0 * x1^2 * sin(x2) / (4+x3+x4+x5)",
      6, DEF_NFCN, DEF_EA, DEF_ER, DEF_IR, DEF_FLIM, fcn1, NULL, NULL, 1.434761888397263,
      { 0.0, 0.0, 0.0,     -1.0, -1.0, -1.0, 0.0, 0.0, 0.0, 0.0 },
      { 2.0, 1.0, PI_PI_2,  1.0,  1.0,  1.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn2", "f(x) = x2^2 * x3 * exp(x2*x3) / (x0+x1+1)^2", 
      4, DEF_NFCN, DEF_EA, DEF_ER, DEF_IR, DEF_FLIM, fcn2, NULL, NULL, 0.5753641449035616,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 2.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn3", "f(x) = 8 / (1 + 2(x0+x1+x2))",
      3, DEF_NFCN, DEF_EA, DEF_ER, PI_IRULE_DIM3_DEG11, DEF_FLIM, fcn3, NULL, NULL,
      2.152142832595894,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn4", "f(x) = cos(x0 + x1 + x2 + x3 + x4)",
      5, DEF_NFCN, DEF_EA, DEF_ER, PI_IRULE_DEG9_OSCIL, DEF_FLIM, fcn4, NULL, NULL, 16.0,
      { 0.0,   0.0,   0.0,   0.0,   0.0,     0.0, 0.0, 0.0, 0.0, 0.0 },
      { PI_PI, PI_PI, PI_PI, PI_PI, PI_PI_2, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn5", "f(x) = sin(10*x1) [4 dims]",
      4, DEF_NFCN, DEF_EA, DEF_ER, PI_IRULE_DEG9_OSCIL, DEF_FLIM, fcn5, NULL, NULL,
      0.1839071529076452,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn6", "f(x) = cos(x0 + x1)",
      2, DEF_NFCN, DEF_EA, DEF_ER, PI_IRULE_DEG9_OSCIL, DEF_FLIM, fcn6, NULL, NULL, -4.0,
      { 0.0,     0.0,     0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 3*PI_PI, 3*PI_PI, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    /* FCN7 (and fcn21, the same fcn) are 3dim functions, but do much better with Rule #4 */
    { "fcn7", "f(x) = 1 / (x0 + x1 + x2)^2", 
      3, DEF_NFCN, DEF_EA, DEF_ER, DEF_IR, DEF_FLIM, fcn7, NULL, NULL, 0.8630462173553432,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn7b", "f(x) = 1 / (x0 + x1 + x2 )^2 [over 10 dims]", 
      10, DEF_NFCN, DEF_EA, DEF_ER, DEF_IR, DEF_FLIM, fcn7, NULL, NULL, 0.8630462173553432,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 } },

    { "fcn8", "f(x) = 605*x1 / ((1+120(1-x1))((1+120(1-x1))^2+25x0^2*x1^2)",
      2, DEF_NFCN, DEF_EA, DEF_ER, PI_IRULE_DIM2_DEG13, DEF_FLIM, fcn8, NULL, NULL, 
      1.047591113142868,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn9", "f(x) = 1 / ((x0^2 + 0.00001) * ((x1+0.25)^2 + 0.0001))",
      2, DEF_NFCN, DEF_EA, DEF_ER, PI_IRULE_DIM2_DEG13, DEF_FLIM, fcn9, NULL, NULL, 
      499.1249442241215,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn10", "f(x) = exp(|x0 + x1 - 1|)",
      2, DEF_NFCN, DEF_EA, DEF_ER, PI_IRULE_DIM2_DEG13, DEF_FLIM, fcn10, NULL, NULL,
      1.436563656918090,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn14", "f(x) = x<=1.0 ? 1/sqrt(x) : 1/sqrt(x-2)",
      2, DEF_NFCN, DEF_EA, DEF_ER, DEF_IR, DEF_FLIM, fcn14, NULL, NULL, 4.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 2.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn15", "f(x) = x^2 + 5x + 10",
      2, DEF_NFCN, DEF_EA, DEF_ER, DEF_IR, DEF_FLIM, fcn15, NULL, NULL, 12.8333333333333,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn16", "f(x) = exp(x + y)",
      2, DEF_NFCN, DEF_EA, DEF_ER, DEF_IR, DEF_FLIM, fcn16, NULL, NULL, 2.95249244199546,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn17", "f(x) = x^i + i * x + i; 1 <= i <= 10",
      2, 10, DEF_EA, DEF_ER, DEF_IR, DEF_FLIM, fcn17, NULL, NULL, 1.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn18", "f(x) = x^-0.2 * y^-0.2 * (x+y)^-0.2",
      2, DEF_NFCN, DEF_EA, DEF_ER, PI_IRULE_DIM2_DEG13, DEF_FLIM, fcn18, NULL, NULL, 1.667629948,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    /* Answer from ParInt (so not guarranteed to all digits): */
    { "fcn19", "f(x) = Like fcn18 in 4 dims",
      4, DEF_NFCN, DEF_EA, DEF_ER, DEF_IR, DEF_FLIM, fcn19, NULL, NULL, 2.211,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0 } },

#if 0
    { "fcn20", "f(x) = 1 / (x0 + x1 + x2)^2; in Fortran", 
      3, DEF_NFCN, DEF_EA, DEF_ER, DEF_IR, DEF_FLIM, fcn20_, NULL, NULL, 0.8630462173553432,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },
#endif

    /* Answer from ParInt (so not guarranteed to all digits): */
    { "fcn21", "f(x) = x^-0.2 * y^-0.2 * z^-0.2 * (x+y+z)^-0.2",
      3, DEF_NFCN, DEF_EA, DEF_ER, PI_IRULE_DIM3_DEG11, DEF_FLIM, fcn21, NULL, NULL, 1.887873,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn22", "f(x) = 1.0 / sqrt(x*y)",
      2, DEF_NFCN, DEF_EA, DEF_ER, PI_IRULE_DIM2_DEG13, DEF_FLIM, fcn22, NULL, NULL, 4.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn23", "f(x) = x^-0.9",
      1, DEF_NFCN, DEF_EA, DEF_ER, PI_IRULE_DQK15, DEF_FLIM, fcn23, NULL, NULL, 10.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn23b", "f(x) = x^-0.9 (in 6 dims)",
      6, DEF_NFCN, DEF_EA, DEF_ER, DEF_IR, DEF_FLIM, fcn23, NULL, NULL, 10.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn24", "f(x) = CERN test peak fcn (in 2 dims)",
      2, DEF_NFCN, 0.0, 1.0E-2, PI_IRULE_DIM2_DEG13, DEF_FLIM, fcn24, NULL, NULL, 3.1413,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn25", "f(x) = 2 pt singularities in 4 dims",
      4, DEF_NFCN, 0.0, 1.0E-2, DEF_IR, DEF_FLIM, fcn25, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    /* Answer from ParInt */
    { "fcn31", "f(x) = 1 / x^0.9", 
      1, DEF_NFCN, 0.0, DEF_ER, PI_IRULE_DQK15, DEF_FLIM, fcn31, NULL, NULL, 10.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn31b", "f(x) = 1 / x^0.99", 
      1, DEF_NFCN, 0.0, DEF_ER, PI_IRULE_DQK15, DEF_FLIM, fcn31b, NULL, NULL, 0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    /* Answer from ParInt */
    { "fcn32", "f(x) = 1 / (x+y)", 
      2, DEF_NFCN, DEF_EA, DEF_ER, DEF_IR, DEF_FLIM, fcn32, NULL, NULL, 1.3862943611,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn34", "f(x) = 1 / (x0 + x1 + x2 + x3)^3",
      4, DEF_NFCN, DEF_EA, DEF_ER, DEF_IR, DEF_FLIM, fcn34, NULL, NULL, 0.33979807084,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    /* Answer from ParInt */
    { "fcn35", "f(x) = 1 / (sum all x[i])^2)",
      10, DEF_NFCN, 0.0, 1.0E-3, DEF_IR, DEF_FLIM, fcn35, NULL, NULL, 0.044832,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 } },

    { "fcn41a", "1D Pt Sing. at (0.0), beta = -0.9",
      1, DEF_NFCN, 0.0, DEF_ER, PI_IRULE_DQK15, DEF_FLIM, fcn41a, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },
    { "fcn41b", "1D Pt Sing. at (0.5), beta = -0.9",
      1, DEF_NFCN, 0.0, DEF_ER, PI_IRULE_DQK15, DEF_FLIM, fcn41b, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn42a", "2D Pt Sing. at (0.1,0.1), beta = -1.9",
      2, DEF_NFCN, 0.0, 1.0e-5, PI_IRULE_DIM2_DEG13, 5000000, fcn42a, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },
    { "fcn42b", "2D Pt Sing. at (0.1,0.5), beta = -1.9",
      2, DEF_NFCN, 0.0, 1.0e-5, PI_IRULE_DIM2_DEG13, 5000000, fcn42b, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },
    { "fcn42c", "2D Pt Sing. at (0.5,0.5), beta = -1.9",
      2, DEF_NFCN, 0.0, 1.0e-5, PI_IRULE_DIM2_DEG13, 5000000, fcn42c, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn43a", "3D Pt Sing. at (0.1,0.1,0.1), beta = -2.9",
      3, DEF_NFCN, 0.0, 1.0e-3, PI_IRULE_DIM3_DEG11, 5000000, fcn43a, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },
    { "fcn43b", "3D Pt Sing. at (0.1,0.1,0.5), beta = -2.9",
      3, DEF_NFCN, 0.0, 1.0e-3, PI_IRULE_DIM3_DEG11, 5000000, fcn43b, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },
    { "fcn43c", "3D Pt Sing. at (0.1,0.5,0.5), beta = -2.9",
      3, DEF_NFCN, 0.0, 1.0e-3, PI_IRULE_DEG7, 5000000, fcn43c, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },
    { "fcn43d", "3D Pt Sing. at (0.5,0.5,0.5), beta = -2.9",
      3, DEF_NFCN, 0.0, 1.0e-3, PI_IRULE_DIM3_DEG11, 5000000, fcn43d, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 } },

    { "fcn44a", "6D Pt Sing. at (0.1,0.1,0.1,0.1,0.1,0.1), beta = -2.0",
      6, DEF_NFCN, 0.0, 1.0e-2, DEF_IR, 5000000, fcn44a, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0 } },
    { "fcn44b", "6D Pt Sing. at (0.1,0.1,0.1,0.1,0.1,0.5), beta = -2.0",
      6, DEF_NFCN, 0.0, 1.0e-2, DEF_IR, 5000000, fcn44b, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0 } },
    { "fcn44c", "6D Pt Sing. at (0.1,0.1,0.1,0.1,0.5,0.5), beta = -2.0",
      6, DEF_NFCN, 0.0, 1.0e-2, DEF_IR, 5000000, fcn44c, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0 } },
    { "fcn44d", "6D Pt Sing. at (0.1,0.1,0.1,0.5,0.5,0.5), beta = -2.0",
      6, DEF_NFCN, 0.0, 1.0e-2, DEF_IR, 5000000, fcn44d, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0 } },
    { "fcn44e", "6D Pt Sing. at (0.1,0.1,0.5,0.5,0.5,0.5), beta = -2.0",
      6, DEF_NFCN, 0.0, 1.0e-2, DEF_IR, 5000000, fcn44e, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0 } },
    { "fcn44f", "6D Pt Sing. at (0.1,0.5,0.5,0.5,0.5,0.5), beta = -2.0",
      6, DEF_NFCN, 0.0, 1.0e-2, DEF_IR, 5000000, fcn44f, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0 } },
    { "fcn44g", "6D Pt Sing. at (0.5,0.5,0.5,0.5,0.5,0.5), beta = -2.0",
      6, DEF_NFCN, 0.0, 1.0e-2, DEF_IR, 5000000, fcn44g, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0 } }
#if 0
    { "finance1", "10D Financial Integral; nearly-linear case",
      10, DEF_NFCN, 0.0, 1.0e-2, DEF_IR, 5000000, finance1, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0 } },
    { "finance2", "8D Financial Integral; nearly-linear case",
      8, DEF_NFCN, 0.0, 1.0e-2, DEF_IR, 5000000, finance1, NULL, NULL, 0.0,
      { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 },
      { 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.0, 0.0 } }
#endif


};

#define PI_IFCN_NUM_IFCNS   (sizeof(pi_ifcn_info_table) / sizeof(pi_ifcn_info_table[0]))
