/*        File: heap.h
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: All versions
   Description: Header file for heap functions
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/include/heap.h,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:31 $

                $Log: heap.h,v $
                Revision 1.1.1.1  2000/06/06 03:13:31  parint
                Initial checkin


       Version: 06/??/95  ???  Changed to vector of integrands and, hence, of errors: rgnerr_v
                                Now rgnerr = infNorn(rgnerr_v,numfun)  >=  0.
                11/??/95  ???  Added more functions (heapDeleteMax, heapRemoveLast, heapEmpty, and
      	                        delet_heap) putting this heap.h and heap.c in line with the one
				used in localhp/single
		01/13/98  rrz  Prepped for Release 1.0
		05/12/98  rrz  Code freeze for ParInt1.0
		09/21/98  rrz  Add proto for heap_merge(), heap_create_empty(),
				and delete_hnode() (from extrap; gheap
				versions)
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
*/

/* This header file contains definitions and prototypes for the heap handling functions */

#ifndef heap_h
#define heap_h

/*  Declare heap handling functions:                                        */
/*    The heap is maintained as a complete, balanced binary tree which      */
/*    satisfies the "heap condition".  The heap condition is the condition  */
/*    that the data value of the root of the tree is larger than or equal   */
/*    to the data value of each of its children and that each child is      */
/*    also a heap.                                                          */

#include <pi-defs.h>


typedef struct node {
      region_t       rgn;
      struct node   *p_nptr;    /* pointer to parent node          */
      struct node   *lc_nptr;   /* ptr to left child node          */
      struct node   *rc_nptr;   /* ptr to right chidl node         */
} hnode;

typedef struct {
      hnode   *rootptr;
      hnode   *availptr;        /* ptr to node to which to add next node */  
      hnode   *lastptr;         /* ptr to last node added to heap        */
} heap_t;


/* Returns the region at the top without removing it from the heap: */
#define heap_peek_at_top(hp)  ((hp).rootptr->rgn)


/****************************************************************************/
/*                                                                          */
/* create_heap                                                              */
/*                                                                          */
/* Creates the heap *hptr, setting the root's region data to newrgn and set-  */
/*   ting availptr to rootptr.  availptr always points to the node to which   */
/*   a new leaf can be added to the heap so that the resulting tree will      */
/*   still be a complete, balanced binary tree.  lastptr is also set rootptr  */
/*   since lastptr always must point to that most recently added node to the  */
/*   heap.                                                                    */

void create_heap(heap_t *hptr, region_t newrgn);



/****************************************************************************/
/*                                                                          */
/* replace_root                                                             */
/*                                                                          */
/* Given the heap with root pointed to by rootptr, this function sets the     */
/* root's rgn (region) data to newrgn and then readjusts the complete,        */
/* balanced tree so that it again satisfies the "heap condition".  So, the    */
/* algorithm starts at the `top' of the tree and shifts data `down' as neces- */
/* sary until the newly added region data is in the correct position to give  */
/* give a heap.                                                               */

void replace_root(heap_t *hptr, region_t newrgn); 



/****************************************************************************/
/*                                                                          */
/* add_heap                                                                 */
/*                                                                          */
/* Attaches a new leaf to the heap *hptr at the position required in order    */
/* to maintain a complete, balanced binary tree and then re-heapifies the     */
/* tree.  "Re-heapifies" means that the algorithm starts at the new leaf and  */
/* shifts data `up' the tree as necessary until the newly added region data   */
/* is in the correct position to give a heap.                                 */
/*                                                                            */
/* availptr points to the node to which the new leaf should be attached.  If  */
/* the new leaf is added as a right-child of the node, then a new available   */
/* parent node must be found (to be used in the next call to add_heap) by a   */
/* call to get_new_avail_parent.                                              */
/*                                                                            */
/* Sets lastptr to point to the new leaf.                                     */

void add_heap(heap_t *hptr, region_t newrgn);


/******************************************************************************/
/*                                                                            */
/* heapDeleteMax                                                              */
/*                                                                            */
/* Removes the region at the root of the heap; moves the data from the node   */
/* pointer to by the heap's lastptr; and then re-adjusts the tree so that it  */
/* again satisfies the "heap condition".                                      */
/*                                                                            */
/* Calls heapRemoveLast.  This function changes lastptr and may change        */
/* availptr.                                                                  */
/*                                                                            */
/* Calls replace_root to do the actual top-down heapification                 */

void heapDeleteMax(heap_t *hptr, region_t *maxptr);



/******************************************************************************/
/*                                                                            */
/* heapRemoveLast                                                             */
/*                                                                            */
/* Removes node *lastptr from the heap.  Updates heap's lastptr and, if       */
/* necessary, updates the heap's availptr.                                    */

void heapRemoveLast(heap_t *hptr);



/******************************************************************************/
/*                                                                            */
/* heapEmpty                                                                  */
/*                                                                            */
/* Returns TRUE if heap is empty; FALSE, otherwise                            */

extern bool heapEmpty(heap_t hp) ;


/******************************************************************************/
/*                                                                            */
/* delete_heap                                                                */
/*                                                                            */
/* Frees all memory allocation for the binary tree with root hptr using post- */
/*    order traversal of the tree                                             */

void delete_heap(heap_t *hptr);

extern void heap_merge(heap_t *hp_dst, heap_t *hp_src);
extern void create_heap_empty(heap_t *heap_p);
extern void delete_hnode(hnode *p);

#endif /* HEAP_H */
