/*        File: heapdeap.h
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: All versions
   Description: Header file for heap/deap functions
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/include/heapdeap.h,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:31 $

                $Log: heapdeap.h,v $
                Revision 1.1.1.1  2000/06/06 03:13:31  parint
                Initial checkin


       Version: 02/18/00  rrz  Created from deap.h and heap.h
*/

/* Note that this header file is now used throughout the ParInt versions.  In the next release,
 * I will eliminate the heap.[ch] files.
 */

#ifndef HEAPDEAP_H
#define HEAPDEAP_H

typedef struct node {
    region_t       rgn;
    struct node *  p_nptr;       /* pointer to parent node                        */
    struct node *  lc_nptr;      /* ptr to left child node                        */
    struct node *  rc_nptr;      /* ptr to right child node                       */
    struct node *  partner;      /* ptr to partner node in other heap (if a deap) */
} hnode;

typedef struct {
    bool is_a_deap;
    hnode *  maxrtptr;          /* ptr to root of the max-heap           */
    hnode *  availptr;          /* ptr to node to which to add next node */
    hnode *  lastptr;           /* ptr to last node added to the heap    */

    /* The following fields defined only if we are a deap: */
    hnode *  minrtptr;          /* ptr to root of the min-heap */
    int      maxhp;             /* 1 whenever availptr pts to a node in max-heap; 0 otw */
    int      num_nodes;         /* # of nodes in the heap/deap */
} heapdeap_t;

/*** Operations for both heaps and deaps: */

/* NOTE TO ME: Verify that these work on both!!! */


extern char *print_region(region_t *rgn_p);
extern void heap_print(heapdeap_t *hpdp_p, int id);
extern void heap_create(heapdeap_t *hpdp_p, bool is_a_deap);
extern bool heap_empty(const heapdeap_t *hpdp_p);
extern pi_base_t heap_max_key(const heapdeap_t *hpdp_p);
extern void heap_merge(heapdeap_t *hp_dst, heapdeap_t *hp_src);
extern void heap_free(heapdeap_t *hpdp_p);
extern void heap_add_node(heapdeap_t *hpdp_p, region_t *newrgn_p);
extern void heap_replace_root(heapdeap_t *hpdp_p, const region_t *newrgn_p);
extern void heap_delete_max(heapdeap_t *hpdp_p, region_t *maxrgn_p);
extern region_t *heap_peek_max(heapdeap_t *hpdp_p);
extern int heap_size(heapdeap_t *hpdp_p);

/*** Operations for just deaps: */
extern void deap_delete_min(heapdeap_t *d, region_t *minptr);
extern pi_base_t deap_min_key(const heapdeap_t *hpdp_p);
extern void deap_print(heapdeap_t *hpdp_p, int id);

#endif /* HEAPDEAP_H */
