/*        File: math-misc.h
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: All versions
   Description: Header file for generic math functions
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/include/math-misc.h,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:31 $

                $Log: math-misc.h,v $
                Revision 1.1.1.1  2000/06/06 03:13:31  parint
                Initial checkin


       Version: 01/13/98  rrz  Original
		05/12/98  rrz  Code freeze for ParInt1.0
		11/09/98  rrz  Add RUID() macro
		07/14/99  rrz  Add stuff for new integration rules
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
*/

#ifndef MATH_MISC_H
#define MATH_MISC_H

#include <math.h>      /* for math constants */
#include <float.h>     /* for machine precision info */
#include <parint.h>

extern pi_base_t max(pi_base_t x, pi_base_t y);
extern pi_base_t absol(pi_base_t x);
extern pi_base_t infNorm(const pi_base_t v[], int size);
extern int flog2(int x);
extern int clog2(int x);
extern char *strncpy0(char *dest, const char *src, int n);

/* Raises 2 to the nth power: */
#define POW2(n)      (1 << (n))

/* Round-up integer division, i.e., (int) ceil((double) x / (double) y): */
#define RUID(x, y)  (((x) + (y) - 1) / (y))

/* min() and max() as macros (careful of the arguments, e.g., min(x, ++y) will be bad!): */
#define MIN(a, b)    (((a) <= (b)) ? (a) : (b))
#define MAX(a, b)    (((a) >= (b)) ? (a) : (b))


/* This re-defines some macros and constants based on whether we are using doubles
 * or long doubles.  If you really want to always use one or the other, then do not
 * use these versions of these items: */
#ifdef USING_LONG_DOUBLES

#include <sunmath.h>

#define pi_sqrt(x)    sqrtl(x)
#define pi_cos(x)     cosl(x)
#define pi_sin(x)     sinl(x)
#define pi_exp(x)     expl(x)
#define pi_pow(x, y)  powl(x, y)
#define pi_log(x)     logl(x)

#define PI_PI       3.141592653589793238462643383279502L
#define PI_PI_2     1.570796326794896619231321691639751L
#define PI_LN2      0.693147180559945309417232121458177L
#define PI_DIG      LDBL_DIG
#define PI_EPSILON  LDBL_EPSILON
#define PI_MIN      LDBL_MIN   

#else

#define pi_sqrt(x)    sqrt(x)
#define pi_cos(x)     cos(x)
#define pi_sin(x)     sin(x)
#define pi_exp(x)     exp(x)
#define pi_pow(x, y)  pow(x, y)
#define pi_log(x)     log(x)

#define PI_PI       M_PI
#define PI_PI_2     M_PI_2
#define PI_LN2      M_LN2
#define PI_DIG      DBL_DIG
#define PI_EPSILON  DBL_EPSILON
#define PI_MIN      DBL_MIN   

#endif

#endif /* MATH_MISC_H */
