/*        File: admint.c
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: All
   Description: Source for basic integration rules (#1-#4)
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/math/admint.c,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:29 $

                $Log: admint.c,v $
                Revision 1.1.1.1  2000/06/06 03:13:29  parint
                Initial checkin


       Version: 04/02/98  rrz  Added header, made modifications for long doubles
		05/12/98  rrz  Code freeze for ParInt1.0
		04/29/99  rrz  BUG FIX: Change abs() to absol() to avoid weird
				Argonne errors
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
*/

/* Note: This file was originally converted from admint.f.  Later, we
needed to add long double capability.  It was decided that it would be
easier to change the .c, rather than change the .f and reconvert using
f2c.  Especially since f77 does not support any form of typedef's. If
we ever do want to re-convret, these changes will have to be re-done,
so they are laid out in detail here.  These changes are:

 - (LATER): Change use of abs() to absol()
 - Removal of function trestr_(), as it is completely unused
 - With the exception of the large arrays in bsdim2_() and bsdim3_(), all doublereal variables
    were changed to be of type pi_base_t.  These arrays would have to be recalculated to
    greater accuracy before they can really be long double values.  It is considered 'ok'
    to leave them as is.
 - All casts to doublereal were changed to be casts to pi_base_t
 - The only math function that is used that needs to be handled is the use of sqrt().  All calls
    to sqrt() are now calls to pi_sqrt(), a macro defined just below, which evaluates to either
    sqrtl() or sqrt().
 - Changes to top section (#include's, macros, etc...)

Also note:
 - The fn pow() is also used, but only in finding 2^k for some k, so it was left as is.
 - All 'double' variables (as opposed to 'doublereal') stayed doubles
 - Similarly, all casts to double stayed unchanged.
 - I did not 'fix' the type of the 'direct' parameter to basrul_().  (It only needs to hold an 
    integer, yet the .f code uses a double.  I do not know why this is.)  Along with all the other
    parameters, I made it be of type pi_base_t.  (I wanted to minimize changes to the code.)
*/


/* admint.f -- translated by f2c (version of 22 July 1992  22:54:52).
   You must link the resulting object file with the libraries:
	-lF77 -lI77 -lm -lc (in that order) */

#include "f2c.h"
#include <math.h>
#include <pi-version.h>
#include <parint.h>

/* I am re-doing this snippet of logic here, rather than #include'ing math-misc.h.  I do this
 * because there are some naming conflicts between items in f2c.h and math-misc.h: */
#ifdef USING_LONG_DOUBLES
#include <sunmath.h>
#define pi_sqrt(x)     sqrtl(x)
#else
#define pi_sqrt(x)     sqrt(x)
#endif

extern pi_base_t absol(pi_base_t x);

/* Table of constant values */
static integer c__2 = 2;

/* Subroutine */ int bsinit_(ndim, key, wtleng, w, g, errcof, rulpts, scales, 
	norms)
integer *ndim, *key, *wtleng;
pi_base_t *w, *g, *errcof, *rulpts, *scales, *norms;
{
    /* System generated locals */
    integer g_dim1, g_offset, i__1, i__2;
    pi_base_t d__1;
     double two=2.;
    /* Builtin functions */
   /* integer pow_ii();*/

    /* Local variables */
    static integer i, j, k;
    extern /* Subroutine */ int bsrl07_(), bsrl09_(), bsdim2_(), bsdim3_();
    static pi_base_t we[14];

/* ***BEGIN PROLOGUE BSINIT */
/* ***PURPOSE BSINIT computes abscissas and weights of the integration */
/*            rule and the null rules to be used in error estimation. */
/*            These are computed as functions of NDIM and KEY. */
/* ***DESCRIPTION BSINIT will for given values of NDIM and KEY compute or 
*/
/*            select the correct values of the abscissas and */
/*            corresponding weights for different */
/*            integration rules and null rules and assign them to */
/*            G and W. */
/*            The heuristic error coefficients ERRCOF */
/*            will be computed as a function of KEY. */
/*            Scaling factors SCALES and normalization factors NORMS */
/*            used in the error estimation are computed. */


/*   ON ENTRY */

/*     NDIM   Integer. */
/*            Number of variables. */
/*     KEY    Integer. */
/*            Key to selected local integration rule. */
/*     WTLENG Integer. */
/*            The number of weights in each of the rules. */

/*   ON RETURN */

/*     W      Real array of dimension (5,WTLENG). */
/*            The weights for the basic and null rules. */
/*            W(1,1), ...,W(1,WTLENG) are weights for the basic rule. */
/*            W(I,1), ...,W(I,WTLENG), for I > 1 are null rule weights. */

/*     G      Real array of dimension (NDIM,WTLENG). */
/*            The fully symmetric sum generators for the rules. */
/*            G(1,J),...,G(NDIM,J) are the generators for the points */
/*            associated with the the Jth weights. */
/*     ERRCOF Real array of dimension 6. */
/*            Heuristic error coefficients that are used in the */
/*            error estimation in BASRUL. */
/*            It is assumed that the error is computed using: */
/*             IF (N1*ERRCOF(1) < N2 and N2*ERRCOF(2) < N3) */
/*               THEN ERROR = ERRCOF(3)*N1 */
/*               ELSE ERROR = ERRCOF(4)*MAX(N1, N2, N3) */
/*             ERROR = ERROR + EP*(ERRCOF(5)*ERROR/(ES+ERROR)+ERRCOF(6)) 
*/
/*            where N1-N3 are the null rules, EP is the error for */
/*            the parent */
/*            subregion and ES is the error for the sibling subregion. */
/*     RULPTS Real array of dimension WTLENG. */
/*            A work array contaiong the number of points produced by */
/*            each generator of the selected rule. */
/*     SCALES Real array of dimension (3,WTLENG). */
/*            Scaling factors used to construct new null rules, */
/*            N1, N2 and N3, */
/*            based on a linear combination of two successive null rules 
*/
/*            in the sequence of null rules. */
/*     NORMS  Real array of dimension (3,WTLENG). */
/*            2**NDIM/(1-norm of the null rule constructed by each of */
/*            the scaling factors.) */

/* ***ROUTINES CALLED  BSDIM2,BSDIM3,BSRL07,BSRL09 */
/* ***END PROLOGUE BSINIT */

/*   Global variables. */


/*   Local variables. */


/* ***FIRST EXECUTABLE STATEMENT BSINIT */

/*   Compute W, G and ERRCOF. */

    /* Parameter adjustments */
    norms -= 4;
    scales -= 4;
    --rulpts;
    --errcof;
    g_dim1 = *ndim;
    g_offset = g_dim1 + 1;
    g -= g_offset;
    w -= 6;

    /* Function Body */
    if (*key == 1) {
	bsdim2_(wtleng, &w[6], &g[g_offset], &errcof[1], &rulpts[1]);
    } else if (*key == 2) {
	bsdim3_(wtleng, &w[6], &g[g_offset], &errcof[1], &rulpts[1]);
    } else if (*key == 3) {
	bsrl09_(ndim, wtleng, &w[6], &g[g_offset], &errcof[1], &rulpts[1]);
    } else if (*key == 4) {
	bsrl07_(ndim, wtleng, &w[6], &g[g_offset], &errcof[1], &rulpts[1]);
    }

/*   Compute SCALES and NORMS. */

    for (k = 1; k <= 3; ++k) {
	i__1 = *wtleng;
	for (i = 1; i <= i__1; ++i) {
	    if (w[k + 1 + i * 5] != 0.) {
		scales[k + i * 3] = -w[k + 2 + i * 5] / w[k + 1 + i * 5];
	    } else {
		scales[k + i * 3] = 100.;
	    }
	    i__2 = *wtleng;
	    for (j = 1; j <= i__2; ++j) {
		we[j - 1] = w[k + 2 + j * 5] + scales[k + i * 3] * w[k + 1 + 
			j * 5];
/* L30: */
	    }
	    norms[k + i * 3] = 0.;
	    i__2 = *wtleng;
	    for (j = 1; j <= i__2; ++j) {
		norms[k + i * 3] += rulpts[j] * (d__1 = we[j - 1], absol(d__1));

/* L40: */
	    }
	    norms[k + i * 3] = pow(two,(double)(*ndim)) / norms[k + i * 3];
/* L50: */
	}
/* L100: */
    }
    return 0;

/* ***END BSINIT */

} /* bsinit_ */

/* Subroutine */ int bsdim2_(wtleng, w, g, errcof, rulpts)
integer *wtleng;
pi_base_t *w, *g, *errcof, *rulpts;
{
    /* Initialized data */

    static doublereal dim2g[16] = { .2517129343453109,.7013933644534266,
	    .9590960631619962,.9956010478552127,.5,.1594544658297559,
	    .3808991135940188,.6582769255267192,.8761473165029315,
	    .998243184053198,.9790222658168462,.6492284325645389,
	    .8727421201131239,.3582614645881228,.5666666666666666,
	    .2077777777777778 };
    static doublereal dim2w[70]	/* was [14][5] */ = { .0337969236013446,
	    .09508589607597761,.1176006468056962,.0265777458632695,
	    .0170144177020064,0.,.0162659309863741,.1344892658526199,
	    .1328032165460149,.0563747476999187,.0039082790813105,
	    .0301279877743215,.1030873234689166,.0625,.3213775489050763,
	    -.1767341636743844,.07347600537466072,-.03638022004364754,
	    .02125297922098712,.1460984204026913,.01747613286152099,
	    .1444954045641582,1.307687976001325e-4,5.380992313941161e-4,
	    1.042259576889814e-4,-.001401152865045733,.008041788181514763,
	    -.1420416552759383,.3372900883288987,-.1644903060344491,
	    .07707849911634622,-.0380447835850631,.02223559940380806,
	    .1480693879765931,4.467143702185814e-6,.150894476707413,
	    3.647200107516215e-5,5.77719899901388e-4,1.041757313688177e-4,
	    -.001452822267047819,.008338339968783705,-.147279632923196,
	    -.8264123822525677,.306583861409436,.002389292538329435,
	    -.1343024157997222,.088333668405339,0.,9.786283074168292e-4,
	    -.1319227889147519,.00799001220015063,.003391747079760626,
	    .002294915718283264,-.01358584986119197,.04025866859057809,
	    .003760268580063992,.6539094339575232,-.2041614154424632,
	    -.174698151579499,.03937939671417803,.006974520545933992,0.,
	    .006667702171778258,.05512960621544304,.05443846381278607,
	    .02310903863953934,.01506937747477189,-.0605702164890189,
	    .04225737654686337,.02561989142123099 };

    static integer i, j;

/* ***BEGIN PROLOGUE BSDIM2 */
/* ***AUTHOR   Jarle Berntsen, Department of Informatics, University of */

/*            Bergen, Allegt. 55, N-5000 Bergen, NORWAY */
/* ***PURPOSE BSDIM2 computes abscissas and weights of a 2 dimensional */
/*            integration rule of degree 13. */
/*            Two null rules of degree 11, one null rule of degree 9 */
/*            and one null rule of degree 7 to be used in error */
/*            estimation are also computed. */
/* ***DESCRIPTION BSDIM2 will select the correct values of the abscissas 
*/
/*            and corresponding weights for different */
/*            integration rules and null rules and assign them to */
/*            G and W. The heuristic error coefficients ERRCOF */
/*            will also be assigned. */


/*   ON ENTRY */

/*     WTLENG Integer. */
/*            The number of weights in each of the rules. */

/*   ON RETURN */

/*     W      Real array of dimension (5,WTLENG). */
/*            The weights for the basic and null rules. */
/*            W(1,1),...,W(1,WTLENG) are weights for the basic rule. */
/*            W(I,1),...,W(I,WTLENG), for I > 1 are null rule weights. */
/*     G      Real array of dimension (NDIM,WTLENG). */
/*            The fully symmetric sum generators for the rules. */
/*            G(1,J),...,G(NDIM,J) are the generators for the points */
/*            associated with the the Jth weights. */
/*     ERRCOF Real array of dimension 6. */
/*            Heuristic error coefficients that are used in the */
/*            error estimation in BASRUL. */
/*     RULPTS Real array of dimension WTLENG. */
/*            The number of points produced by each generator. */
/* ***REFERENCES S.Eriksen, */
/*              Thesis of the degree cand.scient, Dept. of Informatics, */

/*              Univ. of Bergen,Norway, 1984. */

/* ***ROUTINES CALLED NONE */
/* ***END PROLOGUE BSDIM2 */

/*   Global variables */


/*   Local variables. */


    /* Parameter adjustments */
    --rulpts;
    --errcof;
    g -= 3;
    w -= 6;

    /* Function Body */






/* ***FIRST EXECUTABLE STATEMENT BSDIM2 */

/*   Assign values to W. */

    for (i = 1; i <= 14; ++i) {
	for (j = 1; j <= 5; ++j) {
	    w[j + i * 5] = dim2w[i + j * 14 - 15];
/* L10: */
	}
    }

/*   Assign values to G. */

    for (i = 1; i <= 2; ++i) {
	for (j = 1; j <= 14; ++j) {
	    g[i + (j << 1)] = 0.;
/* L20: */
	}
    }
    g[5] = dim2g[0];
    g[7] = dim2g[1];
    g[9] = dim2g[2];
    g[11] = dim2g[3];
    g[13] = dim2g[4];
    g[15] = dim2g[5];
    g[16] = g[15];
    g[17] = dim2g[6];
    g[18] = g[17];
    g[19] = dim2g[7];
    g[20] = g[19];
    g[21] = dim2g[8];
    g[22] = g[21];
    g[23] = dim2g[9];
    g[24] = g[23];
    g[25] = dim2g[10];
    g[26] = dim2g[11];
    g[27] = dim2g[12];
    g[28] = dim2g[13];
    g[29] = dim2g[14];
    g[30] = dim2g[15];

/*   Assign values to RULPTS. */

    rulpts[1] = 1.;
    for (i = 2; i <= 11; ++i) {
	rulpts[i] = 4.;
/* L30: */
    }
    rulpts[12] = 8.;
    rulpts[13] = 8.;
    rulpts[14] = 8.;

/*   Assign values to ERRCOF. */

    errcof[1] = 10.;
    errcof[2] = 10.;
    errcof[3] = 1.;
    errcof[4] = 5.;
    errcof[5] = .5;
    errcof[6] = .25;

/* ***END BSDIM2 */

    return 0;
} /* bsdim2_ */

/* Subroutine */ int bsdim3_(wtleng, w, g, errcof, rulpts)
integer *wtleng;
pi_base_t *w, *g, *errcof, *rulpts;
{
    /* Initialized data */

    static doublereal dim3w[65]	/* was [13][5] */ = { .007923078151105734,
	    .0679717739278808,.001086986538805825,.1838633662212829,
	    .03362119777829031,.01013751123334062,.001687648683985235,
	    .1346468564512807,.001750145884600386,.07752336383837454,
	    .2461864902770251,.06797944868483039,.01419962823300713,
	    1.715006248224684,-.3755893815889209,.1488632145140549,
	    -.2497046640620823,.1792501419135204,.00344612675897389,
	    -.005140483185555825,.006536017839876425,-6.5134549392297e-4,
	    -.006304672433547204,.01266959399788263,-.005454241018647931,
	    .004826995274768427,1.936014978949526,-.3673449403754268,
	    .02929778657898176,-.1151883520260315,.05086658220872218,
	    .04453911087786469,-.022878282571259,.02908926216345833,
	    -.002898884350669207,-.02805963413307495,.05638741361145884,
	    -.02427469611942451,.02148307034182882,.517082819560576,
	    .01445269144914044,-.3601489663995932,.3628307003418485,
	    .007148802650872729,-.09222852896022966,.01719339732471725,
	    -.102141653746035,-.007504397861080493,.01648362537726711,
	    .05234610158469334,.01445432331613066,.003019236275367777,
	    2.05440450381852,.0137775998849012,-.576806291790441,
	    .03726835047700328,.006814878939777219,.05723169733851849,
	    -.04493018743811285,.02729236573866348,3.54747395055699e-4,
	    .01571366799739551,.04990099219278567,.0137791555266677,
	    .002878206423099872 };
    static doublereal dim3g[14] = { .19,.5,.75,.8,.9949999999999999,
	    .99873449983514,.7793703685672423,.9999698993088767,
	    .7902637224771788,.4403396687650737,.4378478459006862,
	    .9549373822794593,.9661093133630748,.4577105877763134 };

    static integer i, j;

/* ***BEGIN PROLOGUE BSDIM3 */
/* ***AUTHOR   Jarle Berntsen, Department of Informatics, University of */

/*            Bergen, Allegt. 55, N-5000 Bergen, Norway */
/* ***PURPOSE BSDIM3 computes abscissas and weights of a 3 dimensional */
/*            integration rule of degree 11. */
/*            Two null rules of degree 9, one null rule of degree 7 */
/*            and one null rule of degree 5 to be used in error */
/*            estimation are also computed. */
/* ***DESCRIPTION BSDIM3 will select the correct values of the abscissas 
*/
/*            and corresponding weights for different */
/*            integration rules and null rules and assign them to G */
/*            and W. */
/*            The heuristic error coefficients ERRCOF */
/*            will also be computed. */


/*   ON ENTRY */

/*     WTLENG Integer. */
/*            The number of weights in each of the rules. */

/*   ON RETURN */

/*     W      Real array of dimension (5,WTLENG). */
/*            The weights for the basic and null rules. */
/*            W(1,1),...,W(1,WTLENG) are weights for the basic rule. */
/*            W(I,1),...,W(I,WTLENG), for I > 1 are null rule weights. */
/*     G      Real array of dimension (NDIM,WTLENG). */
/*            The fully symmetric sum generators for the rules. */
/*            G(1,J),...,G(NDIM,J) are the generators for the points */
/*            associated with the the Jth weights. */
/*     ERRCOF Real array of dimension 6. */
/*            Heuristic error coefficients that are used in the */
/*            error estimation in BASRUL. */
/*     RULPTS Real array of dimension WTLENG. */
/*            The number of points used by each generator. */

/* ***REFERENCES  J.Berntsen, Cautious adaptive numerical integration */
/*               over the 3-cube, Reports in Informatics 17, Dept. of */
/*               Inf.,Univ. of Bergen, Norway, 1985. */
/*               J.Berntsen and T.O.Espelid, On the construction of */
/*               higher degree three-dimensional embedded integration */
/*               rules, SIAM J. Numer. Anal.,Vol. 25,No. 1, pp.222-234, */

/*               1988. */
/* ***ROUTINES CALLED NONE */
/* ***END PROLOGUE BSDIM3 */

/*   Global variables. */


/*   Local variables. */


    /* Parameter adjustments */
    --rulpts;
    --errcof;
    g -= 4;
    w -= 6;

    /* Function Body */






/* ***FIRST EXECUTABLE STATEMENT BSDIM3 */

/*   Assign values to W. */

    for (i = 1; i <= 13; ++i) {
	for (j = 1; j <= 5; ++j) {
	    w[j + i * 5] = dim3w[i + j * 13 - 14];
/* L10: */
	}
    }

/*   Assign values to G. */

    for (i = 1; i <= 3; ++i) {
	for (j = 1; j <= 13; ++j) {
	    g[i + j * 3] = 0.;
/* L20: */
	}
    }
    g[7] = dim3g[0];
    g[10] = dim3g[1];
    g[13] = dim3g[2];
    g[16] = dim3g[3];
    g[19] = dim3g[4];
    g[22] = dim3g[5];
    g[23] = g[22];
    g[25] = dim3g[6];
    g[26] = g[25];
    g[28] = dim3g[7];
    g[29] = g[28];
    g[30] = g[28];
    g[31] = dim3g[8];
    g[32] = g[31];
    g[33] = g[31];
    g[34] = dim3g[9];
    g[35] = g[34];
    g[36] = g[34];
    g[37] = dim3g[11];
    g[38] = dim3g[10];
    g[39] = g[38];
    g[40] = dim3g[12];
    g[41] = g[40];
    g[42] = dim3g[13];

/*   Assign values to RULPTS. */

    rulpts[1] = 1.;
    rulpts[2] = 6.;
    rulpts[3] = 6.;
    rulpts[4] = 6.;
    rulpts[5] = 6.;
    rulpts[6] = 6.;
    rulpts[7] = 12.;
    rulpts[8] = 12.;
    rulpts[9] = 8.;
    rulpts[10] = 8.;
    rulpts[11] = 8.;
    rulpts[12] = 24.;
    rulpts[13] = 24.;

/*   Assign values to ERRCOF. */

    errcof[1] = 4.;
    errcof[2] = 4.;
    errcof[3] = .5;
    errcof[4] = 3.;
    errcof[5] = .5;
    errcof[6] = .25;

/* ***END BSDIM3 */

    return 0;
} /* bsdim3_ */

/* Subroutine */ int bsrl09_(ndim, wtleng, w, g, errcof, rulpts)
integer *ndim, *wtleng;
pi_base_t *w, *g, *errcof, *rulpts;
{
    /* System generated locals */
    integer g_dim1, g_offset, i__1, i__2;
    pi_base_t d__1, d__2, d__3, d__4;
    double two=2.;

    /* Builtin functions */
    /*integer pow_ii();*/

    /* Local variables */
    static pi_base_t lamp;
    static integer i, j;
    static pi_base_t ratio, twondm, lam0, lam1, lam2, lam3;

/* ***BEGIN PROLOGUE BSRL09 */
/* ***KEYWORDS basic integration rule, degree 9 */
/* ***PURPOSE  To initialize a degree 9 basic rule and null rules. */
/* ***AUTHOR   Alan Genz, Computer Science Department, Washington */
/*            State University, Pullman, WA 99163-1210 USA */
/* ***LAST MODIFICATION 88-05-20 */
/* ***DESCRIPTION  BSRL09 initializes a degree 9 integration rule, */
/*            two degree 7 null rules, one degree 5 null rule and one */
/*            degree 3 null rule for the hypercube [-1,1]**NDIM. */

/*   ON ENTRY */

/*   NDIM   Integer. */
/*          Number of variables. */
/*   WTLENG Integer. */
/*          The number of weights in each of the rules. */

/*   ON RETURN */
/*   W      Real array of dimension (5,WTLENG). */
/*          The weights for the basic and null rules. */
/*          W(1,1),...,W(1,WTLENG) are weights for the basic rule. */
/*          W(I,1),...,W(I,WTLENG), for I > 1 are null rule weights. */
/*   G      Real array of dimension (NDIM, WTLENG). */
/*          The fully symmetric sum generators for the rules. */
/*          G(1, J), ..., G(NDIM, J) are the are the generators for the */

/*          points associated with the Jth weights. */
/*   ERRCOF Real array of dimension 6. */
/*          Heuristic error coefficients that are used in the */
/*          error estimation in BASRUL. */
/*   RULPTS Real array of dimension WTLENG. */
/*          A work array. */

/* ***REFERENCES A. Genz and A. Malik, */
/*             "An Imbedded Family of Fully Symmetric Numerical */
/*              Integration Rules", */
/*              SIAM J Numer. Anal. 20 (1983), pp. 580-588. */
/* ***ROUTINES CALLED NONE */
/* ***END PROLOGUE BSRL09 */

/*   Global variables */


/*   Local Variables */


/* ***FIRST EXECUTABLE STATEMENT BSRL09 */


/*     Initialize generators, weights and RULPTS */

    /* Parameter adjustments */
    --rulpts;
    --errcof;
    g_dim1 = *ndim;
    g_offset = g_dim1 + 1;
    g -= g_offset;
    w -= 6;

    /* Function Body */
    i__1 = *wtleng;
    for (j = 1; j <= i__1; ++j) {
	i__2 = *ndim;
	for (i = 1; i <= i__2; ++i) {
	    g[i + j * g_dim1] = 0.;
/* L10: */
	}
	for (i = 1; i <= 5; ++i) {
	    w[i + j * 5] = 0.;
/* L20: */
	}
	rulpts[j] = (pi_base_t) (*ndim << 1);
/* L30: */
    }
    twondm = (pi_base_t)  pow(two,(double)(*ndim));
    rulpts[*wtleng] = twondm;
    if (*ndim > 2) {
	rulpts[8] = (pi_base_t) ((*ndim << 2) * (*ndim - 1) * (*ndim - 2) / 
		3);
    }
    rulpts[7] = (pi_base_t) ((*ndim << 2) * (*ndim - 1));
    rulpts[6] = (pi_base_t) ((*ndim << 1) * (*ndim - 1));
    rulpts[1] = 1.;

/*     Compute squared generator parameters */

    lam0 = .4707;
    lam1 = 4 / (15 - 5 / lam0);
    ratio = (1 - lam1 / lam0) / 27;
    lam2 = (5 - lam1 * 7 - ratio * 35) / (7 - lam1 * 35 / 3 - ratio * 35 / 
	    lam0);
    ratio = ratio * (1 - lam2 / lam0) / 3;
    lam3 = (7 - (lam2 + lam1) * 9 + lam2 * 63 * lam1 / 5 - ratio * 63) / (9 - 
	    (lam2 + lam1) * 63 / 5 + lam2 * 21 * lam1 - ratio * 63 / lam0);
    lamp = .0625;

/*     Compute degree 9 rule weights */

/* Computing 4th power */
    d__1 = lam0 * 3, d__1 *= d__1;
    w[*wtleng * 5 + 1] = 1 / (d__1 * d__1) / twondm;
    if (*ndim > 2) {
/* Computing 3rd power */
	d__1 = lam1 * 6, d__2 = d__1;
	w[41] = (1 - 1 / (lam0 * 3)) / (d__2 * (d__1 * d__1));
    }
    w[36] = (1 - (lam0 + lam1) * 7 / 5 + lam0 * 7 * lam1 / 3) / (lam1 * 84 * 
	    lam2 * (lam2 - lam0) * (lam2 - lam1));
    w[31] = (1 - (lam0 + lam2) * 7 / 5 + lam0 * 7 * lam2 / 3) / (lam1 * 84 * 
	    lam1 * (lam1 - lam0) * (lam1 - lam2)) - w[36] * lam2 / lam1 - (*
	    ndim - 2 << 1) * w[41];
    w[21] = (1 - ((lam0 + lam1 + lam2) / 7 - (lam0 * lam1 + lam0 * lam2 + 
	    lam1 * lam2) / 5) * 9 - lam0 * 3 * lam1 * lam2) / (lam3 * 18 * (
	    lam3 - lam0) * (lam3 - lam1) * (lam3 - lam2));
    w[16] = (1 - ((lam0 + lam1 + lam3) / 7 - (lam0 * lam1 + lam0 * lam3 + 
	    lam1 * lam3) / 5) * 9 - lam0 * 3 * lam1 * lam3) / (lam2 * 18 * (
	    lam2 - lam0) * (lam2 - lam1) * (lam2 - lam3)) - (*ndim - 1 << 1) *
	     w[36];
    w[11] = (1 - ((lam0 + lam2 + lam3) / 7 - (lam0 * lam2 + lam0 * lam3 + 
	    lam2 * lam3) / 5) * 9 - lam0 * 3 * lam2 * lam3) / (lam1 * 18 * (
	    lam1 - lam0) * (lam1 - lam2) * (lam1 - lam3)) - (*ndim - 1 << 1) *
	     (w[36] + w[31] + (*ndim - 2) * w[41]);

/*     Compute weights for 2 degree 7, 1 degree 5 and 1 degree 3 rules */

/* Computing 4th power */
    d__1 = lam0, d__1 *= d__1;
    w[*wtleng * 5 + 2] = 1 / (d__1 * d__1 * 108) / twondm;
    if (*ndim > 2) {
/* Computing 3rd power */
	d__1 = lam0, d__2 = d__1;
/* Computing 3rd power */
	d__3 = lam1 * 6, d__4 = d__3;
	w[42] = (1 - twondm * 27 * w[47] * (d__2 * (d__1 * d__1))) / (d__4 * (
		d__3 * d__3));
    }
/* Computing 2nd power */
    d__1 = lam0;
    w[37] = (1 - lam1 * 5 / 3 - twondm * 15 * w[*wtleng * 5 + 2] * (d__1 * 
	    d__1) * (lam0 - lam1)) / (lam1 * 60 * lam2 * (lam2 - lam1));
/* Computing 2nd power */
    d__1 = lam0;
    w[32] = (1 - (lam1 * 8 * lam2 * w[37] + twondm * w[*wtleng * 5 + 2] * (
	    d__1 * d__1)) * 9) / (lam1 * 36 * lam1) - w[42] * 2 * (*ndim - 2);

    w[22] = (1 - ((lam1 + lam2) / 5 - lam1 * lam2 / 3 + twondm * w[*wtleng * 
	    5 + 2] * lam0 * (lam0 - lam1) * (lam0 - lam2)) * 7) / (lam3 * 14 *
	     (lam3 - lam1) * (lam3 - lam2));
    w[17] = (1 - ((lam1 + lam3) / 5 - lam1 * lam3 / 3 + twondm * w[*wtleng * 
	    5 + 2] * lam0 * (lam0 - lam1) * (lam0 - lam3)) * 7) / (lam2 * 14 *
	     (lam2 - lam1) * (lam2 - lam3)) - (*ndim - 1 << 1) * w[37];
    w[12] = (1 - ((lam2 + lam3) / 5 - lam2 * lam3 / 3 + twondm * w[*wtleng * 
	    5 + 2] * lam0 * (lam0 - lam2) * (lam0 - lam3)) * 7) / (lam1 * 14 *
	     (lam1 - lam2) * (lam1 - lam3)) - (*ndim - 1 << 1) * (w[37] + w[
	    32] + (*ndim - 2) * w[42]);
/* Computing 4th power */
    d__1 = lam0, d__1 *= d__1;
    w[*wtleng * 5 + 3] = 5 / (d__1 * d__1 * 324) / twondm;
    if (*ndim > 2) {
/* Computing 3rd power */
	d__1 = lam0, d__2 = d__1;
/* Computing 3rd power */
	d__3 = lam1 * 6, d__4 = d__3;
	w[43] = (1 - twondm * 27 * w[48] * (d__2 * (d__1 * d__1))) / (d__4 * (
		d__3 * d__3));
    }
/* Computing 2nd power */
    d__1 = lam0;
    w[38] = (1 - lam1 * 5 / 3 - twondm * 15 * w[*wtleng * 5 + 3] * (d__1 * 
	    d__1) * (lam0 - lam1)) / (lam1 * 60 * lam2 * (lam2 - lam1));
/* Computing 2nd power */
    d__1 = lam0;
    w[33] = (1 - (lam1 * 8 * lam2 * w[38] + twondm * w[*wtleng * 5 + 3] * (
	    d__1 * d__1)) * 9) / (lam1 * 36 * lam1) - w[43] * 2 * (*ndim - 2);

    w[28] = (1 - ((lam1 + lam2) / 5 - lam1 * lam2 / 3 + twondm * w[*wtleng * 
	    5 + 3] * lam0 * (lam0 - lam1) * (lam0 - lam2)) * 7) / (lamp * 14 *
	     (lamp - lam1) * (lamp - lam2));
    w[18] = (1 - ((lam1 + lamp) / 5 - lam1 * lamp / 3 + twondm * w[*wtleng * 
	    5 + 3] * lam0 * (lam0 - lam1) * (lam0 - lamp)) * 7) / (lam2 * 14 *
	     (lam2 - lam1) * (lam2 - lamp)) - (*ndim - 1 << 1) * w[38];
    w[13] = (1 - ((lam2 + lamp) / 5 - lam2 * lamp / 3 + twondm * w[*wtleng * 
	    5 + 3] * lam0 * (lam0 - lam2) * (lam0 - lamp)) * 7) / (lam1 * 14 *
	     (lam1 - lam2) * (lam1 - lamp)) - (*ndim - 1 << 1) * (w[38] + w[
	    33] + (*ndim - 2) * w[43]);
/* Computing 4th power */
    d__1 = lam0, d__1 *= d__1;
    w[*wtleng * 5 + 4] = 2 / (d__1 * d__1 * 81) / twondm;
    if (*ndim > 2) {
/* Computing 3rd power */
	d__1 = lam0, d__2 = d__1;
/* Computing 3rd power */
	d__3 = lam1 * 6, d__4 = d__3;
	w[44] = (2 - twondm * 27 * w[49] * (d__2 * (d__1 * d__1))) / (d__4 * (
		d__3 * d__3));
    }
    w[39] = (2 - lam1 * 15 / 9 - twondm * 15 * w[*wtleng * 5 + 4] * lam0 * (
	    lam0 - lam1)) / (lam1 * 60 * lam2 * (lam2 - lam1));
/* Computing 2nd power */
    d__1 = lam0;
    w[34] = (1 - (lam1 * 8 * lam2 * w[39] + twondm * w[*wtleng * 5 + 4] * (
	    d__1 * d__1)) * 9) / (lam1 * 36 * lam1) - w[44] * 2 * (*ndim - 2);

    w[24] = (2 - ((lam1 + lam2) / 5 - lam1 * lam2 / 3 + twondm * w[*wtleng * 
	    5 + 4] * lam0 * (lam0 - lam1) * (lam0 - lam2)) * 7) / (lam3 * 14 *
	     (lam3 - lam1) * (lam3 - lam2));
    w[19] = (2 - ((lam1 + lam3) / 5 - lam1 * lam3 / 3 + twondm * w[*wtleng * 
	    5 + 4] * lam0 * (lam0 - lam1) * (lam0 - lam3)) * 7) / (lam2 * 14 *
	     (lam2 - lam1) * (lam2 - lam3)) - (*ndim - 1 << 1) * w[39];
    w[14] = (2 - ((lam2 + lam3) / 5 - lam2 * lam3 / 3 + twondm * w[*wtleng * 
	    5 + 4] * lam0 * (lam0 - lam2) * (lam0 - lam3)) * 7) / (lam1 * 14 *
	     (lam1 - lam2) * (lam1 - lam3)) - (*ndim - 1 << 1) * (w[39] + w[
	    34] + (*ndim - 2) * w[44]);
    w[15] = 1 / (lam1 * 6);

/*     Set generator values */

    lam0 = pi_sqrt(lam0);
    lam1 = pi_sqrt(lam1);
    lam2 = pi_sqrt(lam2);
    lam3 = pi_sqrt(lam3);
    lamp = pi_sqrt(lamp);
    i__1 = *ndim;
    for (i = 1; i <= i__1; ++i) {
	g[i + *wtleng * g_dim1] = lam0;
/* L40: */
    }
    if (*ndim > 2) {
	g[(g_dim1 << 3) + 1] = lam1;
	g[(g_dim1 << 3) + 2] = lam1;
	g[(g_dim1 << 3) + 3] = lam1;
    }
    g[g_dim1 * 7 + 1] = lam1;
    g[g_dim1 * 7 + 2] = lam2;
    g[g_dim1 * 6 + 1] = lam1;
    g[g_dim1 * 6 + 2] = lam1;
    g[g_dim1 * 5 + 1] = lamp;
    g[(g_dim1 << 2) + 1] = lam3;
    g[g_dim1 * 3 + 1] = lam2;
    g[(g_dim1 << 1) + 1] = lam1;

/*     Compute final weight values. */
/*     The null rule weights are computed from differences between */
/*     the degree 9 rule weights and lower degree rule weights. */

    w[6] = twondm;
    for (j = 2; j <= 5; ++j) {
	i__1 = *wtleng;
	for (i = 2; i <= i__1; ++i) {
	    w[j + i * 5] -= w[i * 5 + 1];
	    w[j + 5] -= rulpts[i] * w[j + i * 5];
/* L50: */
	}
/* L70: */
    }
    i__1 = *wtleng;
    for (i = 2; i <= i__1; ++i) {
	w[i * 5 + 1] = twondm * w[i * 5 + 1];
	w[6] -= rulpts[i] * w[i * 5 + 1];
/* L80: */
    }

/*     Set error coefficients */

    errcof[1] = 5.;
    errcof[2] = 5.;
    errcof[3] = 1.;
    errcof[4] = 5.;
    errcof[5] = .5;
    errcof[6] = .25;

/* ***END BSRL09 */

} /* bsrl09_ */

/* Subroutine */ int bsrl07_(ndim, wtleng, w, g, errcof, rulpts)
integer *ndim, *wtleng;
pi_base_t *w, *g, *errcof, *rulpts;
{
    /* System generated locals */
    integer g_dim1, g_offset, i__1, i__2;
    pi_base_t d__1, d__2;
    double two=2.;

    /* Builtin functions */
  /*  integer pow_ii();*/

    /* Local variables */
    static pi_base_t lamp;
    static integer i, j;
    static pi_base_t ratio, twondm, lam0, lam1, lam2;

/* ***BEGIN PROLOGUE BSRL07 */
/* ***KEYWORDS basic integration rule, degree 7 */
/* ***PURPOSE  To initialize a degree 7 basic rule, and null rules. */
/* ***AUTHOR   Alan Genz, Computer Science Department, Washington */
/*            State University, Pullman, WA 99163-1210 USA */
/* ***LAST MODIFICATION 88-05-31 */
/* ***DESCRIPTION  BSRL07 initializes a degree 7 integration rule, */
/*            two degree 5 null rules, one degree 3 null rule and one */
/*            degree 1 null rule for the hypercube [-1,1]**NDIM. */

/*   ON ENTRY */

/*   NDIM   Integer. */
/*          Number of variables. */
/*   WTLENG Integer. */
/*          The number of weights in each of the rules. */
/*          WTLENG MUST be set equal to 6. */

/*   ON RETURN */
/*   W      Real array of dimension (5,WTLENG). */
/*          The weights for the basic and null rules. */
/*          W(1,1),...,W(1,WTLENG) are weights for the basic rule. */
/*          W(I,1),...,W(I,WTLENG), for I > 1 are null rule weights. */
/*   G      Real array of dimension (NDIM, WTLENG). */
/*          The fully symmetric sum generators for the rules. */
/*          G(1, J), ..., G(NDIM, J) are the are the generators for the */

/*          points associated with the Jth weights. */
/*   ERRCOF Real array of dimension 6. */
/*          Heuristic error coefficients that are used in the */
/*          error estimation in BASRUL. */
/*   RULPTS Real array of dimension WTLENG. */
/*          A work array. */

/* ***REFERENCES A. Genz and A. Malik, */
/*             "An Imbedded Family of Fully Symmetric Numerical */
/*              Integration Rules", */
/*              SIAM J Numer. Anal. 20 (1983), pp. 580-588. */
/* ***ROUTINES CALLED NONE */
/* ***END PROLOGUE BSRL07 */

/*   Global variables */


/*   Local Variables */


/* ***FIRST EXECUTABLE STATEMENT BSRL07 */


/*     Initialize generators, weights and RULPTS */

    /* Parameter adjustments */
    --rulpts;
    --errcof;
    g_dim1 = *ndim;
    g_offset = g_dim1 + 1;
    g -= g_offset;
    w -= 6;

    /* Function Body */
    i__1 = *wtleng;
    for (j = 1; j <= i__1; ++j) {
	i__2 = *ndim;
	for (i = 1; i <= i__2; ++i) {
	    g[i + j * g_dim1] = 0.;
/* L10: */
	}
	for (i = 1; i <= 5; ++i) {
	    w[i + j * 5] = 0.;
/* L20: */
	}
	rulpts[j] = (pi_base_t) (*ndim << 1);
/* L30: */
    }
    twondm = (pi_base_t)  pow(two,(double)(*ndim));
    rulpts[*wtleng] = twondm;
    rulpts[*wtleng - 1] = (pi_base_t) ((*ndim << 1) * (*ndim - 1));
    rulpts[1] = 1.;

/*     Compute squared generator parameters */

    lam0 = .4707;
    lamp = .5625;
    lam1 = 4 / (15 - 5 / lam0);
    ratio = (1 - lam1 / lam0) / 27;
    lam2 = (5 - lam1 * 7 - ratio * 35) / (7 - lam1 * 35 / 3 - ratio * 35 / 
	    lam0);

/*     Compute degree 7 rule weights */

/* Computing 3rd power */
    d__1 = lam0 * 3, d__2 = d__1;
    w[31] = 1 / (d__2 * (d__1 * d__1)) / twondm;
/* Computing 2nd power */
    d__1 = lam1;
    w[26] = (1 - lam0 * 5 / 3) / ((lam1 - lam0) * 60 * (d__1 * d__1));
    w[16] = (1 - lam2 * 5 / 3 - twondm * 5 * w[31] * lam0 * (lam0 - lam2)) / (
	    lam1 * 10 * (lam1 - lam2)) - (*ndim - 1 << 1) * w[26];
    w[11] = (1 - lam1 * 5 / 3 - twondm * 5 * w[31] * lam0 * (lam0 - lam1)) / (
	    lam2 * 10 * (lam2 - lam1));

/*     Compute weights for 2 degree 5, 1 degree 3 and 1 degree 1 rules */

/* Computing 3rd power */
    d__1 = lam0, d__2 = d__1;
    w[32] = 1 / (d__2 * (d__1 * d__1) * 36) / twondm;
/* Computing 2nd power */
    d__1 = lam0;
/* Computing 2nd power */
    d__2 = lam1;
    w[27] = (1 - twondm * 9 * w[32] * (d__1 * d__1)) / (d__2 * d__2 * 36);
    w[17] = (1 - lam2 * 5 / 3 - twondm * 5 * w[32] * lam0 * (lam0 - lam2)) / (
	    lam1 * 10 * (lam1 - lam2)) - (*ndim - 1 << 1) * w[27];
    w[12] = (1 - lam1 * 5 / 3 - twondm * 5 * w[32] * lam0 * (lam0 - lam1)) / (
	    lam2 * 10 * (lam2 - lam1));
/* Computing 3rd power */
    d__1 = lam0, d__2 = d__1;
    w[33] = 5 / (d__2 * (d__1 * d__1) * 108) / twondm;
/* Computing 2nd power */
    d__1 = lam0;
/* Computing 2nd power */
    d__2 = lam1;
    w[28] = (1 - twondm * 9 * w[33] * (d__1 * d__1)) / (d__2 * d__2 * 36);
    w[18] = (1 - lamp * 5 / 3 - twondm * 5 * w[33] * lam0 * (lam0 - lamp)) / (
	    lam1 * 10 * (lam1 - lamp)) - (*ndim - 1 << 1) * w[28];
    w[23] = (1 - lam1 * 5 / 3 - twondm * 5 * w[33] * lam0 * (lam0 - lam1)) / (
	    lamp * 10 * (lamp - lam1));
/* Computing 3rd power */
    d__1 = lam0, d__2 = d__1;
    w[34] = 1 / (d__2 * (d__1 * d__1) * 54) / twondm;
/* Computing 2nd power */
    d__1 = lam0;
/* Computing 2nd power */
    d__2 = lam1;
    w[29] = (1 - twondm * 18 * w[34] * (d__1 * d__1)) / (d__2 * d__2 * 72);
    w[19] = (1 - lam2 * 10 / 3 - twondm * 10 * w[34] * lam0 * (lam0 - lam2)) /
	     (lam1 * 20 * (lam1 - lam2)) - (*ndim - 1 << 1) * w[29];
    w[14] = (1 - lam1 * 10 / 3 - twondm * 10 * w[34] * lam0 * (lam0 - lam1)) /
	     (lam2 * 20 * (lam2 - lam1));

/*     Set generator values */

    lam0 = pi_sqrt(lam0);
    lam1 = pi_sqrt(lam1);
    lam2 = pi_sqrt(lam2);
    lamp = pi_sqrt(lamp);
    i__1 = *ndim;
    for (i = 1; i <= i__1; ++i) {
	g[i + *wtleng * g_dim1] = lam0;
/* L40: */
    }
    g[(*wtleng - 1) * g_dim1 + 1] = lam1;
    g[(*wtleng - 1) * g_dim1 + 2] = lam1;
    g[(*wtleng - 4) * g_dim1 + 1] = lam2;
    g[(*wtleng - 3) * g_dim1 + 1] = lam1;
    g[(*wtleng - 2) * g_dim1 + 1] = lamp;

/*     Compute final weight values. */
/*     The null rule weights are computed from differences between */
/*     the degree 7 rule weights and lower degree rule weights. */

    w[6] = twondm;
    for (j = 2; j <= 5; ++j) {
	i__1 = *wtleng;
	for (i = 2; i <= i__1; ++i) {
	    w[j + i * 5] -= w[i * 5 + 1];
	    w[j + 5] -= rulpts[i] * w[j + i * 5];
/* L50: */
	}
/* L70: */
    }
    i__1 = *wtleng;
    for (i = 2; i <= i__1; ++i) {
	w[i * 5 + 1] = twondm * w[i * 5 + 1];
	w[6] -= rulpts[i] * w[i * 5 + 1];
/* L80: */
    }

/*     Set error coefficients */

    errcof[1] = 5.;
    errcof[2] = 5.;
    errcof[3] = 1.;
    errcof[4] = 5.;
    errcof[5] = .5;
    errcof[6] = .25;

/* ***END BSRL07 */

} /* bsrl07_ */

/* Subroutine */ int basrul_(ndim, center, hwidth, wtleng, g, w, errcof, 
	numfun, funsub, scales, norms, x, null, basval, rgnerr, direct)
integer *ndim;
pi_base_t *center, *hwidth;
integer *wtleng;
pi_base_t *g, *w, *errcof;
integer *numfun;
/* Subroutine */ int (*funsub) ();
pi_base_t *scales, *norms, *x, *null, *basval, *rgnerr, *direct;
{
    /* System generated locals */
    integer null_dim1, null_offset, g_dim1, g_offset, i__1, i__2;
    pi_base_t d__1, d__2, d__3;

    /* Local variables */
    static integer i, j, k;
    static pi_base_t ratio, search, difmax, frthdf, difsum;
    static integer divaxn;
    static pi_base_t rgnvol;
    extern /* Subroutine */ int fulsum_();

/* ***BEGIN PROLOGUE BASRUL */
/* ***KEYWORDS basic numerical integration rule */
/* ***PURPOSE  To compute basic integration rule values. */
/* ***AUTHOR   Alan Genz, Computer Science Department, Washington */
/*            State University, Pullman, WA 99163-1210 USA */
/* ***LAST MODIFICATION 88-04-08 */
/* ***DESCRIPTION BASRUL computes basic integration rule values for a */
/*            vector of integrands over a hyper-rectangular region. */
/*            These are estimates for the integrals. BASRUL also computes 
*/
/*            estimates for the errors and determines the coordinate axis 
*/
/*            where the fourth difference for the integrands is largest. 
*/

/*   ON ENTRY */

/*   NDIM   Integer. */
/*          Number of variables. */
/*   CENTER Real array of dimension NDIM. */
/*          The coordinates for the center of the region. */
/*   HWIDTH Real Array of dimension NDIM. */
/*          HWIDTH(I) is half of the width of dimension I of the region. 
*/
/*   WTLENG Integer. */
/*          The number of weights in the basic integration rule. */
/*   G      Real array of dimension (NDIM,WTLENG). */
/*          The fully symmetric sum generators for the rules. */
/*          G(1,J), ..., G(NDIM,J) are the are the generators for the */
/*          points associated with the Jth weights. */
/*   W      Real array of dimension (5,WTLENG). */
/*          The weights for the basic and null rules. */
/*          W(1,1),...,W(1,WTLENG) are weights for the basic rule. */
/*          W(I,1),...,W(I,WTLENG), for I > 1 are null rule weights. */
/*   ERRCOF Real array of dimension 6. */
/*          The error coefficients for the rules. */
/*          It is assumed that the error is computed using: */
/*           IF (N1*ERRCOF(1) < N2 and N2*ERRCOF(2) < N3) */
/*             THEN ERROR = ERRCOF(3)*N1 */
/*             ELSE ERROR = ERRCOF(4)*MAX(N1, N2, N3) */
/*           ERROR = ERROR + EP*(ERRCOF(5)*ERROR/(ES+ERROR)+ERRCOF(6)) */
/*          where N1-N4 are the null rules, EP is the error */
/*          for the parent */
/*          subregion and ES is the error for the sibling subregion. */
/*   NUMFUN Integer. */
/*          Number of components for the vector integrand. */
/*   FUNSUB Externally declared subroutine. */
/*          For computing the components of the integrand at a point X. */

/*          It must have parameters (NDIM,X,NUMFUN,FUNVLS). */
/*           Input Parameters: */
/*            X      Real array of dimension NDIM. */
/*                   Defines the evaluation point. */
/*            NDIM   Integer. */
/*                   Number of variables for the integrand. */
/*            NUMFUN Integer. */
/*                   Number of components for the vector integrand. */
/*           Output Parameters: */
/*            FUNVLS Real array of dimension NUMFUN. */
/*                   The components of the integrand at the point X. */
/*   SCALES Real array of dimension (3,WTLENG). */
/*          Scaling factors used to construct new null rules based */
/*          on a linear combination of two successive null rules */
/*          in the sequence of null rules. */
/*   NORMS  Real array of dimension (3,WTLENG). */
/*          2**NDIM/(1-norm of the null rule constructed by each of the */

/*          scaling factors.) */
/*   X      Real Array of dimension NDIM. */
/*          A work array. */
/*   NULL   Real array of dimension (NUMFUN, 8) */
/*          A work array. */

/*   ON RETURN */

/*   BASVAL Real array of dimension NUMFUN. */
/*          The values for the basic rule for each component */
/*          of the integrand. */
/*   RGNERR Real array of dimension NUMFUN. */
/*          The error estimates for each component of the integrand. */
/*   DIRECT Real. */
/*          The coordinate axis where the fourth difference of the */
/*          integrand values is largest. */

/* ***ROUTINES CALLED: FULSUM, FUNSUB */

/* ***END PROLOGUE BASRUL */

/*   Global variables. */


/*   Local variables. */


/* ***FIRST EXECUTABLE STATEMENT BASRUL */


/*       Compute volume of subregion, initialize DIVAXN and rule sums; */
/*       compute fourth differences and new DIVAXN (RGNERR is used */
/*       for a work array here). The integrand values used for the */
/*       fourth divided differences are accumulated in rule arrays. */

    /* Parameter adjustments */
    --rgnerr;
    --basval;
    null_dim1 = *numfun;
    null_offset = null_dim1 + 1;
    null -= null_offset;
    --x;
    norms -= 4;
    scales -= 4;
    --errcof;
    w -= 6;
    g_dim1 = *ndim;
    g_offset = g_dim1 + 1;
    g -= g_offset;
    --hwidth;
    --center;

    /* Function Body */
    rgnvol = 1.;
    divaxn = 1;
    i__1 = *ndim;
    for (i = 1; i <= i__1; ++i) {
	rgnvol *= hwidth[i];
	x[i] = center[i];
	if (hwidth[i] > hwidth[divaxn]) {
	    divaxn = i;
	}
/* L10: */
    }
    (*funsub)(ndim, &x[1], numfun, &rgnerr[1]);
    i__1 = *numfun;
    for (j = 1; j <= i__1; ++j) {
	basval[j] = w[6] * rgnerr[j];
	for (k = 1; k <= 4; ++k) {
	    null[j + k * null_dim1] = w[k + 6] * rgnerr[j];
/* L20: */
	}
/* L30: */
    }
    difmax = 0.;
/* Computing 2nd power */
    d__1 = g[g_dim1 * 3 + 1] / g[(g_dim1 << 1) + 1];
    ratio = d__1 * d__1;
    i__1 = *ndim;
    for (i = 1; i <= i__1; ++i) {
	x[i] = center[i] - hwidth[i] * g[(g_dim1 << 1) + 1];
	(*funsub)(ndim, &x[1], numfun, &null[null_dim1 * 5 + 1]);
	x[i] = center[i] + hwidth[i] * g[(g_dim1 << 1) + 1];
	(*funsub)(ndim, &x[1], numfun, &null[null_dim1 * 6 + 1]);
	x[i] = center[i] - hwidth[i] * g[g_dim1 * 3 + 1];
	(*funsub)(ndim, &x[1], numfun, &null[null_dim1 * 7 + 1]);
	x[i] = center[i] + hwidth[i] * g[g_dim1 * 3 + 1];
	(*funsub)(ndim, &x[1], numfun, &null[(null_dim1 << 3) + 1]);
	x[i] = center[i];
	difsum = 0.;
	i__2 = *numfun;
	for (j = 1; j <= i__2; ++j) {
	    frthdf = (1 - ratio) * 2 * rgnerr[j] - (null[j + null_dim1 * 7] + 
		    null[j + (null_dim1 << 3)]) + ratio * (null[j + null_dim1 
		    * 5] + null[j + null_dim1 * 6]);

/*           Ignore differences below roundoff */

	    if (rgnerr[j] + frthdf / 4 != rgnerr[j]) {
		difsum += absol(frthdf);
	    }
	    for (k = 1; k <= 4; ++k) {
		null[j + k * null_dim1] = null[j + k * null_dim1] + w[k + 11] 
			* (null[j + null_dim1 * 5] + null[j + null_dim1 * 6]) 
			+ w[k + 16] * (null[j + null_dim1 * 7] + null[j + (
			null_dim1 << 3)]);
/* L40: */
	    }
	    basval[j] = basval[j] + w[11] * (null[j + null_dim1 * 5] + null[j 
		    + null_dim1 * 6]) + w[16] * (null[j + null_dim1 * 7] + 
		    null[j + (null_dim1 << 3)]);
/* L50: */
	}
	if (difsum > difmax) {
	    difmax = difsum;
	    divaxn = i;
	}
/* L60: */
    }
    *direct = (pi_base_t)divaxn;

/*    Finish computing the rule values. */

    i__1 = *wtleng;
    for (i = 4; i <= i__1; ++i) {
	fulsum_(ndim, &center[1], &hwidth[1], &x[1], &g[i * g_dim1 + 1], 
		numfun, funsub, &rgnerr[1], &null[null_dim1 * 5 + 1]);
	i__2 = *numfun;
	for (j = 1; j <= i__2; ++j) {
	    basval[j] += w[i * 5 + 1] * rgnerr[j];
	    for (k = 1; k <= 4; ++k) {
		null[j + k * null_dim1] += w[k + 1 + i * 5] * rgnerr[j];
/* L70: */
	    }
/* L80: */
	}
/* L90: */
    }

/*    Compute errors. */

    i__1 = *numfun;
    for (j = 1; j <= i__1; ++j) {

/*    We search for the null rule, in the linear space spanned by two 
*/
/*    successive null rules in our sequence, which gives the greatest 
*/
/*    error estimate among all normalized (1-norm) null rules in this 
*/
/*    space. */

	for (i = 1; i <= 3; ++i) {
	    search = 0.;
	    i__2 = *wtleng;
	    for (k = 1; k <= i__2; ++k) {
/* Computing MAX */
		d__2 = search, d__3 = (d__1 = null[j + (i + 1) * null_dim1] + 
			scales[i + k * 3] * null[j + i * null_dim1], absol(d__1)
			) * norms[i + k * 3];
		search = max(d__2,d__3);
/* L100: */
	    }
	    null[j + i * null_dim1] = search;
/* L110: */
	}
	if (errcof[1] * null[j + null_dim1] <= null[j + (null_dim1 << 1)] && 
		errcof[2] * null[j + (null_dim1 << 1)] <= null[j + null_dim1 *
		 3]) {
	    rgnerr[j] = errcof[3] * null[j + null_dim1];
	} else {
/* Computing MAX */
	    d__1 = null[j + null_dim1], d__2 = null[j + (null_dim1 << 1)], 
		    d__1 = max(d__1,d__2), d__2 = null[j + null_dim1 * 3];
	    rgnerr[j] = errcof[4] * max(d__1,d__2);
	}
	rgnerr[j] = rgnvol * rgnerr[j];
	basval[j] = rgnvol * basval[j];
/* L130: */
    }

/* ***END BASRUL */

} /* basrul_ */

/* Subroutine */ int fulsum_(ndim, center, hwidth, x, g, numfun, funsub, 
	fulsms, funvls)
integer *ndim;
pi_base_t *center, *hwidth, *x, *g;
integer *numfun;
/* Subroutine */ int (*funsub) ();
pi_base_t *fulsms, *funvls;
{
    /* System generated locals */
    integer i__1, i__2;

    /* Local variables */
    static integer i, j, l;
    static pi_base_t gi, gl;
    static integer ixchng, lxchng;

/* ***BEGIN PROLOGUE FULSUM */
/* ***KEYWORDS fully symmetric sum */
/* ***PURPOSE  To compute fully symmetric basic rule sums */
/* ***AUTHOR   Alan Genz, Computer Science Department, Washington */
/*            State University, Pullman, WA 99163-1210 USA */
/* ***LAST MODIFICATION 88-04-08 */
/* ***DESCRIPTION FULSUM computes a fully symmetric sum for a vector */
/*            of integrand values over a hyper-rectangular region. */
/*            The sum is fully symmetric with respect to the center of */
/*            the region and is taken over all sign changes and */
/*            permutations of the generators for the sum. */

/*   ON ENTRY */

/*   NDIM   Integer. */
/*          Number of variables. */
/*   CENTER Real array of dimension NDIM. */
/*          The coordinates for the center of the region. */
/*   HWIDTH Real Array of dimension NDIM. */
/*          HWIDTH(I) is half of the width of dimension I of the region. 
*/
/*   X      Real Array of dimension NDIM. */
/*          A work array. */
/*   G      Real Array of dimension NDIM. */
/*          The generators for the fully symmetric sum. These MUST BE */
/*          non-negative and non-increasing. */
/*   NUMFUN Integer. */
/*          Number of components for the vector integrand. */
/*   FUNSUB Externally declared subroutine. */
/*          For computing the components of the integrand at a point X. */

/*          It must have parameters (NDIM, X, NUMFUN, FUNVLS). */
/*           Input Parameters: */
/*            X      Real array of dimension NDIM. */
/*                   Defines the evaluation point. */
/*            NDIM   Integer. */
/*                   Number of variables for the integrand. */
/*            NUMFUN Integer. */
/*                   Number of components for the vector integrand. */
/*           Output Parameters: */
/*            FUNVLS Real array of dimension NUMFUN. */
/*                   The components of the integrand at the point X. */
/*   ON RETURN */

/*   FULSMS Real array of dimension NUMFUN. */
/*          The values for the fully symmetric sums for each component */
/*          of the integrand. */
/*   FUNVLS Real array of dimension NUMFUN. */
/*          A work array. */

/* ***ROUTINES CALLED: FUNSUB */

/* ***END PROLOGUE FULSUM */

/*   Global variables. */


/*   Local variables. */


/* ***FIRST EXECUTABLE STATEMENT FULSUM */

    /* Parameter adjustments */
    --funvls;
    --fulsms;
    --g;
    --x;
    --hwidth;
    --center;

    /* Function Body */
    i__1 = *numfun;
    for (j = 1; j <= i__1; ++j) {
	fulsms[j] = 0.;
/* L10: */
    }

/*     Compute centrally symmetric sum for permutation of G */

L20:
    i__1 = *ndim;
    for (i = 1; i <= i__1; ++i) {
	x[i] = center[i] + g[i] * hwidth[i];
/* L30: */
    }
L40:
    (*funsub)(ndim, &x[1], numfun, &funvls[1]);
    i__1 = *numfun;
    for (j = 1; j <= i__1; ++j) {
	fulsms[j] += funvls[j];
/* L50: */
    }
    i__1 = *ndim;
    for (i = 1; i <= i__1; ++i) {
	g[i] = -g[i];
	x[i] = center[i] + g[i] * hwidth[i];
	if (g[i] < 0.) {
	    goto L40;
	}
/* L60: */
    }

/*       Find next distinct permuation of G and loop back for next sum. */

/*       Permutations are generated in reverse lexicographic order. */

    i__1 = *ndim;
    for (i = 2; i <= i__1; ++i) {
	if (g[i - 1] > g[i]) {
	    gi = g[i];
	    ixchng = i - 1;
	    i__2 = (i - 1) / 2;
	    for (l = 1; l <= i__2; ++l) {
		gl = g[l];
		g[l] = g[i - l];
		g[i - l] = gl;
		if (gl <= gi) {
		    --ixchng;
		}
		if (g[l] > gi) {
		    lxchng = l;
		}
/* L70: */
	    }
	    if (g[ixchng] <= gi) {
		ixchng = lxchng;
	    }
	    g[i] = g[ixchng];
	    g[ixchng] = gi;
	    goto L20;
	}
/* L80: */
    }

/*     Restore original order to generators */

    i__1 = *ndim / 2;
    for (i = 1; i <= i__1; ++i) {
	gi = g[i];
	g[i] = g[*ndim - i + 1];
	g[*ndim - i + 1] = gi;
/* L90: */
    }

/* ***END FULSUM */

} /* fulsum_ */
