/*        File: math-adapt.c
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: All
   Description: Source for misc. math functions needed by adaptive partitioning algorithm
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/math/math-adapt.c,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:29 $

                $Log: math-adapt.c,v $
                Revision 1.1.1.1  2000/06/06 03:13:29  parint
                Initial checkin


       Version: 01/13/98  rrz  Original (from existing, albeit scattered, code)
		05/12/98  rrz  Code freeze for ParInt1.0
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
*/

/* Note: All code in here currently, and must in the future, properly support pi_base_t! */

#include <pi-defs.h>
#include <parint.h>
#include <math-adapt.h>


extern void clear_results(pi_base_t result[], pi_base_t estabs[], int nfcns)
{
    register int i;
    for (i = 0; i < nfcns; i++)
    {
	result[i] = 0.0;
	estabs[i] = 0.0;
    }
} /* clear_results() */


extern void update_results(const update_t *update_p, int nfcns, pi_base_t result[], 
			   pi_base_t estabs[], int *fcncount)
{
    register int i;
    for (i = 0; i < nfcns; i++)
    {
	result[i] += update_p->res[i];
	estabs[i] += update_p->est[i];
    }
    *fcncount += update_p->cnt;
} /* update_results() */


/* This clears l_done */
extern void clear_update(update_t *update_p, int nfcns)
{
    register int i;
    for (i = 0; i < nfcns; i++)
    {
	update_p->res[i] = 0.0;
	update_p->est[i] = 0.0;
    }
    update_p->cnt = 0;
    update_p->l_done = FALSE;
} /* clear_update() */


/* This does not set l_done */
extern void update_update(update_t *update_p, pi_base_t *estabs, const region_t *rgn_p, 
			  int nfcns, int fcns_per_basrul)
{
    register int i;
    for (i = 0; i < nfcns; ++i)
    {
	update_p->res[i] += rgn_p->rgnval[i];                      
	update_p->est[i] += rgn_p->rgnerr_v[i];
	estabs[i]        += rgn_p->rgnerr_v[i];
    }
    update_p->cnt += fcns_per_basrul;
} /* update_update() */


extern void clear_estabs(pi_base_t estabs[], int nfcns)
{
    register int i;
    for (i = 0; i < nfcns; i++)
	estabs[i] = 0.0;
} /* clear_estabs() */


