/*        File: math-intg.c
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: All versions
   Description: Source for integration rules and related functions
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/math/math-intg.c,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:29 $

                $Log: math-intg.c,v $
                Revision 1.1.1.1  2000/06/06 03:13:29  parint
                Initial checkin


       Version: 01/13/98  rrz  Original  (from existing code)
		05/12/98  rrz  Code freeze for ParInt1.0
		05/11/99  rrz  Add good_intg_rule_parameters() to check for
				valid parameters vs the chosen integration rule
		07/14/99  rrz  Add to work and checking functions for new
				integration rules
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
*/

/* Code in here pertains to the actual integration of a sub-region.  It will be specific to 
 * the kind of region, as well.  Be sure to support pi_base_t properly! */

#include <stdio.h>

#include <pi-defs.h>
#include <math-misc.h>
#include <math-intg.h>

/* This is used to declare the arrays below.  See admint.f for more info.  Do not change
 * this! (It's a Fortran thing ;-) ): */
#define MAX_WTLENG   15

/* Maintained between calls of work_initialize() and work_do_round(): */
static int global_irule = 0;

/* These are maintained between calls of work_initialize() and work_do_round() (for 
 * bsinit() and basrul(): */  
static pi_base_t w[5*MAX_WTLENG], g[PI_MAX_NDIM*MAX_WTLENG], errcof[6];
static pi_base_t rulpts[MAX_WTLENG], scales[3*MAX_WTLENG];
static pi_base_t norms[3*MAX_WTLENG], x[PI_MAX_NDIM], null[PI_MAX_NFCN*8], direct;
static int wtleng;

extern void work_initialize(int ndims, int irule)
{
    /* Store globals: */
    global_irule = irule;

    pi_assert(valid_intg_rule(irule), "Incorrect key value in work_initialize()!");
    if (irule == PI_IRULE_DIM2_DEG13 || irule == PI_IRULE_DIM3_DEG11
	|| irule == PI_IRULE_DEG9_OSCIL || irule == PI_IRULE_DEG7)
    {
	wtleng = fun_wtleng(ndims, irule);
	pi_assert(wtleng < MAX_WTLENG, "wtleng value too large from fun_wtleng()");
	bsinit_(&ndims, &irule, &wtleng, w, g, errcof, rulpts, scales, norms);
    }
}  /* work_initialize() */


extern void work_do_round(int ndims, int nfcns, pi_ifcn_t ifcn_ptr, region_t *region_p)
{
    pi_base_t resabs, resasc;
    pi_assert(valid_intg_rule(global_irule), "Incorrect key value in work_do_round()!");
    switch (global_irule)
    {
    case PI_IRULE_DIM2_DEG13:
    case PI_IRULE_DIM3_DEG11:
    case PI_IRULE_DEG9_OSCIL:
    case PI_IRULE_DEG7:
	basrul_(&ndims, region_p->center, region_p->h_width, &wtleng, g, w, errcof, &nfcns, 
		ifcn_ptr, scales, norms, x, null, region_p->rgnval, region_p->rgnerr_v, &direct);
	region_p->divaxn = (int) direct;
	region_p->rgnerr = infNorm(region_p->rgnerr_v, nfcns);
	break;
    case PI_IRULE_DQK15:
	dqk15_(ifcn_ptr, region_p->center[0], region_p->h_width[0], region_p->rgnval, 
	       region_p->rgnerr_v, &resabs, &resasc);
	pi_assert(region_p->rgnerr_v[0] >= 0.0, "Bad rgnerr_v from dqk15()");
	region_p->divaxn = 1;
	region_p->rgnerr = region_p->rgnerr_v[0];
	break;
    case PI_IRULE_DQK21:
	dqk21_(ifcn_ptr, region_p->center[0], region_p->h_width[0], region_p->rgnval, 
	       region_p->rgnerr_v, &resabs, &resasc);
	pi_assert(region_p->rgnerr_v[0] >= 0.0, "Bad rgnerr_v from dqk21()");
	region_p->divaxn = 1;
	region_p->rgnerr = region_p->rgnerr_v[0];
	break;
    case PI_IRULE_DQK31:
	dqk31_(ifcn_ptr, region_p->center[0], region_p->h_width[0], region_p->rgnval, 
	       region_p->rgnerr_v, &resabs, &resasc);
	pi_assert(region_p->rgnerr_v[0] >= 0.0, "Bad rgnerr_v from dqk31()");
	region_p->divaxn = 1;
	region_p->rgnerr = region_p->rgnerr_v[0];
	break;
    case PI_IRULE_DQK41:
	dqk41_(ifcn_ptr, region_p->center[0], region_p->h_width[0], region_p->rgnval, 
	       region_p->rgnerr_v, &resabs, &resasc);
	pi_assert(region_p->rgnerr_v[0] >= 0.0, "Bad rgnerr_v from dqk41()");
	region_p->divaxn = 1;
	region_p->rgnerr = region_p->rgnerr_v[0];
	break;
    case PI_IRULE_DQK51:
	dqk51_(ifcn_ptr, region_p->center[0], region_p->h_width[0], region_p->rgnval, 
	       region_p->rgnerr_v, &resabs, &resasc);
	pi_assert(region_p->rgnerr_v[0] >= 0.0, "Bad rgnerr_v from dqk51()");
	region_p->divaxn = 1;
	region_p->rgnerr = region_p->rgnerr_v[0];
	break;
    case PI_IRULE_DQK61:
	dqk61_(ifcn_ptr, region_p->center[0], region_p->h_width[0], region_p->rgnval, 
	       region_p->rgnerr_v, &resabs, &resasc);
	pi_assert(region_p->rgnerr_v[0] >= 0.0, "Bad rgnerr_v from dqk61()");
	region_p->divaxn = 1;
	region_p->rgnerr = region_p->rgnerr_v[0];
	break;
    }
} /* work_do_round() */


/* Calculates the number of function evaluations done by each call to basrul_()
 */
extern int neval(int ndim, int irule)
{
    int n1;

    pi_assert(valid_intg_rule(irule), "Incorrect key value in neval()!");

    switch (irule)
    {
    case PI_IRULE_DIM2_DEG13:
	return 65;
	break;
    case PI_IRULE_DIM3_DEG11:
	return 127;
	break;
    case PI_IRULE_DEG9_OSCIL:
	if (ndim == 2) 
	    return 33;
	n1 = ndim - 1;
	return 1 + 8*ndim + 6*ndim*n1 + (4*ndim*n1*(ndim-2))/3 + POW2(ndim);
	break;
    case PI_IRULE_DEG7:
	return 1 + 6*ndim + 2*ndim*(ndim-1) + POW2(ndim);
	break;
    case PI_IRULE_DQK15:
	return 15;
	break;
    case PI_IRULE_DQK21:
	return 21;
	break;
    case PI_IRULE_DQK31:
	return 31;
	break;
    case PI_IRULE_DQK41:
	return 41;
	break;
    case PI_IRULE_DQK51:
	return 51;
	break;
    case PI_IRULE_DQK61:
	return 61;
	break;
    }

    return 0;
}  /* neval() */


/* Compute wtleng as a function of irule and ndim.  (A wtleng is the number of weights in 
 * the integration rule, as used by basrul_()).  Note that only the 4 admint rules use
 * this function.
 */
extern int fun_wtleng(int ndim, int irule)
{
    switch (irule)
    {
    case PI_IRULE_DIM2_DEG13:
	return 14;
	break;
    case PI_IRULE_DIM3_DEG11:
	return 13;
	break;
    case PI_IRULE_DEG9_OSCIL:
	if (ndim == 2) 
	    return 8;
	else
	    return 9;
	break;
    case PI_IRULE_DEG7:
	return 6;
	break;
    default:
	pi_assert(FALSE, "Incorrect key value in fun_wtleng()!");
	break;
    }

    return 0;
} /* fun_wtleng() */


/* This verifies that the chosen integration rule is valid given the other parameters.  A message
 * is printed and FALSE is returned if there is a problem, else, TRUE is returned.  Other 
 * parameters to check will be added as needed (an iparm_p structure was not used as one is not
 * always available when this is called).
 */
extern bool good_intg_rule_parameters(int irule, int ndims, int nfcns)
{
    pi_assert(valid_intg_rule(irule), "Incorrect key value in fun_wtleng()!");

    switch (irule)
    {
    case PI_IRULE_DIM2_DEG13:
	if (ndims != 2)
	{
	    printf("ERROR: Rule #%d can only be used on 2 dimensional integrand functions\n",
		   PI_IRULE_DIM2_DEG13);
	    return FALSE;
	}
	break;
    case PI_IRULE_DIM3_DEG11:
	if (ndims != 3)
	{
	    printf("ERROR: Rule #%d can only be used on 3 dimensional integrand functions\n",
		   PI_IRULE_DIM3_DEG11);
	    return FALSE;
	}
	break;
    case PI_IRULE_DQK15:
    case PI_IRULE_DQK21:
    case PI_IRULE_DQK31:
    case PI_IRULE_DQK41:
    case PI_IRULE_DQK51:
    case PI_IRULE_DQK61:
	if (ndims != 1)
	{
	    printf("ERROR: Rule #%d can only be used on 1 dimensional integrand functions\n", 
		   irule);
	    return FALSE;
	}
	if (nfcns != 1)
	{
	    printf("ERROR: Rule #%d can only be used with # Functions equal to 1\n", irule);
	    return FALSE;
	}
	break;
    }

    return TRUE;

} /* good_intg_rule_parameters() */
