/*        File: math-misc.c
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: All versions
   Description: Source for small, generic math-related functions
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/math/math-misc.c,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:29 $

                $Log: math-misc.c,v $
                Revision 1.1.1.1  2000/06/06 03:13:29  parint
                Initial checkin


       Version: 01/13/98  rrz  Original (from existing code)
		05/12/98  rrz  Code freeze for ParInt1.0
		07/11/99  rrz  Add 'const' specifier to infNorm()
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
*/

#include <string.h>
#include <parint.h>
#include <math-misc.h>

/* Note: All functions in this file must work regardless of the actual value of pi_base_t */

/* Returns the maximum of x and y
 */
extern pi_base_t max(pi_base_t x, pi_base_t y)
{
    if (x >= y)
	return x;
    else
	return y;
} /* max() */


/* Returns the absolute value of x
 */
extern pi_base_t absol(pi_base_t x)
{
    if (x < 0) 
	return -x; 
    else
	return x;
} /* absol() */


/* Returns max of { |v[i]| : 0 <= i < size } 
 */
extern pi_base_t infNorm(const pi_base_t v[], int size)
{
    pi_base_t max_val, absol_vk;  
    register int k;

    max_val = absol(v[0]);
    for (k = 1; k < size; k++)
    {
	absol_vk = absol(v[k]);
	if (absol_vk > max_val) 
	    max_val = absol_vk;
    }

    return max_val;
} /* infNorm() */



/* Returns the floor of the log-based-2 of x.
 */
extern int flog2(int x)
{
    if (x == 0)
    {
	return 0;
    }
    else
    {
	unsigned int temp = (unsigned) x;
	int result = 0;
	
	while (temp != 1)
	{
	    temp = temp >> 1;
	    ++result;
	}
	
	return result;
    }
} /* flog2() */



/* Returns the ceiling of the log base 2 of x.  Code from Peter Pacheco's
 * book _Parallel Programming with MPI_ (pg. 67)
 */
extern int clog2(int x)
{
    if (x == 0)
    {
	return 0;
    }
    else
    {
	unsigned int temp = (unsigned) x - 1;
	int result = 0;
    
	while (temp != 0)
	{
	    temp = temp >> 1;
	    ++result;
	}
	
	return result;
    }
} /* clog2() */


/* The same as strncpy(), but the dest is guarranteed to be '\0'-terminated.  The value of n
 * is the maximum non-null-terminated length, so dest occupies a total of n+1 bytes.
 */
extern char *strncpy0(char *dest, const char *src, int n)
{
    strncpy(dest, src, n);
    dest[n] = '\0';        /* (is really n+1 - 1; indexing conversion) */
    return dest;
} /* strncpy0() */
