/*        File: cmd-line-args.c
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: mpi, seq
   Description: Code for command line arguments and printing parameters
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/misc/cmd-line-args.c,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:31 $

                $Log: cmd-line-args.c,v $
                Revision 1.1.1.1  2000/06/06 03:13:31  parint
                Initial checkin


       Version: 05/01/99  rrz  Original
		05/11/99  rrz  Add call to good_intg_rule_parameter() (to check
				chosen rule against other parameters)
		05/12/99  rrz  BUG FIX: Given an error if user specifies -onr
				with too large a value (doh!)
		05/12/99  rrz  Organize the -o options and add to the -h
				output
		05/13/99  rrz  Add -ed as a command line argument
		05/25/99  rrz  Add support for -plfile option
		07/14/99  rrz  Remove -l; add -lr and -lf; modify error checks
				for intg. rules
		07/14/99  rrz  Remove killheap/keepheap option
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
		02/18/00  rrz  Add parameter for max heap size
		03/25/00  rrz  Move rparm_load_defaults() away to version
		                specific dir
		06/01/00  rrz  Eliminate several compiler warnings			
*/

/* Note: This will read in the command line arguments and convert them into rparm_t and iparm_t
structures.  This is a simplification, for later, the reading in of the rparms will be separated
(in the code) from the iparms, for the rparms are version specific, but the iparms are not.  There
is also code here for printing the iparms (but parint.c or sparint.c must print out the rparms).
*/

#include <stdio.h>
#include <pi-vspec.h>
#include <pi-version.h>
#include <pi-defs.h>
#include <parint.h>
#include <pi-ifcn.h>
#include <cmd-line-args.h>
#include <math-misc.h>     /* for strncpy0() */
#include <math-intg.h>     /* for valid_intg_rule() & good_intg_rule_parameters() */

#define OPT_STR_SIMPRGNS     "simprgns"
#define OPT_STR_NO_SIMPRGNS  "no-simprgns"
#define OPT_STR_LB           "lb"
#define OPT_STR_NO_LB        "no-lb"
#define OPT_STR_CAW          "caw"
#define OPT_STR_NO_CAW       "no-caw"

/* Provided by [s]parint.c when we are testing parameters: */
extern void print_rparm(pi_rparm_t *rparm_p);

/* For reading in long-double command line arguments: */
#ifdef USING_LONG_DOUBLES

#define pi_atof(s)   atold(s)
static long double atold(const char *arg)
{
    long double val;
    sscanf(arg, "%Lg", &val);
    return val;
} /* atold() */

#else

#define pi_atof(s)   atof(s)

#endif


extern void print_short_usage(void)
{
   printf("parint [-f fcn-name | -s script-file | -h | -i[1|2] ]\n"
	  "       [-r rule] [-ea eps-a] [-er eps-r] [-ed eps-d]\n"
	  "       [-lf fcn_count_limit] [-lr rgn_count_limit]\n"
	  "       [-rgn a1 a2 ... an b1 b2 ... bn] [-ohs heap_size]\n" 
	  "       [-o optval] [-ohr help_ratio] [-ons ns] [-onr nruns]\n"
          "       [-v] [-vp vis-port] [-plfile file-name]\n");
} /* print_short_usage() */


extern void print_long_usage(void)
{
    print_short_usage();
    printf("OPTIONS:\n");
    printf("   -f fcn-name        : Integration function name\n");
    printf("   -s script-file     : Script file to execute\n");
    printf("   -h                 : Prints this help\n");
    printf("   -i[1|2]            : Prints list of available functions\n");
    printf("   -r rule            : Integration rule to use\n");
    printf("   -ea eps-a          : Absolute error tolerance\n");
    printf("   -er eps-r          : Relative error tolerance\n");
    printf("   -ed eps-d          : Relative error tolerance in # of digits\n");
    printf("   -lf fcn_count_limit : Max # of function evaluations\n");
    printf("   -lr rgn_count_limit : Max # of regions evaluations\n");
    printf("   -rgn ....          : Boundaries of hyper-rectangular region\n");
    printf("   -o optval          : Sets flag options:\n");
    printf("       [%s|%s|%s|%s|%s|%s]\n",
	   OPT_STR_SIMPRGNS, OPT_STR_NO_SIMPRGNS, OPT_STR_LB, OPT_STR_NO_LB,
	   OPT_STR_CAW, OPT_STR_NO_CAW);
    printf("   -ohr help_ratio    : Sets ratio at which help is refused\n");
    printf("   -ons ns            : Adjusts granularity of integral workers\n");
    printf("   -onr nr            : Sets number of identical runs to perform\n");
    printf("   -ohs hs            : Sets maximum size of local heap\n");
    printf("   -v                 : Turns on user/gui visualization\n");
    printf("   -vp vis-port       : Sets port # for user/gui communication\n");
    printf("   -plfile file-name  : Sets output file for logging\n");
} /* print_long_usage() */


extern void print_iparm(pi_iparm_t *iparm_p)
{
    char *ifcn_desc;
    int i;

    printf("INTG PARMS: %s: %s\n",
	   iparm_p->ifcn_name, 
	   ((ifcn_desc = pi_ifcn_lookup_desc(iparm_p->ifcn_name))
	    ? ifcn_desc
	    : "Invalid Ifcn Name"));
    printf("            #Dims: %d; #Fcns: %d; Intg Rule: %d\n", 
	   iparm_p->ndims, iparm_p->nfcns, iparm_p->intg_rule);
    printf("            Fcn Eval Limit: %d (== Rgn Eval Limit %d)\n", 
	   iparm_p->fcn_eval_limit, 
	   RUID(iparm_p->fcn_eval_limit, neval(iparm_p->ndims, iparm_p->intg_rule)));
#ifdef USING_LONG_DOUBLES
    printf("            eps-a: %.33LG; eps-r: %.33LG\n", iparm_p->eps_a, iparm_p->eps_r);
#else
    printf("            eps-a: %.15G; eps-r: %.15G\n", iparm_p->eps_a, iparm_p->eps_r);
#endif    
    printf("       A[]: ");

#ifdef USING_LONG_DOUBLES
    for (i = 0; i < iparm_p->ndims; i++)
    {
	if (!(i % 2) && i > 0)
	    printf("\n            ");
	printf("%38.33LG ", iparm_p->a[i]);
    }
    printf("\n       B[]: ");
    for (i = 0; i < iparm_p->ndims; i++)
    {
	if (!(i % 2) && i > 0)
	    printf("\n            ");
	printf("%38.33LG ", iparm_p->b[i]);
    }

#else

    for (i = 0; i < iparm_p->ndims; i++)
    {
	if (!(i % 3) && i)
	    printf("\n            ");
	printf("%21.15G ", iparm_p->a[i]);
    }
    printf("\n       B[]: ");
    for (i = 0; i < iparm_p->ndims; i++)
    {
	if (!(i % 3) && i)
	    printf("\n            ");
	printf("%21.15G ", iparm_p->b[i]);
    }
#endif

    printf("\n");

} /* print_iparm() */



/* This sets the char ptr arg_str to point to the desired string argument.  This argument
 * is either at the end of the arg_num^th argument (beginning at character spos), or,
 * forms the entire next argument (in which case arg_num is incremented).  If there is an 
 * error, 1 is returned (and a message is printed), else 0 is returned.  arg_num is 0-indexed,
 * and (as is normal) argc is 1-indexed.
 */
static int get_arg_string(int *arg_num, int argc, char *argv[], int spos, char **arg_str)
{
    if (argv[*arg_num][spos] == '\0')
    {
	if (*arg_num == argc - 1)
	{
	    printf("ERROR: No specifier for argument '%s'\n", argv[argc-1]);
	    return 1;
	}
	else
	{
	    ++(*arg_num);
	    *arg_str = argv[*arg_num];
	    return 0;
	}
    }
    else
    {
	*arg_str = &argv[*arg_num][spos];
	return 0;
    }
} /* get_arg_string() */


/* This reads the current argument when it is a -o or -oxx argument.  I separated
 * it out from get_cmd_line_args() because I expect this function to change
 * between versions.
 */
static int pi_get_o_arg(int *arg, int argc, char *argv[], pi_rparm_t *rparm_p, int nPE)
{
    char *arg_p = argv[*arg];
    char *arg_str;

    pi_assert(STRNCMP(arg_p, ==, "-o", 2), "pi_get_o_arg() incorrectly called");
    
    if (STRNCMP(arg_p, ==, "-ohr", 4))
    {
	if (get_arg_string(arg, argc, argv, 4, &arg_str))
	{
	    return 1;
	}
	else if ((rparm_p->lb_help_ratio = atof(arg_str)) < 1.0)
	{
	    printf("ERROR: help_ratio must be at least 1.0\n");
	    return 1;
	}
    }
    else if (STRNCMP(arg_p, ==, "-ons", 4))
    {
	if (get_arg_string(arg, argc, argv, 4, &arg_str))
	{
	    return 1;
	}
	else if ((rparm_p->ns = atoi(arg_str)) < 1)
	{
	    printf("ERROR: ns value must be at least 1\n");
	    return 1;
	}
    }
    else if (STRNCMP(arg_p, ==, "-ohs", 4))
    {
	if (get_arg_string(arg, argc, argv, 4, &arg_str))
	{
	    return 1;
	}
	else if ((rparm_p->max_heap_size = atoi(arg_str)) < 1)
	{
	    printf("ERROR: hs value must be at least 0\n");
	    return 1;
	}
    }
    else if (STRNCMP(arg_p, ==, "-onr", 4))
    {
	if (get_arg_string(arg, argc, argv, 4, &arg_str))
	{
	    return 1;
	}
	else if ((rparm_p->nruns = atoi(arg_str)) < 1)
	{
	    printf("ERROR: nruns must be at least 1\n");
	    return 1;
	}
	else if (rparm_p->nruns > PI_MAX_NRUNS)
	{
	    printf("ERROR: nruns must be no more than %d\n", PI_MAX_NRUNS);
	    return 1;
	}
    }
    else /* begin the -o binary options */
    {
	if (get_arg_string(arg, argc, argv, 2, &arg_str))
	{
	    return 1;
	}
	else if (STRCMP(arg_str, ==, OPT_STR_SIMPRGNS))
	{
	    rparm_p->simple_rgns = TRUE;
	}
	else if (STRCMP(arg_str, ==, OPT_STR_NO_SIMPRGNS))
	{
	    rparm_p->simple_rgns = FALSE;
	}
	else if (STRCMP(arg_str, ==, OPT_STR_NO_LB))
	{
	    rparm_p->do_lb = FALSE;
	}
	else if (STRCMP(arg_str, ==, OPT_STR_LB))
	{
	    rparm_p->do_lb = TRUE;
	}
	else if (STRCMP(arg_str, ==, OPT_STR_CAW))
	{
	    rparm_p->ic_is_worker= TRUE;
	}
	else if (STRCMP(arg_str, ==, OPT_STR_NO_CAW))
	{
	    if (nPE == 1)
	    {
		printf("ERROR: IC must be a worker if # of processes is one\n");
		return 1;
	    }
	    else
	    {
		rparm_p->ic_is_worker= FALSE;
	    }
	}
	else
	{
	    printf("ERROR: Invalid specifier of '%s' to -o parameter\n", arg_str);
	    return 1;
	}
    }

    return 0;

} /* pi_rparm_t() */


static int get_pl_arg(int *arg, int argc, char *argv[], pi_rparm_t *rparm_p)
{
    char *arg_p = argv[*arg];
    char *arg_str;

    pi_assert(STRNCMP(arg_p, ==, "-pl", 3), "get_pl_arg() incorrectly called");
    
    if (STRNCMP(arg_p, ==, "-plfile", 7))
    {
	if (get_arg_string(arg, argc, argv, 7, &arg_str))
	    return 1;
	else
	    strncpy0(rparm_p->pl_file, arg_str, PI_MAX_PATH);
    }
    else
    {
	printf("ERROR: Invalid parameter of '%s'\n", arg_p);
	return 1;
    }

    return 0;
} /* get_pl_arg() */


static int get_rgn_arguments(int *arg_num, int argc, char *argv[], int ndims, pi_iparm_t *iparm_p)
{
    bool doing_a_values = TRUE;
    bool first_arg = TRUE;
    bool done = FALSE;
    int cur_dim = 0;

    while (!done)
    {
	char *arg_p;
	
	if (first_arg)
	{
	    /* The initial argument is the -rgn argument.  It may have the first value stuck on
	       the end of the "-rgn" string.  Handle this: */
	    if (strlen(argv[*arg_num]) > 4)
	    {
		arg_p = &argv[*arg_num][4];
	    }
	    else
	    {
		++(*arg_num);
		if (*arg_num == argc)
		{
		    /* This error hit when the "-rgn" appears as the last argument: */
		    printf("ERROR: Hit end of arguments while scanning for region values\n");
		    return 1;
		}
		arg_p = argv[*arg_num];
	    }
	    first_arg = FALSE;
	}
	else
	{
	    ++(*arg_num);
	    if (*arg_num == argc)
	    {
		printf("ERROR: Hit end of arguments while scanning for region values\n");
		return 1;
	    }
	    arg_p = argv[*arg_num];
	}

	/* (the arg_p value is now set for this iteration) */
	
	if (doing_a_values)
	    iparm_p->a[cur_dim] = pi_atof(arg_p);
	else
	    iparm_p->b[cur_dim] = pi_atof(arg_p);
	
	++cur_dim;
	if (cur_dim == ndims)
	{
	    if (doing_a_values)
	    {
		doing_a_values = FALSE;
		cur_dim = 0;
	    }
	    else
	    {
		done = TRUE;
	    }
	}
    }

    return 0;

} /* get_rgn_arguments() */


extern int get_cmd_line_args(int argc, char *argv[], int *run_option, pi_rparm_t *rparm_p,
			     pi_iparm_t *iparm_p, char script_file[], int nPE)
{
    
    /* These flags tell me whether the user has set certain integral parameters
       or not.  At the end, if the user has not set them, then I'll load the default
       iparms for the ifcn: */
    bool have_rgn = FALSE;
    bool have_eps_a = FALSE;
    bool have_eps_r = FALSE;
    bool have_fcn_eval_limit = FALSE;
    bool have_rgn_eval_limit = FALSE;
    bool have_intg_rule = FALSE;

    /* These flags tell me if I've received the -s or -f option; they help me to make sure
       that I get only one of these options, and then only exactly once: */
    bool have_script_file = FALSE;
    bool have_ifcn_name = FALSE;

    pi_iparm_t *def_iparms_p = NULL;
    int rgn_eval_limit = 0;
    int arg = 1;
    pi_rparm_t cur_rparm;
    pi_iparm_t cur_iparm;

    bool bad_parms = FALSE;
    
    /* The default run-parms can be loaded from the compile-time defaults: */
    rparm_load_defaults(&cur_rparm);

    /* If the first argument is -h or -i, then we won't enter the loop at all: */
    if (STRNCMP(argv[1], ==, "-h", 2))
    {
	*run_option = RUN_DO_HELP;
	return 0;
    }
    else if (STRNCMP(argv[1], ==, "-i", 2))
    {
	if (strlen(argv[1]) > 2)
	{
	    if (argv[1][2] == '1')
	    {
		*run_option = RUN_DO_INFO_1;
		return 0;
	    }
	    else if (argv[1][2] == '2')
	    {
		*run_option = RUN_DO_INFO_2;
		return 0;
	    }
	    else
	    {
		printf("ERROR: Invalid -i argument of '%s' (must be -i[1|2])\n", argv[1]);
		print_short_usage();
		return 1;
	    }
	}
	else
	{
	    *run_option = RUN_DO_INFO_1;
	    return 0;
	}
    }
    
    /* -- big implicit else -- */

    while (arg < argc && !bad_parms)
    {
	char *arg_p = argv[arg];
	char *arg_str;

	/* Since so many things can go wrong, the default value for bad_parms is TRUE;
	   if this iteration results in a good paramter, then I'll clear this before
	   the next iteration: */
	bad_parms = TRUE;
	
	/* I tried a switch() statement, but it was even messier.  So here is the world's
	   largest if statement: */
	if (STRNCMP(arg_p, ==, "-f", 2))
	{
	    if (have_ifcn_name)
	    {
		printf("ERROR: -f option must appear at most once\n");
	    }
	    else if (have_script_file)
	    {
		printf("ERROR: At most one of the -s or -f options may be used\n");
	    }
	    else if (get_arg_string(&arg, argc, argv, 2, &arg_str))
	    {
		do_nothing();
	    }
	    else if (!(def_iparms_p = pi_ifcn_lookup_iparms(arg_str)))
	    {
		printf("ERROR: Invalid fcn name of '%s'\n", arg_str);
	    }
	    else 
	    {
		have_ifcn_name = TRUE;
		*run_option = RUN_DO_INTEGRAL;
		bad_parms = FALSE;
	    }
	}
	else if (STRNCMP(arg_p, ==, "-s", 2))
	{
	    if (have_script_file)
	    {
		printf("ERROR: -s option must appear at most once\n");
	    }
	    else if (have_ifcn_name)
	    {
		printf("ERROR: At most one of the -s or -f options may be used\n");
	    }
	    else if (get_arg_string(&arg, argc, argv, 2, &arg_str))
	    {
		do_nothing();
	    }
	    else if (have_rgn || have_eps_a || have_eps_r || have_fcn_eval_limit || have_intg_rule)
	    {
		printf("ERROR: No integral-specific option may be specified when using -s\n");
	    }
	    else
	    {
		strncpy0(script_file, arg_str, PI_MAX_PATH);
		have_script_file = TRUE;
		*run_option = RUN_DO_SCRIPT;
		bad_parms = FALSE;
	    }
	}
	else if (STRNCMP(arg_p, ==, "-h", 2))
	{
	    printf("ERROR: The -h option must appear as the first and only parameter\n");
	}
	else if (STRNCMP(arg_p, ==, "-i", 2))
	{
	    printf("ERROR: The -i[1|2] option must appear as the first and only parameter\n");
	}
	else if (STRNCMP(arg_p, ==, "-lf", 3))
	{
	    if (have_script_file)
	    {
		printf("ERROR: No integral-specific option may be specified when using -s\n");
	    }
	    else if (get_arg_string(&arg, argc, argv, 3, &arg_str))
	    {
		do_nothing();
	    }
	    else if ((cur_iparm.fcn_eval_limit = atoi(arg_str)) <= 0)
	    {
		printf("ERROR: fcn_eval_limit must be positive.\n");
	    }
	    else
	    {
		have_fcn_eval_limit = TRUE;
		have_rgn_eval_limit = FALSE;
		bad_parms = FALSE;
	    }
	}
	else if (STRNCMP(arg_p, ==, "-lr", 3))
	{
	    if (have_script_file)
	    {
		printf("ERROR: No integral-specific option may be specified when using -s\n");
	    }
	    else if (get_arg_string(&arg, argc, argv, 3, &arg_str))
	    {
		do_nothing();
	    }
	    else if ((rgn_eval_limit = atoi(arg_str)) <= 0)
	    {
		printf("ERROR: rgn_eval_limit must be positive.\n");
	    }
	    else
	    {
		have_rgn_eval_limit = TRUE;
		have_fcn_eval_limit = FALSE;
		bad_parms = FALSE;
	    }
	}
	else if (STRNCMP(arg_p, ==, "-rgn", 4))
	{
	    if (have_script_file)
	    {
		printf("ERROR: No integral-specific option may be specified when using -s\n");
	    }
	    else if (!have_ifcn_name)
	    {
		printf("ERROR: Function name (-f) must be specified before the region (-rgn)\n");
	    }
	    else if (!(get_rgn_arguments(&arg, argc, argv, def_iparms_p->ndims, &cur_iparm)))
	    {
		bad_parms = FALSE;
		have_rgn = TRUE;
	    }
	}
	else if (STRNCMP(arg_p, ==, "-r", 2))
	{
	    if (have_script_file)
	    {
		printf("ERROR: No integral-specific option may be specified when using -s\n");
	    }
	    else if (get_arg_string(&arg, argc, argv, 2, &arg_str))
	    {
		do_nothing();
	    }
	    else if (!valid_intg_rule(cur_iparm.intg_rule = atoi(arg_str)))
	    {
		printf("ERROR: Invalid integration rule of %d\n", cur_iparm.intg_rule);
	    }
	    else
	    {
		have_intg_rule = TRUE;
		bad_parms = FALSE;
	    }
	}
	else if (STRNCMP(arg_p, ==, "-ea", 3))
	{
	    if (have_script_file)
	    {
		printf("ERROR: No integral-specific option may be specified when using -s\n");
	    }
	    else if (get_arg_string(&arg, argc, argv, 3, &arg_str))
	    {
		do_nothing();
	    }
	    else if ((cur_iparm.eps_a = pi_atof(arg_str)) < 0.0)
	    {
		printf("ERROR: eps-a value must be at least 0.0\n");
	    }
	    else
	    {
		have_eps_a = TRUE;
		bad_parms = FALSE;
	    }
	}
	else if (STRNCMP(arg_p, ==, "-er", 3))
	{
	    if (have_script_file)
	    {
		printf("ERROR: No integral-specific option may be specified when using -s\n");
	    }
	    else if (get_arg_string(&arg, argc, argv, 3, &arg_str))
	    {
		do_nothing();
	    }
	    else if ((cur_iparm.eps_r = pi_atof(arg_str)) < 0.0)
	    {
		printf("ERROR: eps-r value must be at least 0.0\n");
	    }
	    else
	    {
		have_eps_r = TRUE;
		bad_parms = FALSE;
	    }
	}
	else if (STRNCMP(arg_p, ==, "-ed", 3))
	{
	    int eps_d;
	    if (have_script_file)
	    {
		printf("ERROR: No integral-specific option may be specified when using -s\n");
	    }
	    else if (get_arg_string(&arg, argc, argv, 3, &arg_str))
	    {
		do_nothing();
	    }
	    else if ((eps_d = atoi(arg_str)) < 0)
	    {
		printf("ERROR: eps_d value must be at least 0\n");
	    }
	    else
	    {
		cur_iparm.eps_r = pi_pow(10.0, -eps_d);
		have_eps_r = TRUE;
		bad_parms = FALSE;
	    }
	}
	else if (STRNCMP(arg_p, ==, "-o", 2))
	{
	    if (!pi_get_o_arg(&arg, argc, argv, &cur_rparm, nPE))
		bad_parms = FALSE;
	}
	else if (STRNCMP(arg_p, ==, "-vp", 3))
	{
	    if (get_arg_string(&arg, argc, argv, 3, &arg_str))
	    {
		do_nothing();
	    }
	    else if ((cur_rparm.vis_port = atoi(arg_str)) < 1)
	    {
		printf("ERROR: vis-port must be at least 1\n");
	    }
	    else
	    {
		bad_parms = FALSE;
	    }
	}
	else if (STRNCMP(arg_p, ==, "-v", 2))
	{
	    cur_rparm.vis_type = VIS_ON;
	    bad_parms = FALSE;
	}
	else if (STRNCMP(arg_p, ==, "-pl", 3))
	{
	    if (!get_pl_arg(&arg, argc, argv, &cur_rparm))
		bad_parms = FALSE;
	}
	else
	{
	    printf("ERROR: Invalid parameter of '%s'\n", arg_p);
	}

	++arg;
    }

    if (bad_parms)
    {
	print_short_usage();
	return 1;
    }
    if (!have_ifcn_name && !have_script_file)
    {
	printf("ERROR: One of -h, -i, -f, or -s must be specified\n");
	print_short_usage();
	return 1;
    }

    /* If we received a -h or -i, we already exited.  Or, if we did not receive a function
       name we exited.  Now, after scanning through all the args, we want to store the actual
       values in rparm_p and iparm_p: */
    
    /* This stuff can also only be checked after all the args have been read: */
#ifndef USE_LOGGING
    if (strlen(cur_rparm.pl_file))
    {
	printf("ERROR: The USE_LOGGING token must be enabled before using logging\n");
	return 1;
    }
#endif

    if (strlen(cur_rparm.pl_file) && cur_rparm.vis_type == VIS_ON)
    {
	printf("ERROR: Both logging and visualization cannot be on\n");
	return 1;
    }
    if (cur_rparm.vis_type == VIS_ON && cur_rparm.vis_port == 0)
    {
	printf("ERROR: Visualization is on, but vis-port was not specified\n");
	return 1;
    }
    if (cur_rparm.vis_type == VIS_OFF && cur_rparm.vis_port != 0)
    {
	printf("ERROR: Vis-port was specified, but visualization is off\n");
	return 1;
    }

    /* Load up iparm values that are based on the ifcn and cannot be changed by user: */
    strncpy0(iparm_p->ifcn_name, def_iparms_p->ifcn_name, PI_MAX_IFCN_NAME);
    iparm_p->ndims = def_iparms_p->ndims;
    iparm_p->nfcns = def_iparms_p->nfcns;

    /* Set the integration rule: If we do not have an intg rule, then we will use the default
       rule, and it must be a good rule for the given function.  The fn good_rule_pararameters()
       *will* print out an error message for us: */
    if (have_intg_rule)
    {
	if (!good_intg_rule_parameters(cur_iparm.intg_rule, iparm_p->ndims, iparm_p->nfcns))
	    return 1;
	iparm_p->intg_rule = cur_iparm.intg_rule;
    }
    else
    {
	iparm_p->intg_rule = def_iparms_p->intg_rule;
    }
	
    /* Must do this after getting a good rule.  Currently, one or the other of have_rgn_eval_limit
       or have_fcn_eval_limit is set, or neither.  If we got from the user the rgn_eval_limit,
       then turn this into a fcn_eval_limit. If neither is set, then default to the default
       fcn eval limit for this rule: */
    iparm_p->fcn_eval_limit = (have_rgn_eval_limit
			       ? (neval(iparm_p->ndims, iparm_p->intg_rule) * rgn_eval_limit)
			       : (have_fcn_eval_limit
				  ? cur_iparm.fcn_eval_limit
				  : def_iparms_p->fcn_eval_limit));
    have_fcn_eval_limit = TRUE;

    /* Load up rparms (whether we have a script file or an ifcn name): */
    *rparm_p = cur_rparm;
    
    /* Now check these values: */
    if (iparm_p->ndims < 1 || iparm_p->ndims > PI_MAX_NDIM)
    {
	printf("ERROR: Function '%s' in function library has invalid number of dimensions\n", 
	       iparm_p->ifcn_name);
	return 1;
    }
    if (iparm_p->nfcns < 1 || iparm_p->nfcns > PI_MAX_NFCN)
    {
	printf("ERROR: Function '%s' in function library has invalid number of functions\n", 
	       iparm_p->ifcn_name);
	return 1;
    }
    
    /* Load up iparms with the other parameters that can be changed.  Use what the user gave
       us, or, the default: */
    iparm_p->eps_a = (have_eps_a ? cur_iparm.eps_a : def_iparms_p->eps_a);
    iparm_p->eps_r = (have_eps_r ? cur_iparm.eps_r : def_iparms_p->eps_r);
    if (have_rgn)
    {
	memcpy(iparm_p->a, cur_iparm.a, sizeof(pi_base_t) * PI_MAX_NDIM);
	memcpy(iparm_p->b, cur_iparm.b, sizeof(pi_base_t) * PI_MAX_NDIM);
    }
    else
    {
	memcpy(iparm_p->a, def_iparms_p->a, sizeof(pi_base_t) * PI_MAX_NDIM);
	memcpy(iparm_p->b, def_iparms_p->b, sizeof(pi_base_t) * PI_MAX_NDIM);
    }
    /* More checks that must come after we have all the data: */
    {
	int i;
	for (i = 0; i < iparm_p->ndims; ++i)
	{
	    if (iparm_p->a[i] > iparm_p->b[i])
	    {
		printf("ERROR: Region boundary a[%d] is greater than b[%d]\n", i, i);
		return 1;
	    }
	}
    }
    if (iparm_p->eps_a == 0.0 && iparm_p->eps_r < pi_pow(10.0, -PI_DIG))
    {
	printf("ERROR: If eps-a is 0.0, then eps-r must be at least %G\n", pow(10.0, -PI_DIG));
	return 1;
    }
    
    return 0;
} /* get_cmd_line_args() */


#if 0
/* This fn only exists for when the cmd line args need to be tested.  It is being
 * retained for when new arguments are added in future releases
 */
extern void test_cmd_line_args(int argc, char *argv[], int nPE)
{
    char script_file[PI_MAX_PATH + 1];
    pi_iparm_t iparm;
    pi_rparm_t rparm;
    int run_option;

    if (argc == 1)
    {
	printf("No commmand line argument to test!\n");
	exit(1);
    }
    
    if (get_cmd_line_args(argc, argv, &run_option, &rparm, &iparm, script_file, nPE) != 0)
    {
	pi_terminate(PI_ERR_PARMS, "Parint executable called with bad parameters");
	exit(1);
    }

    if (run_option == RUN_DO_HELP)
    {
	print_long_usage();
    }
    else if (run_option == RUN_DO_INFO_1)
    {
	pi_ifcn_print_list_sm();
    }
    else if (run_option == RUN_DO_INFO_2)
    {
	pi_ifcn_print_list_lrg();
    }
    else if (run_option == RUN_DO_INTEGRAL)
    {
	print_rparm(&rparm);
	print_iparm(&iparm);
    }
    else if (run_option == RUN_DO_SCRIPT)
    {
	print_rparm(&rparm);
	printf("Script file is '%s'\n", script_file);
    }
    else
    {
	printf("Invalid run option of %d\n", run_option);
    }
    
} /* test_cmd_line_args() */
#endif
