/*        File: ifcn-ctrl.c
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: All
   Description: Source for functions providing an interface to the integration functions
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/misc/ifcn-ctrl.c,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:30 $

                $Log: ifcn-ctrl.c,v $
                Revision 1.1.1.1  2000/06/06 03:13:30  parint
                Initial checkin


       Version: 03/04/98  rrz  Original
		05/12/98  rrz  Code freeze for ParInt1.0
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
*/

/* This source contains the C functions that provide an interface to the ifcn library */

#include <stdio.h>
#include <string.h>
#include <parint.h>
#include <pi-ifcn.h>
#include <pi-defs.h>
#include <math-misc.h>

/* This file referenced from where the user places it: */
#include <ifcn-ctrl.h>


/* This allows test programs to get access to *all* known functions for 
 * ease in testing
 */
extern pi_ifcn_info_t *pi_ifcn_get_table(int *nproblems)
{
    *nproblems = PI_IFCN_NUM_IFCNS;
    return pi_ifcn_info_table;
} /* pi_get_ifcn_table() */


/* Returns the entire info entry for the given function name.  Mostly used internally
 * and by test programs
 */
extern pi_ifcn_info_t *pi_ifcn_lookup_info(const char *ifcn_name)
{
    int i;
    for (i = 0; i < PI_IFCN_NUM_IFCNS; ++i)
    {
	if (STRCMP(ifcn_name, ==, pi_ifcn_info_table[i].name))
	    return &pi_ifcn_info_table[i];
    }
    return NULL;
} /* pi_ifcn_lookup_info() */


/* Looks up the ifcn info and returns the info in a iparms structure
 */
extern pi_iparm_t *pi_ifcn_lookup_iparms(const char *ifcn_name)
{
    /* A ptr to this will be returned to the user.  The user must copy out or use the
       contents before calling this function again: */
    static pi_iparm_t iparms;

    pi_ifcn_info_t *ifcn_info_p;

    if ((ifcn_info_p = pi_ifcn_lookup_info(ifcn_name)) != NULL)
    {
	/* Transfer contents of ifcn_info_p to a pi_iparm_t structure: */
	strncpy0(iparms.ifcn_name, ifcn_name, PI_MAX_IFCN_NAME);
	iparms.ndims = ifcn_info_p->ndims;
	iparms.nfcns = ifcn_info_p->nfcns;
	iparms.intg_rule = ifcn_info_p->def_intg_rule;
	iparms.eps_a = ifcn_info_p->def_eps_a;
	iparms.eps_r = ifcn_info_p->def_eps_r;
	iparms.fcn_eval_limit = ifcn_info_p->def_fcn_eval_limit;
	if (ifcn_info_p->def_a && ifcn_info_p->def_b)
	{
	    /* Copy only ndims, since that is the expected size of ifcn_info_p->def_[a|b]: */
	    memcpy(iparms.a, ifcn_info_p->def_a, sizeof(pi_base_t) * PI_MAX_NDIM);
	    memcpy(iparms.b, ifcn_info_p->def_b, sizeof(pi_base_t) * PI_MAX_NDIM);
	}
	return &iparms;
    }
    else
    {
	return NULL;
    }
} /* pi_ifcn_lookup_iparms() */


/* Returns the ifcn ptr for the given function
 */
extern pi_ifcn_t pi_ifcn_lookup_ifcn(const char *ifcn_name)
{
    pi_ifcn_info_t *ifcn_info_p;
    return ((ifcn_info_p = pi_ifcn_lookup_info(ifcn_name))
	    ? ifcn_info_p->ifcn_p
	    : NULL);
} /* pi_ifcn_lookup_ifcn() */


/* Returns the description of the given function
 */
extern char *pi_ifcn_lookup_desc(const char *ifcn_name)
{
    pi_ifcn_info_t *ifcn_info_p;
    return ((ifcn_info_p = pi_ifcn_lookup_info(ifcn_name))
	    ? ifcn_info_p->desc
	    : NULL);
} /* pi_ifcn_lookup_desc() */


/* Prints out a list of the available functions.  Used by the parint executable
 * to support the -i or -i1 option
 */
extern void pi_ifcn_print_list_sm(void)
{
    int i;
    printf(" %-15s %-60s\n", "Intg. Fcn. Name", "Intg. Fcn. Description");
    printf(" %-15s %-60s\n", 
	   "---------------", "------------------------------------------------------------");
    for (i = 0; i < PI_IFCN_NUM_IFCNS; ++i)
	printf(" %-15s %-60s\n", pi_ifcn_info_table[i].name, pi_ifcn_info_table[i].desc);
    printf("\n");
} /* pi_ifcn_print_list_sm() */


/* Prints out all info about all ifcns in the fcn library.  Used by the parint executable
 * to support the -i2 option; the listing is used by the gui to provide the gui user
 * with an up-to-date list of all the functions
 */
extern void pi_ifcn_print_list_lrg(void)
{
    int i;
    for (i = 0; i < PI_IFCN_NUM_IFCNS; ++i)
    {
	pi_ifcn_info_t *ii = &pi_ifcn_info_table[i];      /* (to save references and space!) */
	int j;
	
#ifdef USING_LONG_DOUBLES
	printf("%-15s %-60s %3d %2d %1d %.33LG %.33LG %8d", 
	       ii->name, ii->desc, ii->ndims, ii->nfcns, ii->def_intg_rule, ii->def_eps_a,
	       ii->def_eps_r, ii->def_fcn_eval_limit);
#else
	printf("%-15s %-60s %3d %2d %1d %.15G %.15G %8d", 
	       ii->name, ii->desc, ii->ndims, ii->nfcns, ii->def_intg_rule, ii->def_eps_a,
	       ii->def_eps_r, ii->def_fcn_eval_limit);
#endif

	if (ii->def_a && ii->def_b)
	{
#ifdef USING_LONG_DOUBLES
	    for (j = 0; j < ii->ndims; ++j)
		printf(" %.33LG", ii->def_a[j]);
	    for (j = 0; j < ii->ndims; ++j)
		printf(" %.33LG", ii->def_b[j]);
#else
	    for (j = 0; j < ii->ndims; ++j)
		printf(" %.15G", ii->def_a[j]);
	    for (j = 0; j < ii->ndims; ++j)
		printf(" %.15G", ii->def_b[j]);
#endif
	}
	printf("\n");
    }
} /* pi_ifcn_print_list_lrg() */

