/*        File: random.c
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: All
   Description: Source for helper functions for standard pseudo-random number generators
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/misc/random.c,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:31 $

                $Log: random.c,v $
                Revision 1.1.1.1  2000/06/06 03:13:31  parint
                Initial checkin


       Version: 09/13/99  rrtz  Original (from old code of mine)
*/

#include <stdlib.h>

#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>

#include <pi-defs.h>
#include <random.h>

/* This prototype is supposed to be in sys/timeb.h, but it is not there: */
extern int ftime(struct timeb *tp);

/* This static ensures that the random number generator is only
 * seeded once across the two init functions: */
static bool already_seeded = FALSE;


/* Initializes the random number generator based on the current time
 */
extern void random_init(void)
{ 
    if (!already_seeded)
    {
        struct timeb tb;
        ftime(&tb);
        srand((unsigned int) tb.time);
        already_seeded = TRUE;

        /* Print seed to reveal the seed of a particular nasty run: */
        /* printf("Seed is %d\n", tb.time); */
    }
}                               /* random_init() */


/* This fn seeds the random number generator using the current time and
 * the 'help' value.  This fn is used in multi-processing circumstances
 * where roughly simultaneously seeding the random number generator
 * based on the current time (even in seconds) in multiple concurrent
 * proceses may result in the same seed for several processes.  The help
 * value is expected to identify the current process (a pid_t) or the current
 * machine (the task_id in PVM, or maybe the host address).  To avoid
 * overflow problems, the time is converted to a short, and, the help value
 * is a short (possibly requiring non-trivial conversion from a long or int
 * identifier by the caller).
 */
extern void random_init_with_help(short help)
{
    if (!already_seeded)
    {
        struct timeb tb;
        ftime(&tb);

        /* The cast to short should result in peeling off the low-order
           bits, preserving a value that changes every second, yet repeats
           more often that the complete time value (but still only
           every 2^16 seconds!): */
        srand((unsigned int) ((short) tb.time * help));
        already_seeded = TRUE;
    }
} /* random_init_with_help() */
