/*        File: timer.c
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: All
   Description: Source for timing events
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/misc/timer.c,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:30 $

                $Log: timer.c,v $
                Revision 1.1.1.1  2000/06/06 03:13:30  parint
                Initial checkin


      Version : 07/21/97  rrz  Original
		05/01/99  rrz  Modified from very old version
		05/25/99  rrz  Use new routine pi_sys_terminate()
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
		11/23/99  rrz  Add nap routine
*/

#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>

#include <time.h>
#include <sys/timeb.h>
#include <pi-defs.h>
#include <timer.h>

#define NAP_LOOP_GRANULARITY    1000

/* This prototype missing from the system header files on some OS's: */
extern int ftime(struct timeb *tp);


/* These hold the start time for start_time() and stop_time(): */
static time_t start_time_secs;
static unsigned short start_time_millitm;

/* Stores the current time in static variables
 */
extern void timer_start(void)
{
    struct timeb tb;
    if (ftime(&tb) == -1)
	pi_sys_terminate("Error from ftime() function");

    start_time_secs = tb.time;
    start_time_millitm = tb.millitm;
}                               /* timer_start() */


/* Stops the clock and returns the time since start_timer() was
 * last called.  Can be called multiple times if you want to just read 
 * the clock value.
 */
extern double timer_retrieve(void)
{
    struct timeb tb;
    if (ftime(&tb) == -1)
	pi_sys_terminate("Error from ftime() function");
    return timer_diff2dbl(start_time_secs, start_time_millitm, tb.time, tb.millitm);
}                               /* timer_retrieve() */


/* Retrieves the time so that the caller can maintain multiple, overlapping
 * timers.
 */
extern void timer_retrieve_raw(int *secs, int *msecs)
{
    struct timeb tb;
    ftime(&tb);
    *secs  = tb.time;
    *msecs = tb.millitm;
}				/* timer_retrieve() */


extern void nap(double naptime)
{
    int start_secs, start_msecs;
    int cur_secs, cur_msecs;
    int i;

    if (naptime == 0.0)
	return;

    timer_retrieve_raw(&start_secs, &start_msecs);

    do
    {
	for (i = 0; i < NAP_LOOP_GRANULARITY; ++i)
	    ;
	timer_retrieve_raw(&cur_secs, &cur_msecs);
    } until (timer_diff2dbl(start_secs, start_msecs, cur_secs, cur_msecs) >= naptime);
    
} /* nap() */

