/*        File: ctrl-as-wrkr.c
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: MPI
   Description: Header file for source that implements IC as IW functionality
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/mpi/ctrl-as-wrkr.c,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:33 $

                $Log: ctrl-as-wrkr.c,v $
                Revision 1.1.1.1  2000/06/06 03:13:33  parint
                Initial checkin


       Version: 01/06/98  rrz  Original (from hier-ctrlAsWrkr version)
		05/12/98  rrz  Code freeze for ParInt1.0
		06/28/98  rrz  Modified for new caw_init_intg() proto (for
				visualization)
		7/9/98    jli  code for visulization
		10/22/98  rrz  For keeping heap: Added keep heap logic
		11/17/98  rrz  Modify call to get_initial_region()
		12/18/98  rrz  Modify for changed get_initial_region()
				prototype
		05/06/99  rrz  Duplicate the 03/15/99 ParInt1.0 bug fix in this
				(nextrel) code
		07/13/99  rrz  Add logging calls
		07/14/99  rrz  BUG FIX: Add explicit check after receiving a
				region to see if we are idle or not
		07/19/99  rrz  Modify logging of recv-helper and recv-rgn
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
		03/18/00  rrz  Add changes for using deap
*/

/* #define DBG_THIS_FILE */

/* #define SPECIAL_GRAPH_2 */

/* This source file was written to hold functions that implement the integral
 * worker functionality for the controller.  The functions in here implement
 * pieces of the worker logic and are called by the integral controller at various
 * points in intg-ctrl.c.  The functions in here *only* work when called
 * by the integral controller (though similar functions could be written
 * for other non-integral-worker processes to call)
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <mpi.h>
#include <parint.h>
#include <pi-vspec.h>
#include <pi-defs.h>
#include <heapdeap.h>
#include <math-misc.h>
#include <math-rgn.h>
#include <math-intg.h>
#include <math-adapt.h>
#include <messaging.h>
#include <measures.h>
#include <visual.h>
#include <logging.h>

/*** Variables global to this file: */

/* These make sure functions are called in the correct order: */
static bool init_first = FALSE;
static bool init_intg = FALSE;

/* These variables are constant for a given virtual machine, and therefore 
 * for an entire run of ParInt: */
static MPI_Comm comm;
static int id;

/* These are parameters of a single integral.  (The variable nworkers is included here since it
 * includes the IC if it is working, which is a run-time parameter, and can therefore change
 * between integrals.): */
static pi_ifcn_t ifcn_ptr;
static MPI_Datatype update_mpi_t, region_mpi_t;
static pi_base_t *a, *b;
static int ndims, nfcns, irule, fcns_per_basrul, nworkers, vis_type, max_heap_size;
static bool do_lb;
static double lb_help_ratio;
static pi_rparm_t *rparm_p;
static bool using_deap;

/* These variables are maintained throughout the calc of a single integral: */
static heapdeap_t hpdp;
static pi_base_t estabs[PI_MAX_NFCN];     /* Local ESTABS */
static pi_base_t full_vol;                /* Volume of original full region      */
static pi_base_t local_vol;               /* Starting volume of the worker's rgn */
static pi_base_t size_ratio;              /* Local_vol / Total_vol               */
static update_t wrkr_update;

static  double start_time, current_time; /* !!jli */

/* This fn is called to initialize the basic parameters of a single run of ParInt.
 * It only needs to be called once; if it is called more than once then 
 * it just does nothing (this makes it easier on intg-ctrl.c).
 */
extern void caw_init_first(MPI_Comm comm_parm, int id_parm)
{
    if (!init_first)
    {
	comm        = comm_parm;
	id          = id_parm;
	init_first  = TRUE;
	start_time  = MPI_Wtime();/* jli!! record the start time */
    }
} /* caw_init_once() */


/* This function initializes parameters for a single integration evaluation.
 * It should be called after caw_init_first(), and if there have been previous
 * integrals evaluated, then caw_finalize_intg() must have been called 
 * when they were completed. All parameters are IN parameters, and have their 
 * value transferred to variables global to this source file for use by other
 * functions
 */
extern void caw_init_intg(int ndims_parm, pi_base_t a_parm[], pi_base_t b_parm[], 
			  pi_ifcn_t ifcn_ptr_parm, int nfcns_parm, int irule_parm, 
			  int nworkers_parm, pi_rparm_t *rparm_p_parm)
{
    pi_assert(init_first && !init_intg, "Ctrl-as-worker functions called out of order!\n");
    
    /* I put these here in case we ever change them based on ndims or nfcns: */
    create_mpi_type_region(&region_mpi_t);
    create_mpi_type_update(&update_mpi_t);

    rparm_p = rparm_p_parm;
    ifcn_ptr = ifcn_ptr_parm;
    ndims = ndims_parm;
    a = a_parm;
    b = b_parm;
    nfcns = nfcns_parm;
    irule = irule_parm;
    nworkers = nworkers_parm;
    do_lb = rparm_p_parm->do_lb;
    lb_help_ratio = rparm_p_parm->lb_help_ratio;
    vis_type = rparm_p_parm->vis_type;
    fcns_per_basrul = neval(ndims, irule);
    using_deap = (rparm_p_parm->max_heap_size > 0);
    max_heap_size = (rparm_p_parm->max_heap_size > 0) ? rparm_p_parm->max_heap_size : -1;
    pl_log_iw_start();
    init_intg = TRUE;
} /* caw_init_intg() */

/* Shadows, of shadows, passing. It is now 1831, and as always, I am absorbed with a delicate
   thought.  It is how poetry has indefinite sensations, to which end, music is an essential. 
   Since the comprehension of sweet sound is our most indefinite conception, music, when combined
   with a pleasurable idea, is poetry.  Music, without the idea, is simply music.  Without music,
   or an intriguing idea, color becomes pallor, man becomes coccus, home becomes catacomb, and the
   dead, are but for a moment, motionless.  -- Edgar Allan Poe. */

/* This function performs the 'first pass' evaluation.  It calculates its portion
 * of the region, evaluates it, and places the region on the heap.  The result
 * is returned via the parameter (an OUT parameter)
 */
extern void caw_first_pass(update_t **ctrl_update)
{
    region_t rgn;

    pi_assert(init_intg, "Ctrl-as-worker functions called out of order!\n");
    
    clear_update(&wrkr_update, nfcns);
    clear_estabs(estabs, nfcns);
    /* TRUE == ic_is_working, which is true, since I am the IC!: */
    get_initial_region(ndims, nworkers, a, b, id, &rgn, &full_vol, &local_vol, rparm_p);
    size_ratio = local_vol / full_vol;
    D(printf("[IW-%d]: My region: local_vol: %g; full_vol: %g; ratio: %f\n", 
	     id, (double) local_vol, (double) full_vol, (double) size_ratio));

    /*  Initialize the heap with results from the initial region: */
    work_initialize(ndims, irule);
    work_do_round(ndims, nfcns, ifcn_ptr, &rgn);
        
    update_update(&wrkr_update, estabs, &rgn, nfcns, fcns_per_basrul);
    pl_log_work(pl_work_init, &rgn, NULL, estabs, wrkr_update.cnt, 0.0);
    heap_create(&hpdp, using_deap);
    heap_add_node(&hpdp, &rgn);

   /* !! jli visualization */
   if (vis_type == VIS_ON)
   {
       current_time = MPI_Wtime();
       vis_init_min_err(rgn);
       /* update l_eps, #regions, etc. */
       vis_caw_update_update(&wrkr_update,estabs,nfcns,current_time - start_time, rgn.rgnerr);
   }
	
    ms_update_fcncount(fcns_per_basrul);
    D(printf("[IW-%d]: Completing first pass of work\n", id));
    
    /* At this point, the integral worker usually sends up a update, block receives a STOP
       or an EPS message and either ends, or, checks to see if locally done. Since I am the
       controller, I will just terminate. The caller can look at my results via the wrkr_update 
       structure: */
    *ctrl_update = &wrkr_update;
    pl_log_send_upd(pl_upd_init, &wrkr_update);

} /* caw_first_pass() */


/* This function checks to see if this worker (the controller) is locally done.
 * The parameter g_eps (IN) is the current global eps value, the bool at the locally_idle ptr
 * (OUT) is set to 1 if we are done, 0 otw.  That value is also returned.
 */
extern bool caw_check_ldone(pi_base_t g_eps, bool *locally_idle)
{
    pi_assert(init_intg, "Ctrl-as-worker functions called out of order!\n");
    return (*locally_idle 
	    = wrkr_update.l_done 
	    = result_close_enough(estabs, g_eps * size_ratio, nfcns));
} /* caw_check_ldone() */


/* This performs a single 'pass' though the IW's main loop.  The parameters: g_eps (IN)
 * is the current eps value; helper_id (IN) is the id of a worker that can be sent some help
 * (helper_id is -1 if there is no helper available), the result of the update is passed back via
 * ctrl_update (OUT), which should only be used if use_update (OUT) is set.  Locally_done (OUT) is
 * set if we end up being locally done as a result of the pass.  The flag used_helper is set if a
 * valid helper_is is provided, and, a message 4 is sent to that helper (the caller will use that
 * flag to mark the helper as non-idle) containing a valid region.  This saves the helper from
 * having to send a msg 5 when it gets a non-valid region, and, if we are locally done, then we 
 * want to make sure to not mark the helper as non-idle.  Note that the used_helper flag pointer
 * may be NULL if we are not doing any load-balancing
 */
extern void caw_one_pass(pi_base_t g_eps, int helper_id, bool *use_update, update_t **ctrl_update, 
			 bool *locally_done, bool *used_helper)
{
    pi_base_t l_eps = g_eps * size_ratio;
    pi_assert(init_intg, "Ctrl-as-worker functions called out of order!\n");

    /* Default value: */
    if (used_helper)
	*used_helper = FALSE;

    /* If I am locally done, then check for a SEND_RGN message: */
    if (caw_check_ldone(g_eps, locally_done))
    {
	MPI_Status status;

	if (pi_iprobe(MPI_ANY_SOURCE, TAG_LB_SEND_RGN, comm, &status))
	{
	    region_t newrgn;
	    /* pi_assert(used_helper, "Invalid used_helper pointer in caw_one_pass()"); */
	    pi_recv(&newrgn, 1, region_mpi_t, status.MPI_SOURCE, TAG_LB_SEND_RGN, comm, &status);

	    if (newrgn.divaxn >= 0)
	    {
		/* Help is accepted.  Prepare to work!: */
		int k;
		heap_add_node(&hpdp, &newrgn);
		local_vol += newrgn.rgnvol;
		for (k = 0; k < nfcns; k++)
		    estabs[k] += newrgn.rgnerr_v[k];
		size_ratio = local_vol / full_vol;
		l_eps = g_eps * size_ratio;
		if (using_deap && heap_size(&hpdp) > max_heap_size)
		{
		    region_t rgn_to_throw_out;
		    deap_delete_min(&hpdp, &rgn_to_throw_out);
		    D(printf("[IW-%d]: Threw out region (after lb) %G\n", id,
			     rgn_to_throw_out.rgnerr));
		}
		/* BUG FIX (RRZ-07/13/99): We used to assume that the sent region would
		   make us non-idle.  This was a bad asumption.  Now I make a check: */
		wrkr_update.l_done = caw_check_ldone(g_eps, locally_done);
		pl_log_recv_rgn((wrkr_update.l_done ? pl_lb_fail : pl_lb_success),
				status.MPI_SOURCE, &newrgn, size_ratio, l_eps,
				estabs, wrkr_update.l_done);
		/* (Note: the caw does not send an idle-correction msg here, unlike the IW) */
		if (vis_type == VIS_ON)
		    vis_min_rgn_err(newrgn);
		D(printf("[IW-%d]: Received SEND_RGN w/rgn from %d; new size_ratio "
			 "%G; new erratio %G%s\n", id, status.MPI_SOURCE, size_ratio,
			 err_ratio_2(estabs, nfcns, l_eps), (locally_done ? "; STILL IDLE" : "")));
	    }
	    else
	    {
		/* Help rejected; inform controller still idle: */
		wrkr_update.l_done = *locally_done = TRUE;
		pl_log_recv_rgn(pl_lb_nowork, status.MPI_SOURCE, NULL, 0.0, 0.0, NULL, TRUE);
		D(printf("[IW-%d]: Received SEND_RGN from %d, but help rejected\n",
			 id, status.MPI_SOURCE));
	    }
	}
    }

    /* If I am done, and I got no SEND_RGN msg, then there is nothing to be done: */
    if (wrkr_update.l_done)
    {
	*use_update = FALSE;
	D(printf("[IW-%d]: In caw_one_pass(), but locally done\n", id));
    }
    else
    {
	region_t oldrgn, new[REGION_SPLITS];
	int j, k;
	
	/* Start off by clearing the wrkr_update structure, since every time 
	   this function exits I am updating the controller: */
	clear_update(&wrkr_update, nfcns);

	/* Since we are the controller, only ever do a single round of work.  Get a region
	   with the largest error: */
	oldrgn = *heap_peek_max(&hpdp);
	for (k = 0; k < nfcns; k++)
	{
	    wrkr_update.res[k] -= oldrgn.rgnval[k];
	    wrkr_update.est[k] -= oldrgn.rgnerr_v[k];
	    estabs[k] -= oldrgn.rgnerr_v[k];
	}

	/* Split the retrieved region: */
	split_region(new, &oldrgn);

	/* Calc the integral over each of the new regions: */
	for (j = 0; j < REGION_SPLITS; j++ )
	{
	    /* This does a complete round of work, including the updating
	       of rgnerr and divaxn in new[j]: */
	    work_do_round(ndims, nfcns, ifcn_ptr, &new[j]);
	    
	    /* Update the worker's update and add the new region onto the heap: */
	    update_update(&wrkr_update, estabs, &new[j], nfcns, fcns_per_basrul);
	    pl_log_work((j == 0 ? pl_work_first : pl_work_second), &new[j], &oldrgn,
			estabs, wrkr_update.cnt, l_eps);
	    if (j == 0)
		heap_replace_root(&hpdp, &new[j]);
	    else
		heap_add_node(&hpdp, &new[j]);
	    /* !! jli */
	    if (vis_type == VIS_ON)
	    {
		long num_region;
		vis_min_rgn_err(new[j]);
		current_time = MPI_Wtime();
		num_region = vis_caw_get_num_region(hpdp.maxrtptr);
		/* update l_eps, #regions, etc. */
		vis_update_update(&wrkr_update, estabs, nfcns, current_time - start_time,
				  num_region);
   	    }
	    ms_update_fcncount(fcns_per_basrul);
	}
	if (using_deap && heap_size(&hpdp) > max_heap_size)
	{
	    region_t rgn_to_throw_out;
	    deap_delete_min(&hpdp, &rgn_to_throw_out);
	    D(printf("[IW-%d]: Threw out region %G\n", id, rgn_to_throw_out.rgnerr));
	}
	D(printf("[IW-%d]: CAW after work (l_eps ratio %.8f; hpsz %d)\n",
		 id, (double) (infNorm(estabs, nfcns) / l_eps), heap_size(&hpdp)));

	/* If the controller (myself) provided me with a helper_id, and I am not locally done, 
	   and I am not close to being locally done, then send the helper a region.  Note that
	   this is different from the IW code, in that if I do not need to send a region, I do 
	   not, since I can easily tell the IC (myself) that I did not send a region, and that
	   therefore the helper_id IW is still idle.  (I pass this info back to the IC code
	   via the used_helper flag): */
	if (   do_lb
	    && helper_id != -1
	    && !wrkr_update.l_done
	    && (infNorm(estabs, nfcns) > lb_help_ratio * l_eps)) 
	{
#ifdef USE_LOGGING
	    pi_base_t old_err_ratio = err_ratio_2(estabs, nfcns, l_eps);
#endif
	    region_t newrgn;
	    int k;
	    D(printf("[IW-%d]: Received helper id of %d; accepting offer of help "
		     "(factor: %f)\n", id, helper_id, (double) (infNorm(estabs, nfcns) / l_eps)));
	    heap_delete_max(&hpdp, &newrgn);
	    for (k = 0; k < nfcns; k++)
		estabs[k] -= newrgn.rgnerr_v[k]; 
	    local_vol -= newrgn.rgnvol;
	    size_ratio = local_vol / full_vol;
	    l_eps = g_eps * size_ratio;
	    pi_send(&newrgn, 1, region_mpi_t, helper_id, TAG_LB_SEND_RGN, comm);
	    *used_helper = TRUE;
	    pl_log_recv_helper(pl_lb_sending, old_err_ratio, &newrgn, size_ratio,
			       err_ratio_2(estabs, nfcns, l_eps), estabs);
	    /* (Note that the caw never logs refused instances of help, because help is sent
	       in such a different manner than with the workers) */
	}

#ifdef SPECIAL_GRAPH_2
	printf("    %e     %e\n", oldrgn.rgnerr, infNorm(wrkr_update.est, nfcns));
#endif
	
#if 0
	wrkr_update.l_estabs = infNorm(estabs, nfcns);
#endif
	*use_update = TRUE;
	*ctrl_update = &wrkr_update;
	pl_log_send_upd(pl_upd_reg, &wrkr_update);
    }
    
    /* Leave off with a check of the eps; if we are done then we may be able to
       help before the next iteration: */
    /* BUG FIX (R05/06/99; RRZ) SEE comment in the ParInt1.0 code in this spot as to why we
       do *not* want to check for being done here */
    /* caw_check_ldone(g_eps, locally_done); */
} /* caw_one_pass() */


/* This function exists to clean up after the work done by the controller as 
 * a worker.
 */
extern void caw_finalize_intg(void)
{
    MPI_Status status;

    pi_assert(init_intg, "Ctrl-as-worker functions called out of order!");
    pl_log_iw_stop();
    heap_free(&hpdp);
    
    /* Right here I need to clean out some messages.  There is only a single kind of message
       that the IC-as-IW gets, specifically, SEND_RGN messages.  So clean them out in-line
       here: */
    while (pi_iprobe(MPI_ANY_SOURCE, TAG_LB_SEND_RGN, comm, &status))
    {
	region_t newrgn;
	pi_recv(&newrgn, 1, region_mpi_t, status.MPI_SOURCE, TAG_LB_SEND_RGN, comm, &status);
    }

    MPI_Type_free(&update_mpi_t);
    MPI_Type_free(&region_mpi_t);
    init_intg = FALSE;
} /* caw_finalize() */


/* jli!!  get histogram */
void caw_vis_get_hist(hist_t * hist,int hist_type)
{
    current_time = MPI_Wtime();
    vis_get_hist(hpdp.maxrtptr, hist,hist_type,current_time - start_time);
}


