/*        File: log-filter.c
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: MPI (logging)
   Description: Source used by both filter programs and logging funcitons
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/mpi/log-filter.c,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:33 $

                $Log: log-filter.c,v $
                Revision 1.1.1.1  2000/06/06 03:13:33  parint
                Initial checkin


       Version: 05/21/99  rrz  Original
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
*/

/* Is there a better way to do this?  I do not know it.  The #include of pi-version.h
 * must be first: */
#include <pi-version.h>
#ifdef USE_LOGGING

/* Note: The source in here was broken out of logging.c because it was needed by the filter
 * programs, and, we do not want the filter programs to need to link with logging.c, because
 * then MPI code is needed.  It is assumed that the filter programs will *not* be MPI programs */

#include <math-misc.h>     /* for MAX() */
#include <log-filter.h>

/* These specify the base size, in bytes, of each event record.  (The "base size" is the # of 
 * bytes used by non-variable sized fields in a log record.)  The BASE_BASE size is the
 * # of bytes written by prepare_log_buffer() (and represents the # of bytes taken up by fields
 * common to all event records): */
#define BASE_BASE_SIZE           14
#define BASE_SIZE_IC_STOP        (BASE_BASE_SIZE + 1)
#define BASE_SIZE_RECV_UPD       (BASE_BASE_SIZE + 43)
#define BASE_SIZE_RECV_IDLE      (BASE_BASE_SIZE + 1)
#define BASE_SIZE_SEND_EPS       (BASE_BASE_SIZE + 17)
#define BASE_SIZE_SEND_HELPER    (BASE_BASE_SIZE + 2)

#define BASE_SIZE_IW_START       (BASE_BASE_SIZE + 0)
#define BASE_SIZE_SEND_UPD       (BASE_BASE_SIZE + 10)
#define BASE_SIZE_WORK           (BASE_BASE_SIZE + 47)
#define BASE_SIZE_RECV_RGN       (BASE_BASE_SIZE + 40)
#define BASE_SIZE_RECV_EPS       (BASE_BASE_SIZE + 25)
#define BASE_SIZE_IW_STOP        (BASE_BASE_SIZE + 0)
#define BASE_SIZE_SEND_IDLE      (BASE_BASE_SIZE + 1)
#define BASE_SIZE_RECV_HELPER    (BASE_BASE_SIZE + 38)


/* All of the stored sizes use many literal constants that are only used here; if the events
 * are changed, this function must be changed.  Note that the sizes here refer to the size of
 * the record as it is stored on disk, in a very compact form.
 */
extern void calc_ic_record_sizes(int *max_logrec_size, int event_rec_sizes[], 
				 int nPE, int nfcns, int ndims)
{
    event_rec_sizes[pl_event_ic_stop] = BASE_SIZE_IC_STOP;
    event_rec_sizes[pl_event_recv_upd] = (BASE_SIZE_RECV_UPD 
					  + 2 * nfcns * SZ_BASE_T + nPE * SZ_BOOL_T);
    event_rec_sizes[pl_event_recv_idle] = BASE_SIZE_RECV_IDLE + nPE * SZ_BOOL_T;
    event_rec_sizes[pl_event_send_eps] = BASE_SIZE_SEND_EPS;
    event_rec_sizes[pl_event_send_helper] = BASE_SIZE_SEND_HELPER + nPE * SZ_BOOL_T;
    if (max_logrec_size)
    {
	int i;
	for (i = MIN_IC_EVENT; i <= MAX_IC_EVENT; ++i)
	    *max_logrec_size = MAX(*max_logrec_size, event_rec_sizes[i]);
    }
} /* calc_ic_record_sizes() */


/* Same comment as above applies here
 */
extern void calc_iw_record_sizes(int *max_logrec_size, int event_rec_sizes[], int nfcns, int ndims)
{
    event_rec_sizes[pl_event_iw_start] = BASE_SIZE_IW_START;
    event_rec_sizes[pl_event_send_upd] = BASE_SIZE_SEND_UPD + 2 * SZ_BASE_T * nfcns;
    event_rec_sizes[pl_event_work] = (BASE_SIZE_WORK + 
				      2 * SZ_BASE_T * ndims 
				      + 3 * SZ_BASE_T * nfcns);
    event_rec_sizes[pl_event_recv_rgn] = BASE_SIZE_RECV_RGN + SZ_BASE_T * nfcns;
    event_rec_sizes[pl_event_iw_stop] = BASE_SIZE_IW_STOP;
    event_rec_sizes[pl_event_recv_eps] = BASE_SIZE_RECV_EPS;
    event_rec_sizes[pl_event_send_idle] = BASE_SIZE_SEND_IDLE;
    event_rec_sizes[pl_event_recv_helper] = BASE_SIZE_RECV_HELPER + SZ_BASE_T * nfcns;

    if (max_logrec_size)
    {
	int i;
	for (i = MIN_IW_EVENT; i <= MAX_IW_EVENT; ++i)
	    *max_logrec_size = MAX(*max_logrec_size, event_rec_sizes[i]);
    }
} /* calc_iw_record_sizes() */

#endif
