/*        File: log-filter.h
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: MPI (logging)
   Description: Header file for stuff used by both logging and filtering code
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/mpi/log-filter.h,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:33 $

                $Log: log-filter.h,v $
                Revision 1.1.1.1  2000/06/06 03:13:33  parint
                Initial checkin


       Version: 06/02/99  rrz  Original
		07/14/99  rrz  Remove killheap/keepheap option
		07/19/99  rrz  Moved things to filter.h from here; modified
				some enumerated types.
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
*/

/* Note that since filtering code will be including this, and the filter programs are not MPI 
 * programs, no MPI defs or tokens should be in this file.  Also note that any changes here
 * may require changes to the filter code (e.g., filter.c), as well as any filter programs,
 * and may also make existing log files incomptable with the resulting modified filter 
 * programs.
 */

#ifndef LOG_FILTER_H
#define LOG_FILTER_H

#include <pi-version.h>

#ifndef USE_LOGGING
#error USE_LOGGING must be on to use log-filter.h
#endif

#ifdef USING_LONG_DOUBLES
#error Compilation problem: Cannot use long doubles with the logger!
#endif

#include <pi-defs.h>    /* for bool, etc... */

/* (Note that pl_nnum_t and pl_rank_t are in pi-defs.h, as they are used there, and, logging.h
 * should depend upon pi-defs.h, not the other way around)
 */
typedef double        pl_base_t;
typedef double        pl_time_t;
typedef ulong         pl_enums_t;
typedef ushort        pl_flags_t;
typedef unsigned char pl_enum_t;   /* A single enum value; used in raw log file */

/* These are all the enumerated values for the logging; they all start at 1, for better error
 * checking (and it works out to take up no extra space): */
enum { pl_upd_init = 1, pl_upd_reg, pl_upd_braking, pl_upd_max };
enum { pl_stop_quick_eps = 1, pl_stop_quick_fcn, pl_stop_eps, pl_stop_fcn, pl_stop_all_idle, 
       pl_stop_max };
enum { pl_eps_init = 1, pl_eps_reg, pl_eps_max };
enum { pl_lb_success = 1, pl_lb_nowork, pl_lb_fail, pl_lb_sending, pl_lb_max };
enum { pl_work_first = 1, pl_work_second, pl_work_init, pl_work_max };
enum { pl_idle_quick = 1, pl_idle_lb_nowork, pl_idle_lb_fail, pl_idle_max };
enum { pl_event_ic_stop = 1, pl_event_recv_upd, pl_event_recv_idle, pl_event_send_eps, 
       pl_event_send_helper, pl_event_iw_start, pl_event_iw_stop, pl_event_recv_eps, 
       pl_event_recv_helper, pl_event_send_upd, pl_event_work, pl_event_send_idle, 
       pl_event_recv_rgn, pl_event_max };

#define MIN_IC_EVENT pl_event_ic_stop
#define MAX_IC_EVENT pl_event_send_helper
#define MIN_IW_EVENT pl_event_iw_start
#define MAX_IW_EVENT pl_event_recv_rgn

#define PL_IW_ARR_SZ            4                   /* relates to the max # of workers logged */
#define PL_MAX_WORKERS        (PL_IW_ARR_SZ * 16)
#define BITS_PER_FLAGS_T       16

/* This is the log header record structure. It holds all needed parameters of the problem.  It
 * consists mostly of fields from pi_rparm_t and pi_iparm_t */
typedef struct {
    char      ifcn_name[PI_MAX_IFCN_NAME + 1];
    int       ndims;                          
    int       nfcns;                          
    int       intg_rule;                      
    pl_base_t eps_a;                          
    pl_base_t eps_r;                          
    int       fcn_eval_limit;                 
    bool      do_lb;                 
    bool      ic_is_worker;          
    bool      simple_rgns;           
    double    lb_help_ratio;         
    int       ns;                    
    int       fcns_per_irule;
    int       nPE;
    pl_base_t * a;
    pl_base_t * b;
} pl_header_rec_t;

/*** Source provided by log-filter.c: */
extern void calc_ic_record_sizes(int *max_logrec_size, int event_rec_sizes[], 
				 int nPE, int nfcns, int ndims);
extern void calc_iw_record_sizes(int *max_logrec_size, int event_rec_sizes[], int nfcns, 
				 int ndims);


/* Here just for succinctness of code, and for keeping the code from being too wordy, and in
 * an attempt to make the code cleaner, and, overall, less cluttered with a lot of junk ( ;-) ): */
#define SZ_RANK_T      sizeof(pl_rank_t)
#define SZ_BOOL_T      sizeof(bool)
#define SZ_BASE_T      sizeof(pl_base_t)
#define SZ_ENUM_T      sizeof(pl_enum_t)
#define SZ_NNUM_T      sizeof(pl_nnum_t)
#define SZ_ULONG_T     sizeof(ulong)
#define SZ_USHORT_T    sizeof(ushort)

#endif /* LOG_FILTER_H */
