/*        File: logging.h
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: MPI (logging)
   Description: Header file for all logging #def's, typedefs, etc...
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/mpi/logging.h,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:33 $

                $Log: logging.h,v $
                Revision 1.1.1.1  2000/06/06 03:13:33  parint
                Initial checkin


       Version: 05/20/99  rrz  Original
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
*/

/* This header file is only included by ParInt code that writes log records.  It is not included
 * by the filter programs. */

#ifndef _LOGGING_H
#define _LOGGING_H

#include <pi-version.h>
#include <pi-vspec.h>
#include <pi-defs.h>    /* for bool, etc... */

/* This section defines the macros that evaluate to logging functions, or, nothing, based
 * on the USE_LOGGING cpp token: */

#ifdef USE_LOGGING

/* I think this is sound: the .c includes logging.h, which can be included whether or not
 * logging is turned on.  Then, logging.h only conditionally includes log-filter.h (only if
 * logging is on), because I do not want to have extra header files included: */
#include <log-filter.h>

/* MPI types for typedef's types for logging: */
#define MPI_PL_NNUM_T    MPI_UNSIGNED_LONG
#define MPI_PL_RANK_T    MPI_UNSIGNED_CHAR

#define pl_finalize()                                 _pl_finalize()
#define pl_ic_init(fn, nd, nf, ir, fel, ea, er, a, b, rparm, nPE, id)  _pl_ic_init(fn, nd, nf, ir, fel, ea, er, a, b, rparm, nPE, id)
#define pl_log_send_helper(recip, helper, idle)       _pl_log_send_helper(recip, helper, idle)
#define pl_log_send_eps(epst, eps, leps)              _pl_log_send_eps(epst, eps, leps)
#define pl_log_recv_idle(sender, idle)                _pl_log_recv_idle(sender, idle)
#define pl_log_recv_upd(updt, upd, sdr, fcnc, eps, leps, res, est, idle) _pl_log_recv_upd(updt, upd, sdr, fcnc, eps, leps, res, est, idle)
#define pl_log_ic_stop(stopt)                         _pl_log_ic_stop(stopt)

#define pl_iw_init(nd, nf, ir, rparm, nPE, id)        _pl_iw_init(nd, nf, ir, rparm, nPE, id)
#define pl_log_iw_start()                             _pl_log_iw_start()
#define pl_log_send_upd(updt, upd)                    _pl_log_send_upd(updt, upd)
#define pl_log_work(wt, rgn, prgn, est, fcnc, leps)   _pl_log_work(wt, rgn, prgn, est, fcnc, leps)
#define pl_log_recv_rgn(lbt, sid, rgn, sr, le, est, ld) _pl_log_recv_rgn(lbt, sid, rgn, sr, le, est, ld)
#define pl_log_recv_eps(epst, geps, leps, errrat)     _pl_log_recv_eps(epst, geps, leps, errrat)
#define pl_log_send_idle(idlet)                       _pl_log_send_idle(idlet)
#define pl_log_recv_helper(lbt, oer, rgn, sr, ner, est) _pl_log_recv_helper(lbt, oer, rgn, sr, ner, est)
#define pl_log_iw_stop()                              _pl_log_iw_stop()

extern void _pl_finalize(void);
extern void _pl_ic_init(const char fcn_name[], int ndims_parm, int nfcns_parm, int irule, 
			int fcn_eval_limit, pi_base_t eps_a, pi_base_t eps_r,
			const pi_base_t a[], const pi_base_t b[], const pi_rparm_t *rparms_p,
			int nPE_parm, int id_parm);
extern void _pl_log_send_helper(int recipient_id, int helper_id, const bool idle_workers[]);
extern void _pl_log_send_eps(int pl_eps_type, pi_base_t eps, pi_base_t last_sent_eps);
extern void _pl_log_recv_idle(int sender_id, const bool idle_workers[]);
extern void _pl_log_recv_upd(int pl_upd_type, const update_t *update_p, int sender_id, 
			     int fcn_count, pi_base_t eps, pi_base_t last_sent_eps,
			     const pi_base_t result[], const pi_base_t estabs[], 
			     const bool idle_workers[]);
extern void _pl_log_ic_stop(int pl_stop_type);
extern void _pl_iw_init(int ndims_parm, int nfcns_parm, int irule, const pi_rparm_t *rparms_p,
			int nPE_parm, int id_parm);
extern void _pl_log_iw_start(void);
extern void _pl_log_send_upd(int pl_upd_type, update_t *update_p);
extern void _pl_log_work(int pl_work_type, region_t *rgn_p, const region_t *parent_rgn_p,
			 const pl_base_t estabs[], int fcn_count, pl_base_t local_eps);
extern void _pl_log_recv_rgn(int pl_lb_type, int sender_id, const region_t *rgn_p,
			     pi_base_t size_ratio, pi_base_t local_eps, const pi_base_t estabs[],
			     bool locally_done);
extern void _pl_log_recv_eps(int pl_eps_type, pi_base_t global_eps, pi_base_t local_eps,
			     pi_base_t error_ratio);
extern void _pl_log_send_idle(int pl_idle_type);
extern void _pl_log_recv_helper(int pl_lb_type, pi_base_t cur_err_ratio, const region_t *rgn_p,
				pi_base_t size_ratio, pi_base_t new_error_ratio,
				pi_base_t estabs[]);
extern void _pl_log_iw_stop(void);

#else

#define pl_finalize()
#define pl_ic_init(fn, nd, nf, ir, fel, ea, er, a, b, rparm, nPE, id)
#define pl_log_send_helper(recip, helper, idle)
#define pl_log_send_eps(epst, eps, leps)
#define pl_log_recv_idle(sender, idle)
#define pl_log_recv_upd(updt, upd, sdr, fcnc, eps, leps, res, est, idle)
#define pl_log_ic_stop(stopt)

#define pl_iw_init(nd, nf, ir, rparm, nPE, id)
#define pl_log_iw_start()
#define pl_log_work(wt, rgn, prgn, est, fcnc, leps)
#define pl_log_send_upd(updt, upd)
#define pl_log_recv_rgn(lbt, sid, rgn, sr, le, est, ld)
#define pl_log_recv_eps(epst, geps, leps, errrat)
#define pl_log_send_idle(idlet)
#define pl_log_recv_helper(lbt, oer, rgn, sr, ner, est)
#define pl_log_iw_stop()

#endif

#endif /* _LOGGING_H */
