/*        File: measures.h
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: MPI
   Description: Header file for measurement functions
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/mpi/measures.h,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:33 $

                $Log: measures.h,v $
                Revision 1.1.1.1  2000/06/06 03:13:33  parint
                Initial checkin


       Version: 01/17/98  rrz  Original
		05/12/98  rrz  Code freeze for ParInt1.0
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
		08/20/99  rrz  Add in: measuring idle time
*/

#ifndef MEASURES_H
#define MEASURES_H

#include <pi-version.h>
#include <pi-defs.h>

#define MS_COLLECTOR_ID   0
#define MAX_MSGS_TAGS    (TAG_MAX - TAG_BASE)

#ifdef USE_MEASUREMENTS

/* This is used to report additional information beyond the pi_status_t structure.  
 * USE_MEASUREMENTS must be on to get this information: */
typedef struct
{
    double idle_percent;
} ms_morestatus_t;

extern void ms_get_morestatus(ms_morestatus_t *morestatus_p);

#define ms_update_sends_by_tag(tag)         _ms_update_sends_by_tag(tag)
#define ms_update_comm_time(cstrt, cstop)   _ms_update_comm_time(cstrt, cstop)
#define ms_update_fcncount(calls)           _ms_update_fcncount(calls)
#define ms_update_idle(idle)                _ms_update_idle(idle)
#define ms_stop_clock()                     _ms_stop_clock()
#define ms_start_clock()                    _ms_start_clock()
#define ms_handle_results(fper, run)        _ms_handle_results(fper, run)
#define ms_initialize_run()                 _ms_initialize_run()
#define ms_mrun_print(t, f, nr, lim)        _ms_mrun_print(t, f, nr, lim)
#define ms_mrun_initialize()                _ms_mrun_initialize()

extern void _ms_update_sends_by_tag(int tag);
extern void _ms_update_comm_time(double comm_start, double comm_stop);
extern void _ms_update_fcncount(int fcn_calls);
extern void _ms_update_idle(bool now_idle);
extern void _ms_stop_clock(void);
extern void _ms_start_clock(void);
extern void _ms_handle_results(int fcns_per_basrul, int run);
extern void _ms_initialize_run(void);
extern void _ms_mrun_print(double mrun_times[], int mrun_fcn_cnts[], int nruns, 
			   int fcn_eval_limit);
extern void _ms_mrun_initialize(void);

#else

#define ms_update_sends_by_tag(tag)         
#define ms_update_comm_time(cstrt, cstop)   
#define ms_update_fcncount(calls)           
#define ms_update_idle(idle)
#define ms_stop_clock()                     
#define ms_start_clock()                    
#define ms_handle_results(fper, run)
#define ms_initialize_run()
#define ms_mrun_print(t, f, nr, lim)
#define ms_mrun_initialize()
#endif

#endif /* MEASURES_H */
