/*        File: messaging.h
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: MPI
   Description: Header file for MPI specific functions, definitions, etc...
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/mpi/messaging.h,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:34 $

                $Log: messaging.h,v $
                Revision 1.1.1.1  2000/06/06 03:13:34  parint
                Initial checkin


       Version: 01/15/98  rrz  Original
		05/12/98  rrz  Code freeze for ParInt1.0
                09/17/98  jli  Changes for visualization
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
*/

/* This header file is for all defintions and prototypes that rely on the message passing
 * system being used.  As such, it is version specific to the extent that different versions
 * may use MPI, PVM, etc.  Anything in here that is even more version specific should go in
 * pi-version.h (since pi-version.h relies on this; e.g., items that differ between two
 * MPI versions, even if that item relies on mpi.h, go in pi-version.h).
 */

#ifndef MESSAGING
#define MESSAGING

#include <mpi.h>
#include <pi-version.h>
#include <pi-defs.h>

#define mpi_bool_t MPI_CHAR

#ifdef USING_LONG_DOUBLES
#define pi_base_mpi_t MPI_LONG_DOUBLE
#else
#define pi_base_mpi_t MPI_DOUBLE
#endif

extern void create_mpi_type_region(MPI_Datatype *region_mpi_t);
extern void create_mpi_type_update(MPI_Datatype *update_mpi_t);
extern void create_mpi_type_hist(MPI_Datatype *hist_mpi_t); /* jli!! */
extern void clean_message_buffers(MPI_Comm comm);

/* These wrappers always need to be defined as functions (to preserve return values) */
extern int pi_send_dummy(int source, int tag, MPI_Comm comm);
extern int pi_recv_dummy(int source, int tag, MPI_Comm comm, MPI_Status *status);
extern bool pi_iprobe(int source, int tag, MPI_Comm comm, MPI_Status *status);
    

#if defined(USE_MEASUREMENTS) || defined(USE_MSG_DEBUG)

#define pi_send(msg, cnt, typ, dest, tag, com)      _ms_pi_send(msg, cnt, typ, dest, tag, com)
#define pi_recv(msg, cnt, typ, src, tag, com, stat) _ms_pi_recv(msg, cnt, typ, src, tag, com, stat)
#define pi_probe(src, tag, com, stat)               _ms_pi_probe(src, tag, com, stat)
extern int _ms_pi_send(void *msg, int cnt, MPI_Datatype typ, int dest, int tag, MPI_Comm comm);
extern int _ms_pi_recv(void *msg, int cnt, MPI_Datatype typ, int src, int tag, MPI_Comm comm, 
		       MPI_Status *status);
extern int _ms_pi_probe(int src, int tag, MPI_Comm comm, MPI_Status *status);

#else

#define pi_send(msg, cnt, typ, dest, tag, com)      MPI_Send(msg, cnt, typ, dest, tag, com)
#define pi_recv(msg, cnt, typ, src, tag, com, stat) MPI_Recv(msg, cnt, typ, src, tag, com, stat)
#define pi_probe(src, tag, com, stat)               MPI_Probe(src, tag, com, stat)

#endif

#endif /* MESSAGING_H */
