/*        File: parint.c
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: MPI
   Description: *The* top-level parint executable (i.e., called by users)
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/mpi/parint.c,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:33 $

                $Log: parint.c,v $
                Revision 1.1.1.1  2000/06/06 03:13:33  parint
                Initial checkin


       Version: 03/01/98  rrz  Original
		05/12/98  rrz  Code freeze for ParInt1.0
		07/02/98  rrz  Added new cmd line arguments for visualization
		10/22/98  rrz  For keeping heap: Add cmd line option; print new
				rparm field
		11/14/98  rrz  Change to allow eps_r to be 0.0 if eps_a is >
				0.0
		12/18/98  rrz  Modify print_rparm to print rparm_p->simple_rgns
		12/27/98  rrz  Add printing of PI_MAX_NFCN and PI_MAX_NDIM to
				print_rparm()
		05/01/99  rrz  Remove code that is now in cmd-line-args.c
		05/11/99  rrz  Expand printing of rparms
		05/12/99  rrz  Update new field fcn_evals_per_rule within
				pi_status_t
		07/14/99  rrz  Remove killheap/keepheap option
		07/19/99  rrz  If init region has zero volume, return a zero
				result immediately
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
		02/18/00  rrz  Add to print_rparm() for max_heap_size
*/


#define DBG_THIS_FILE

#include <stdio.h>

#include <pi-vspec.h>
#include <pi-version.h>
#include <pi-defs.h>
#include <parint.h>
#include <pi-ifcn.h>
#include <math-misc.h>
#include <math-intg.h>
#include <math-adapt.h>     /* for clear_results() */
#include <math-rgn.h>       /* for region_has_zero_volume() */
#include <messaging.h>
#include <measures.h>
#include <cmd-line-args.h>


/* This is used in enough places that is is worth making it global: */
static int nPE;

#ifdef WE_ARE_AT_WMU
/*** Note that at some point this needs to go in a version-specific source file: */
extern void print_rparm(pi_rparm_t *rparm_p)
{
    printf("   VERSION: %s\n", VERSION_PHRASE);
    printf(" RUN PARMS: nPE %d; simp_rgns? %c; lb? %c; caw? %c; help_ratio %6.2f\n",
	   nPE, B2YN(rparm_p->simple_rgns), B2YN(rparm_p->do_lb), 
	   B2YN(rparm_p->ic_is_worker), rparm_p->lb_help_ratio);
    printf(" RUN PARMS: ns %2d; #runs %2d", rparm_p->ns, rparm_p->nruns);
    if (rparm_p->max_heap_size > 0)
	printf(" MaxHeapSz %d\n", rparm_p->max_heap_size);
    else
	printf(" MaxHeapSz: No limit\n");
    printf(" RUN PARMS: uservis %1c; vis-port %5d; plfile %s\n",
	   B2YN(rparm_p->vis_type), (int) rparm_p->vis_port,
	   (strlen(rparm_p->pl_file) ? rparm_p->pl_file : "<NONE>"));
    printf("COMP PARMS: MAX_NFCN %d; MAX_NDIM %d; Assertions? ",
	   PI_MAX_NFCN, PI_MAX_NDIM);
#ifdef USE_MY_ASSERTIONS
    printf("Y");
#else
    printf("N");
#endif    
    printf("; ldbls? ");
#ifdef USING_LONG_DOUBLES
    printf("Y");
#else
    printf("N");
#endif    
    printf("; Measuremnts? ");
#ifdef USE_MEASUREMENTS
    printf("Y\n");
#else
    printf("N\n");
#endif    
    
} /* print_rparm() */
#endif


/* This fn is called by multiple processes concurrently */
static int do_single_integral(pi_iparm_t *iparm_p, pi_rparm_t *rparm_p, int id)
{
    /* These variables are used by the controller (id==0): */
    pi_base_t result[PI_MAX_NDIM], estabs[PI_MAX_NDIM];
    pi_status_t status;

    /* These variables are used equally by all PE's executing this code: */
    bool result_will_be_zero;
    pi_ifcn_t ifcn_p;
    int run, nworkers;
    
    if (id == 0)
    {
#ifdef WE_ARE_AT_WMU
	print_rparm(rparm_p);
#endif
	print_iparm(iparm_p);
    }
    
    if (!(ifcn_p = pi_ifcn_lookup_ifcn(iparm_p->ifcn_name)))
	return pi_terminate(PI_ERR_PARMS, "Invalid integral function name");
    nworkers = nPE - (rparm_p->ic_is_worker ? 0 : 1);
    if (nworkers == 0)
	return pi_terminate(PI_ERR_NUM_PE, "ERROR: IC must be a worker if # of processes is one");

    if (rparm_p->nruns > 1)
	multi_run_init();

    result_will_be_zero = region_has_zero_volume(iparm_p->a, iparm_p->b, iparm_p->ndims);

    for (run = 0; run < rparm_p->nruns; ++run)
    {
	MPI_Barrier(MPI_COMM_WORLD);
	ms_initialize_run();
	
	if (id == IC_ID)
	{
	    /* Use bool's for internal parint boolean types, even though we present
	       int flags to the user (in pi_status_t): */
	    bool fcn_limit_flag;
	    
	    if (result_will_be_zero)
	    {
		clear_results(result, estabs, iparm_p->nfcns);
		status.fcn_limit_flag = 0;
		status.total_time = 0.0;
		status.fcn_eval_count = 0;
	    }
	    else
	    {
		integ_controller(ifcn_p, iparm_p->ndims, iparm_p->nfcns, iparm_p->intg_rule, 
				 iparm_p->fcn_eval_limit, iparm_p->eps_a, iparm_p->eps_r,
				 iparm_p->a, iparm_p->b, result, estabs, id, nworkers,
				 MPI_COMM_WORLD, rparm_p, &fcn_limit_flag, &status.fcn_eval_count,
				 &status.total_time, iparm_p->ifcn_name);
		status.fcn_limit_flag = fcn_limit_flag;
	    }
	}
	else
	{
	    if (!result_will_be_zero)
		integ_worker(ifcn_p, iparm_p->ndims, iparm_p->nfcns, iparm_p->intg_rule,
			     iparm_p->a, iparm_p->b, nworkers, id, MPI_COMM_WORLD, rparm_p);
	}
	    
	ms_handle_results(neval(iparm_p->ndims, iparm_p->intg_rule), run);
	clean_message_buffers(MPI_COMM_WORLD);
	if (rparm_p->nruns > 1)
	    multi_run_update(&status, run);
    }

    if (id == IC_ID)
    {
	if (rparm_p->nruns > 1)
	    multi_run_finish(&status, rparm_p->nruns, iparm_p->fcn_eval_limit);
	status.fcn_evals_per_rule = neval(iparm_p->ndims, iparm_p->intg_rule);
	pi_print_results(result, estabs, iparm_p->nfcns, &status);
    }

    return 0;
} /* do_single_integral() */


int main(int argc, char *argv[])
{
    /* These variables are used equally by all PE's executing this code: */
    int id, run_option;
    pi_iparm_t iparm;
    pi_rparm_t rparm;

    /* All PE's read this option, but note that only the controller tries to read from
       the script file: */
    char script_file[PI_MAX_PATH + 1];

#if 0
    /* This just to test the cmd line args; when doing this testing, parint can be
       run without any mpirun stuff: */
    nPE = 1;
    test_cmd_line_args(argc, argv, nPE);
    exit(0);
#endif

    MPI_Init(&argc, &argv);
    MPI_Comm_size(MPI_COMM_WORLD, &nPE);
    MPI_Comm_rank(MPI_COMM_WORLD, &id);

    if (argc > 1)
    {
	/* The user is supplying cmd-line arguments.  All PE's can read them.  The run_option
	   specifies what kind of run this is (help, info, single-integral, or 
	   script-file): */
	if (get_cmd_line_args(argc, argv, &run_option, &rparm, &iparm, script_file, nPE) != 0)
	{
	    pi_terminate(PI_ERR_PARMS, "Parint executable called with bad parameters");
	    exit(1);
	}
    }
    else if (id == 0)
    {
	/* We will use character based menus; I am the controller, so I will
	   read them in from the user and send them to the other PE's.  However, since the
	   doc for ParInt1.0 does not even mention character-based menus, there error
	   message simply states that there must be cmd line args: */
	/* read in and send options here */
	return pi_terminate(PI_ERR_PARMS, "Command line parameters must be specified");
    }
    else
    {
	/* We are using menus, but I am not the controller, so I will just
	   receive the values from the controller: */
	/* receive options here */
	return pi_terminate(PI_ERR_NO_SUPPORT, "ParInt does not yet support character menus");
    }

    switch (run_option)
    {
    case RUN_DO_HELP:
	print_long_usage();
	break;
    case RUN_DO_INFO_1:
	pi_ifcn_print_list_sm();
	break;
    case RUN_DO_INFO_2:
	pi_ifcn_print_list_lrg();
	break;
    case RUN_DO_INTEGRAL:
	do_single_integral(&iparm, &rparm, id);
	break;
    case RUN_DO_SCRIPT:
	return pi_terminate(PI_ERR_NO_SUPPORT, "ParInt does not yet support script files\n");
	break;
    default:
	pi_assert(FALSE, "Invalid run option");
	break;
    }
    
    MPI_Finalize();
    exit(0);
} /* main() */
