/*        File: pi-version.h
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: MPI
   Description: Header file, defined for each ParInt version.  Defines basic parameters.
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/mpi/pi-version.h,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:32 $

                $Log: pi-version.h,v $
                Revision 1.1.1.1  2000/06/06 03:13:32  parint
                Initial checkin


       Version: 01/13/98  rrz  Original
		05/12/98  rrz  Code freeze for ParInt1.0
		05/12/99  rrz  Add USE_COMM_MEASUREMENTS as a separate token
		07/13/99  rrz  Add tokens for logging
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
*/

/* This header file belongs in each version's directory, and will have the same form across
 * all versions, differing only in what items are enabled or disabled.  Note that if the 
 * Makefiles do not include all dependencies (which, as of now, they do not), then after making
 * a change in here, you should do a make clean before recompiling ParInt.
 */

#ifndef PI_VERSION_H
#define PI_VERSION_H

/* These control the debugging messages over all files.  NO_FILES overrules ALL_FILES.  If
 * neither is defined, then debugging is on a file-by-file basis: */
/* #define DBG_ALL_FILES */
#define DBG_NO_FILES

/* This enables compile-time options wanted when using ParInt to experiment with new
 * algorithms, etc...  Mostly it just turns on some more printing: */
/* #define WE_ARE_AT_WMU */

/* Enable to this add extra, paranoid, run-time checks: */
/* #define USE_MY_ASSERTIONS */

/* Enable this to use long doubles, else use doubles (for all results, regions, etc...): */
/* #define USING_LONG_DOUBLES */

/* This turns on the 'measurement' functionality.  It provides a chart of #sends per
 * # PE, per tag, comm time and total time, and # fn calls per worker: */
/* #define USE_MEASUREMENTS */

/* This turns on the communication time measurement funcitonality.  This is separate, as all of the
 * MPI_Wtime() calls slow things --w--a--y-- down.  This should *only* be turned on if
 * USE_MEASUREMENTS is turned on.  Note that if USE_MEASUREMENTS is on, and this is off, then
 * idle time will be counted: */
/* #define USE_COMM_MEASUREMENTS */

/* This turns on verbose debugging messages about each send, receive, and probe.  This is
 * preferred to just using the -ltmpi library, as there are a lot of iprobe() calls in parint,
 * and to have each iprobe result in a start and end debug message results in a bazillion lines
 * of output.  There is no reason why USE_MSG_DEBUG couldn't be turned off, while -ltmpi is 
 * turned on, if that is preferred: */
/* #define USE_MSG_DEBUG */

/* NOTE: If PI_MAX_NDIM changes, then the arrays in ifcn-ctrl.h must change.  This is annoying,
 * but vastly simplifies things.  This annoyance will pass when we use dynamically allocated
 * arrays: */
#define PI_MAX_NDIM   10             /* Max dimension over which integrating     */
#define PI_MAX_NFCN   10             /* Max number of integrands                 */

/* This defines the value of the first message tag.  It should be changed by users if
 * its installed value conflicts with the user's tag values: */
#define TAG_BASE  500

/* Logging stuff: */
// #define USE_LOGGING
#define PL_LOG_DIR 	"/tmp"

/*******************************************************************************************/
/*** Error check (compile-time) on the measurement tokens: */
#if (defined(USE_COMM_MEASUREMENTS) && !defined(USE_MEASUREMENTS))
#error COMPILE-TOKEN ERROR: USE_COMM_MEASUREMENTS can only be on if USE_MEASUREMENTS is on
#endif

#if (defined(USING_LONG_DOUBLES) && defined(USE_LOGGING))
#error COMPILE-TOKEN ERROR: USE_LOGGING and USING_LONG_DOUBLES cannot both be on
#endif

#endif /* PI_VERSION_H */

