/*        File: pi-vspec.h
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: MPI
   Description: Header file of version specific items
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/mpi/pi-vspec.h,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:32 $

                $Log: pi-vspec.h,v $
                Revision 1.1.1.1  2000/06/06 03:13:32  parint
                Initial checkin


       Version: 02/19/98  rrz  Original
		05/12/98  rrz  Code freeze for ParInt1.0
		06/18/98  rrz  Added fields to pi_rparm_t for visualization
		07/02/98  rrz  Added fields to pi_rparm_t for visualization
		07/03	  jli  added tag for visulization
		10/22/98  rrz  For keeping heap: Add iw_kill_heap to rparm_t
				structure
		12/18/98  rrz  For simple region selection: add field to
				rparm_t
		05/11/99  rrz  Add VERSION_PHRASE
		05/25/99  rrz  Add pl_file to pi_rparm_t
		07/14/99  rrz  Remove killheap/keepheap option
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
		02/18/00  rrz  Add to pi_rparms_t for max_heap_size
*/

/* This header file contains items which are specific to this version, and, may likely only exist
 * in this version (tag definitions, prototypes, etc...). 
 */

#ifndef PI_VSPEC_H
#define PI_VSPEC_H

#include <mpi.h>
#include <pi-version.h>
#include <parint.h>
#include <pi-defs.h>

/* The global rank of the integral controller and ctrl-as-worker.  These cannot be changed without
 * changing lots of IC and IW code: */
#define IC_ID        0
#define CAW_ID       0

/* This is not a crucial constant, as it is not used to declare large arrays.  We may want
 * a tighter definition, or a malloc()'ed approach, if its use changes.  And if we can run
 * ParInt beyond 64 workers, then I'll buy you lunch!: */
#define PI_MAX_WORKERS  64

/* Tags are numbered sequentially beginning at TAG_BASE.  When changing/adding tags, make
 * sure to update measures.c (si nec.): */
/* jli!! added tags TAG_HIST & TAG_VIS_THRESHOLD for visualization */
enum { TAG_INIT_UPDATE = TAG_BASE, TAG_INIT_EPS, TAG_INTG_STOP, TAG_WRKR_UPDATE, 
       TAG_NEW_EPS, TAG_LB_WRKR_IDLE, TAG_LB_HELPER_ID, TAG_LB_SEND_RGN, 
       TAG_MEASURES, TAG_HIST, TAG_VIS_THRESHOLD, TAG_MAX};

/* Used in an assertion in pi_send() and similar functions: */
#define valid_pi_tag(tag)     (((tag) >= TAG_BASE && (tag) < TAG_MAX) || ((tag) == MPI_ANY_TAG))

/* This is version-specific because the available run-time options depend upon the
 * specific version used (e.g., global heap size only changeable in global heap version): */
typedef struct 
{
    bool      do_lb;                  /* TRUE => do load balancing                   */
    bool      ic_is_worker;           /* Intg. controller is also Intg worker?       */
    bool      simple_rgns;            /* Forces init rgn calc to be simple           */
    int       nruns;                  /* Number of identical runs in this test       */
    double    lb_help_ratio;          /* Ratio below which help is refused           */
    int       ns;                     /* # of iterations for IW's between updates    */
    int       max_heap_size;          /* Max size of deap, or 0 if no limit          */
    int       vis_type;               /* Type of user visualization to do (enum type)  */
    short     vis_port;               /* TCP/IP port used for comm. with user for vis. */
    char      pl_file[PI_MAX_PATH+1]; /* logging file; only valid if strlen(pl_file) > 0 */
} pi_rparm_t;

/* Currently supported types of user visualization: */
enum { VIS_OFF = 0, VIS_ON };

/* These are default values for run-time-changeable parameters: */
#define DEF_DO_LB         TRUE
#define DEF_NUM_RUNS      1
#define DEF_LB_HELP_RATIO 1.2
#define DEF_NS            3
#define DEF_MAX_HEAP_SIZE 0
#define DEF_IC_IS_WORKER  TRUE
#define DEF_SIMPLE_RGNS   FALSE
#define DEF_VIS_TYPE      VIS_OFF

/*** This gets printed out by print_rparm(): */
#define VERSION_PHRASE       "ParInt1.1 (beta) MPI Version"

extern void integ_controller(pi_ifcn_t ifcn, int ndims, int nfcns, int irule, 
			     int fcn_eval_limit, pi_base_t eps_a, pi_base_t eps_r,
			     pi_base_t a[], pi_base_t b[], pi_base_t result[], pi_base_t estabs[], 
			     int id, int nworkers, MPI_Comm comm, pi_rparm_t *rparms_p, 
			     bool *fcn_limit_flag, int *fcn_eval_count, double *total_time,
			     char fcn_name[]);
extern void integ_worker(pi_ifcn_t ifcn, int ndims, int nfcns, int irule,
			 pi_base_t a[], pi_base_t b[], int nworkers, int id, MPI_Comm comm, 
			 pi_rparm_t *rparms_p);

extern void caw_init_first(MPI_Comm comm_parm, int id_parm);
extern void caw_init_intg(int ndims_parm, pi_base_t a_parm[], pi_base_t b_parm[], 
			  pi_ifcn_t ifcn_ptr_parm, int nfcns_parm, int irule_parm, 
			  int nworkers_parm, pi_rparm_t *rparm_p_parm);
extern void caw_first_pass(update_t **ctrl_update);
extern bool caw_check_ldone(pi_base_t g_eps, bool *locally_idle);
extern void caw_one_pass(pi_base_t g_eps, int helper_id, bool *use_update, update_t **ctrl_update, 
			 bool *locally_done, bool *used_helper);
extern void caw_finalize_intg(void);

extern void rparm_load_defaults(pi_rparm_t *rparm_p);

extern void multi_run_finish(pi_status_t *status_p, int nruns, int fcn_eval_limit);
extern void multi_run_init(void);
extern void multi_run_update(pi_status_t *status_p, int run);


#endif /* PI_VSPEC_H */
