/*        File: visual-ctrl.c
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: MPI (visualization)
   Description: Source to implement the visualization
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/mpi/visual-ctrl.c,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:34 $

                $Log: visual-ctrl.c,v $
                Revision 1.1.1.1  2000/06/06 03:13:34  parint
                Initial checkin


     Version:     7/3/98  jli  original code for visulization
 	        09/17/98  rrz  Remove compiler warning about sscanf() argument & close() prototype
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
*/            



/* This file contains most functions related to visualization;
   It's intended to easily seperate the original parint1.0 code
   with what I wrote */
#define DBG_THIS_FILE

#include <sys/types.h> 
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>    
#include <fcntl.h>
#include <unistd.h>
#include <strings.h>
#include <stdio.h>
#include <mpi.h>

#include <parint.h>
#include <math-adapt.h>
#include <pi-defs.h>
#include <messaging.h>
#include <pi-vspec.h>
#include <measures.h>
#include <heap.h>
#include <visual.h>


extern void caw_vis_get_hist(hist_t * hist,int hist_type) ;

static MPI_Datatype hist_mpi_t; /* MPI datatype for hsitogram */
static double vis_threshold = 0; /* visualization threshold */
static int vis_workerid = -1; /* worker to be queryed for histogram */ 
static int sockfd =0; /* the socket id */

/* send the histogram to the Applet */
static void vis_send_hist(int sockfd, hist_t hist);

static void send_to_each_worker(void *msg, int count, MPI_Datatype mtype, int tag, int num_non_ic_workers, MPI_Comm comm);

/* read request from the java Applet */
extern void vis_read_request(int new_sockfd,int num_non_ic_workers, bool ic_is_worker,MPI_Comm comm){

	char buf[128]; /* send, recv buffer */
	D(printf("[IC_VIS]wants to recv request from applet\n"));
	if (recv(new_sockfd, buf, sizeof(buf),0) >=0) {
			D(printf("[IC_VIS]recv buf:%s\n",buf));
                        if (buf[0]== 'T') {/*threshold */

                                vis_threshold = atof(buf + 1);
				D(printf("[IC_VIS]recv threshold:%f\n",vis_threshold));
                                send_to_each_worker(&vis_threshold, 1, MPI_DOUBLE, TAG_VIS_THRESHOLD, num_non_ic_workers, comm);
                        }
                        else if (buf[0]== 'H'){ /*histogram */
				hist_t hist;
				char dummy;
				double lowbound, highbound;

				sscanf(buf,"%c%c%d%c%lf%c%lf",&dummy,&dummy, &vis_workerid, 
				       &dummy, &lowbound, &dummy, &highbound);
				D(printf("[IC_VIS]recv initial hist request: %d % g % g\n",vis_workerid, lowbound,highbound));
				hist.worker_id = vis_workerid;
				hist.low_exp = lowbound;
				hist.high_exp = highbound;
				if (vis_workerid == IC_ID && ic_is_worker) {
					/* controller is also a worker */

					D(printf("[IC_VIS]send hist request to caw\n"));
					caw_vis_get_hist(&hist,1);
					vis_send_hist(new_sockfd,hist);
				}
				else if (vis_workerid > 0 && vis_workerid <= num_non_ic_workers)
                                	pi_send(&hist, 1,hist_mpi_t, vis_workerid, TAG_HIST,comm);
				else
					perror("error worker id");
                        }
        }
}

/* called by intg_controller(),
   initialize visualization */
extern void vis_init_connect(int portno,int num_non_ic_workers,int *new_sockfd,MPI_Comm comm){
	struct sockaddr_in inet_addr; /* the inet address */

	/* !jli */

	/* first, create mpi datatype for histogram */
	create_mpi_type_hist(&hist_mpi_t);
        if ((sockfd = socket(AF_INET,SOCK_STREAM,0)) <0){
                fprintf(stderr,"socket error\n");
                exit(1);
        }
        bzero((char *)&inet_addr,sizeof(inet_addr));
        inet_addr.sin_family = AF_INET;
        /*bcopy(hp->h_addr_list[0],(caddr_t)&inet_addr.sin_addr,hp->h_length);
	*/
	inet_addr.sin_addr.s_addr = htonl(INADDR_ANY);
        inet_addr.sin_port = htons(portno);
	D(printf("[IC_VIS]before bind\n"));
        if (bind(sockfd,(struct sockaddr *)&inet_addr, sizeof(inet_addr))<0){
                fprintf(stderr,"socket bind error\n");
                exit(1);
        }      
	if (listen(sockfd,5)<0){
                fprintf(stderr,"socket  listen error\n");
                exit(1);
        }
	D(printf("[IC_VIS]after listen\n"));
        *new_sockfd = accept(sockfd,(struct sockaddr *)0, (int *)0);
	D(printf("[IC_VIS]after accept\n"));
        if (*new_sockfd <0){
                perror("accept error");
                exit(1);
        }

	vis_read_request(*new_sockfd,num_non_ic_workers,0,comm);

	/* set to unblock I/O */
	if (fcntl(*new_sockfd,F_SETFL,O_NONBLOCK) <0){
		perror("fcntl");
		exit(1);
	}

}	

/* send local eps & # of regions to Applet */
extern void vis_send_update(int new_sockfd, int worker_id,update_t wrkr_update){

	
	char buf[128]; /* send, recv buffer */
	sprintf(buf,"%c %d %ld %.10f %.10f",'e',worker_id,wrkr_update.num_region, wrkr_update.l_estabs, wrkr_update.l_time);
	D(printf("[IC_VIS]update to send: %s\n",buf));
	if (send(new_sockfd,buf, strlen(buf) +1,0) <0){
		perror("send Applet update");
		exit(1);
	}

}

/* receive histogram from a worker, then send it to the Applet */
extern void vis_recv_hist(int new_sockfd,MPI_Comm comm){
	MPI_Status status;
	hist_t hist; /* the histogram values */

	if (pi_iprobe(MPI_ANY_SOURCE, TAG_HIST, comm , &status)) {                                  
                        pi_assert(vis_workerid != -1,"vis_worker_id equals -1");
			/* receive histogram information */
                        pi_recv(&hist, 1, hist_mpi_t, status.MPI_SOURCE, TAG_HIST, comm, &status);              
			D(printf("[IC-VIS]recv a hist from the worker: %d\n",status.MPI_SOURCE));
                        /* send histogram to Applet */
                        vis_send_hist(new_sockfd,hist);
			vis_workerid = -1;
                 
	}
}

/* send the histogram to the Applet */
static void vis_send_hist(int new_sockfd, hist_t hist){

	int i;
	
	char buf[1024]; /* send, recv buffer */
	sprintf(buf,"%c %d %.10f %.10f %.10f",'h',hist.worker_id,hist.low_exp, hist.high_exp, hist.time);
	
	for (i = 0;i<HISTOSIZE;i++)
		sprintf(buf,"%s %ld",buf,hist.num_region[i]);	
	pi_assert(strlen(buf) <= 1024, "size of hist buf too big!");
	D(printf("[IC_VIS]histogram to send: %s\n",buf));
	if (send(new_sockfd,buf, strlen(buf) +1,0) <0){
		perror("send Applet histogram");
		exit(1);
	}
	D(printf("[IC_VIS]after sending histogram \n"));

}


static void send_to_each_worker(void *msg, int count, MPI_Datatype mtype, int tag, int num_non_ic_workers, MPI_Comm comm)
{
    int i;
    /* The first non-ic-worker always has a rank of 1: */
    for (i = 1; i <= num_non_ic_workers; ++i)
        pi_send(msg, count, mtype, i, tag, comm);
} /* send_to_each_worker() */

/* return # region for controller as worker */
long vis_caw_get_num_region(hnode* p){
	return vis_get_num_region(p,vis_threshold);

}

void vis_caw_update_update(update_t *update_p, pi_base_t *estabs, int nfcns, double l_time, double rgnerr)
{
	long num_region;
	
	num_region = (rgnerr >=vis_threshold?1:0);
        vis_update_update(update_p,estabs,nfcns,l_time,num_region);
}

/* close the sockets */
void vis_close(int new_sockfd){
	close(sockfd);
	close(new_sockfd);
}
