/*        File: visual-wrkr.c
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: MPI (visualization)
   Description: Source to implement the visualization
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/mpi/visual-wrkr.c,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:33 $

                $Log: visual-wrkr.c,v $
                Revision 1.1.1.1  2000/06/06 03:13:33  parint
                Initial checkin


       Version:   7/3/98  jli  original code for visulization
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
		03/18/00  rrz  Add changes for using deap
*/



/* This file contains most functions related to visualization;
   It's intended to easily seperate the original parint1.0 code
   with what I wrote */

#define DBG_THIS_FILE

#include <sys/types.h> 
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>    
#include <fcntl.h>
#include <strings.h>
#include <stdio.h>
#include <mpi.h>

#include <parint.h>
#include <math-adapt.h>
#include <pi-defs.h>
#include <messaging.h>
#include <pi-vspec.h>
#include <measures.h>
#include <heap.h>

static pi_base_t min_rgnerr ; /* minimum region error in the heap, maintained for the histogram */ 


static void count_hist(double err, double bound[], hist_t* hist);

/* get number of regions of a heapnode */
/* this fn start from the root, and go through the heap until
the threshold is less than the threshold. Because of
the property of the heap we don't need to go through the
entire heap */
/* this is the recursive version */
long vis_get_num_region(hnode* p, double threshold){

	long count = 0;

	if (p != NULL){
		if ((p->rgn).rgnerr >= threshold){
			count ++;
			count += vis_get_num_region(p->lc_nptr,threshold);
			count += vis_get_num_region(p->rc_nptr,threshold);
			return count;
		}
		else
			return count;
	}
	else 
		return 0;
			


}
/* */
/*
long vis_get_num_region(hnode* p, double threshold){
	
	long count = 0;
	hnode *node = p,*tmp_node;

	if (node == NULL)
		return 0;

   while(1){
	   if (node->rgn.rgnerr>= threshold)
		count ++;
           if ( node->rgn.rgnerr>= threshold &&node->lc_nptr != NULL){
                tmp_node = node;   
                node = node->lc_nptr;
           }
           else if ( node->rgn.rgnerr>= threshold &&node->rc_nptr != NULL){
                tmp_node = node;
                node = node->rc_nptr;
           }
           else {
                while (node->p_nptr != NULL){
                        tmp_node = node->p_nptr;
                        if (tmp_node->rc_nptr == node ||(tmp_node->lc_nptr == node && tmp_node->rc_nptr == NULL))
 Jie */
                                /* back from left tree, and right subtree is nil */                          
                                /* or back from the right subtree */
/* Jie
                                node = node->p_nptr;
                        else  {
                                pi_assert(tmp_node->lc_nptr == node ,"left subtree has not been visited yet ");
                                break;  
                        }
                }
                if (node->p_nptr == NULL)
                        return count;
                tmp_node = node->p_nptr;
                node = tmp_node->rc_nptr;
           }
        }     


}
*/

/* get the histogram from the heap */
void vis_get_hist(hnode *p, hist_t * hist,int hist_type,double l_time){
   double bounds[HISTOSIZE +1];
   double high_exp,low_exp ;
   hnode *node;
   hnode *tmp_node = NULL;

   int i;

   node = p;

   pi_assert(p != NULL, "Null ptr in vis_get_hist() (may be a deap?)");
 
   D(printf("get hist: %d %f % f\n",hist->worker_id, hist->low_exp, hist->high_exp));
   if (node == NULL){
	D(printf("heap root is null\n"));
	return;
   }
   else {
	if ((int)(hist->low_exp) == -2 && (int)(hist->high_exp) == -2){
		high_exp = node->rgn.rgnerr;
		low_exp = min_rgnerr;
		hist->high_exp = high_exp;
		hist->low_exp = low_exp;
		D(printf("initial hist request for this worker\n"));
	}
	else {
		high_exp = hist->high_exp;
		low_exp = hist->low_exp;
	}
   }

   if (hist_type == 1) {/* only eps */
	for ( i = 0; i<HISTOSIZE + 1; i++){
		bounds[i] = low_exp + i* ((high_exp - low_exp)/HISTOSIZE);
		if (i <HISTOSIZE)
			hist->num_region[i] = 0;
	}
	while(1){
	   count_hist(node->rgn.rgnerr,bounds,hist);
	   if (node->lc_nptr != NULL){
		tmp_node = node;
		node = node->lc_nptr;
	   }
	   else if (node->rc_nptr != NULL){
		tmp_node = node;
		node = node->rc_nptr;
	   }
	   else {
		while (node->p_nptr != NULL){
			tmp_node = node->p_nptr;
			if (tmp_node->rc_nptr == node ||(tmp_node->lc_nptr == node && tmp_node->rc_nptr == NULL))
				/* back from left tree, and right subtree is nil */
				/* or back from the right subtree */
				node = node->p_nptr;
			else  {
				pi_assert(tmp_node->lc_nptr == node ,"left subtree has not been visited yet ");
				break;
			}
		}
	 	if (node->p_nptr == NULL)
			break;
		tmp_node = node->p_nptr;
		node = tmp_node->rc_nptr;
	   }/* else */
	}/* while */      
	hist->time = l_time;
   }/* if */
   D(printf("after calc hist: %d %f % f\n",hist->worker_id, hist->low_exp, hist->high_exp));
	
}

/* the following two fns are to 
 maintain the minimum region err */
void vis_init_min_err(region_t rgn){

	min_rgnerr = rgn.rgnerr;
}

void vis_min_rgn_err(region_t rgn) {

	if (rgn.rgnerr < min_rgnerr)
		min_rgnerr = rgn.rgnerr;

}

/* update # regions, estabs..etc. */
extern void vis_update_update(update_t *update_p, pi_base_t *estabs, int nfcns, double l_time, long num_region)
{
 
            update_p->l_estabs = infNorm(estabs,nfcns);
            update_p->l_time = l_time;
            update_p->num_region = num_region;
 
} /* vis_update_update() */

static void count_hist(double err, double bound[], hist_t* hist){
	
	int i;

	for ( i =0;i<HISTOSIZE;i++){
		if (err < bound[i+1] && err >= bound[i]){
			hist->num_region[i] += 1;
			break;
		}
	}


}
