/*        File: intg-seq-solve.c
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: SEQ (only!)
   Description: Source called to actually solve the integral
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/seq/intg-seq-solve.c,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:34 $

                $Log: intg-seq-solve.c,v $
                Revision 1.1.1.1  2000/06/06 03:13:34  parint
                Initial checkin


       Version: 05/01/99  rrz  Original
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
		03/18/00  rrz  Add changes for using deap
*/

#include <stdio.h>
#include <parint.h>
#include <pi-defs.h>
#include <pi-vspec.h>
#include <math-adapt.h>
#include <math-intg.h>
#include <math-rgn.h>
#include <heapdeap.h>

/* #define GET_SEQ_ERR_CURVE */


#ifdef GET_SEQ_ERR_CURVE
#include <pi-ifcn.h>
#include <math.h>
static bool corner_region(region_t *rgn_p, int ndims)
{
    register int i;
    for (i = 0; i < ndims; ++i)
    {
	if (rgn_p->center[i] - rgn_p->h_width[i] != 0.0)
	    return FALSE;
    }
    return TRUE;
} /* corner_region() */
#endif


extern void integ_seq_solve(pi_ifcn_t ifcn_ptr, pi_iparm_t *iparm_p, pi_rparm_t *rparm_p,
			    pi_base_t result[], pi_base_t estabs[], 
			    bool *fcn_limit_flag, int *fcncount)
{
    register int i;

    region_t rgn, new_rgns[REGION_SPLITS];
    region_t *new_rgn;
    bool done = FALSE;
    pi_base_t eps;
    heapdeap_t hpdp;
    int j;
    
    /* Set up some local vars for speed: */
    int nfcns = iparm_p->nfcns;
    int ndims = iparm_p->ndims;
    pi_base_t eps_a = iparm_p->eps_a;
    pi_base_t eps_r = iparm_p->eps_r;
    int irule = iparm_p->intg_rule;
    int fcn_eval_limit = iparm_p->fcn_eval_limit;
    bool using_deap = (rparm_p->max_heap_size > 0);
    int fcns_per_basrul = neval(ndims, irule);
    int max_heap_size;
    
#ifdef GET_SEQ_ERR_CURVE
    pi_base_t answer = (pi_ifcn_lookup_info(iparm_p->ifcn_name))->def_answer;
    int have_answer = (answer > 0.0);
    int iteration_number = 0;
#endif
    
    max_heap_size = using_deap ? rparm_p->max_heap_size : -1;

    /* Init parameters: */
    *fcn_limit_flag = FALSE;
    *fcncount = 0;

    clear_results(result, estabs, nfcns);

    D(printf("Starting work; using deap? %c; max heap size %d\n", B2YN(using_deap),
	     max_heap_size));

    {
	pi_base_t local_vol, full_vol;
	get_initial_region(ndims, 1, iparm_p->a, iparm_p->b, 0, &rgn, &full_vol, &local_vol, 
			   rparm_p);
    }

    work_initialize(ndims, irule);
    work_do_round(ndims, nfcns, ifcn_ptr, &rgn);
    heap_create(&hpdp, using_deap);
    heap_add_node(&hpdp, &rgn);
    
    /* In lieu of calling update_update(): */
    for (i = 0; i < nfcns; ++i)
    {
	result[i] += rgn.rgnval[i];                      
	estabs[i] += rgn.rgnerr_v[i];
    }
    *fcncount += fcns_per_basrul;

    D(printf("Initial work; err ratio %.2f; Fcn count %d; heap size %d\n", 
	     (double) (infNorm(estabs, nfcns) / calc_new_eps(eps_a, eps_r, result, nfcns)),
	     *fcncount, heap_size(&hpdp)));

#ifdef GET_SEQ_ERR_CURVE
    {
	pi_base_t log_est_err_ratio = log10(err_ratio(infNorm(estabs, nfcns), 
						      calc_new_eps(eps_a, eps_r, result, nfcns)));
	if (have_answer)
	{
	    if (corner_region(&rgn, ndims))
		printf("%10d %20f %20f %20f\n", iteration_number, log_est_err_ratio, 
		       log10(err_ratio(absol(answer - result[0]), max(eps_a, eps_r * answer))),
		       log_est_err_ratio);
	    
	    else
		printf("%10d %20f %20f\n", iteration_number, log_est_err_ratio,
		       log10(err_ratio(absol(answer - result[0]), max(eps_a, eps_r * answer))));
	}
	else
	{
	    if (corner_region(&rgn, ndims))
		printf("%10d %20f %20f\n", iteration_number, log_est_err_ratio, log_est_err_ratio);
	    
	    else
		printf("%10d %20f\n", iteration_number, log_est_err_ratio);
	}
	++iteration_number;
    }
#endif    

    if (*fcncount >= fcn_eval_limit)
    {
	done = TRUE;
	*fcn_limit_flag = TRUE;
    }
    else
    {
	eps = calc_new_eps(eps_a, eps_r, result, nfcns);
	if (result_close_enough(estabs, eps, nfcns))
	    done = TRUE;
    }
    
    while (!done)
    {
	/* Get a region with the largest error: */
	rgn = *heap_peek_max(&hpdp);

	for (i = 0; i < nfcns; i++)
	{
	    result[i] -= rgn.rgnval[i];
	    estabs[i] -= rgn.rgnerr_v[i];
	}
	
	D(printf("Next rgn to consider: %G\n", rgn.rgnerr));

	/* Split the retrieved region: */
	split_region(new_rgns, &rgn);

	/* Calc the integral over each of the new regions: */
	for (j = 0, new_rgn = new_rgns; j < REGION_SPLITS; j++, new_rgn++)
	{
	    /* This does a complete round of work, including the updating
	       of rgnerr and divaxn in new_rgns[j]: */
	    work_do_round(ndims, nfcns, ifcn_ptr, new_rgn);
 
	    /* Update result[] and estabs[] (in lieu of calling update_update()) and
	       put stuff back on the heap: */
	    for (i = 0; i < nfcns; ++i)
	    {
		result[i] += new_rgn->rgnval[i];
		estabs[i] += new_rgn->rgnerr_v[i];
	    }
	    *fcncount += fcns_per_basrul;

	    if (j == 0)
		heap_replace_root(&hpdp, new_rgn);
	    else
		heap_add_node(&hpdp, new_rgn);
	}

	D(printf("Finished work; err ratio %.2f; Fcn count %d; heap size %d\n", 
		 (double) (infNorm(estabs, nfcns) / calc_new_eps(eps_a, eps_r, result, nfcns)),
		 *fcncount, heap_size(&hpdp)));

	if (using_deap && heap_size(&hpdp) > max_heap_size)
	{
	    region_t rgn_to_throw_out;
	    D(deap_print(&hpdp, 0));
	    deap_delete_min(&hpdp, &rgn_to_throw_out);
	    D(printf("Threw out region %G\n", rgn_to_throw_out.rgnerr));
	    /* (note that the sum of the regions on the heap no longer corresponds with
	       the total error, local volume, etc.  This is ok, but may require closer
	       attention when future changes are made) */
	}

	/* See if we are done: */
	if (*fcncount >= fcn_eval_limit)
	{
	    done = TRUE;
	    *fcn_limit_flag = TRUE;
	}
	else
	{
	    eps = calc_new_eps(eps_a, eps_r, result, nfcns);
	    if (result_close_enough(estabs, eps, nfcns))
		done = TRUE;
	}

#ifdef GET_SEQ_ERR_CURVE
	{
	    pi_base_t log_est_err_ratio = log10(err_ratio(infNorm(estabs, nfcns), 
							  calc_new_eps(eps_a, eps_r,
								       result, nfcns)));
	    if (have_answer)
	    {
		if (corner_region(&rgn, ndims))
		    printf("%10d %20f %20f %20f\n", 
			   iteration_number, log_est_err_ratio, 
			   log10(err_ratio(absol(answer - result[0]), max(eps_a, eps_r * answer))),
			   log_est_err_ratio);
		
		else
		    printf("%10d %20f %20f  \n", 
			   iteration_number, log_est_err_ratio, 
			   log10(err_ratio(absol(answer - result[0]), 
					   max(eps_a, eps_r * answer))));
	    }
	    else
	    {
		if (corner_region(&rgn, ndims))
		    printf("%10d %20f %20f\n", 
			   iteration_number, log_est_err_ratio, log_est_err_ratio);
		
		else
		    printf("%10d %20f\n", iteration_number, log_est_err_ratio);
	    }
	    ++iteration_number;
	}
#endif    

    }

    /* Anything else to do here? */
    heap_free(&hpdp);

} /* integ_seq_solve() */

