/*        File: pi-vspec.h
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: SEQ (only!)
   Description: Header file of version specific items
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/seq/pi-vspec.h,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:34 $

                $Log: pi-vspec.h,v $
                Revision 1.1.1.1  2000/06/06 03:13:34  parint
                Initial checkin


       Version: 02/19/98  rrz  Original
		05/12/98  rrz  Code freeze for ParInt1.0
		06/18/98  rrz  Added fields to pi_rparm_t for visualization
		07/02/98  rrz  Added fields to pi_rparm_t for visualization
		07/03	  jli  added tag for visulization
		10/22/98  rrz  For keeping heap: Add iw_kill_heap to rparm_t
				structure
		12/18/98  rrz  For simple region selection: add field to
				rparm_t
		05/01/99  rrz  Copied from MPI version and MPI stuff removed
		05/11/99  rrz  Add VERSION_PHRASE
		02/18/00  rrz  Add to pi_rparms_t for max heap size
*/

/* This header file contains items which are specific to this version, and, may likely only exist
 * in this version (tag definitions, prototypes, etc...). 
 */

#ifndef PI_VSPEC_H
#define PI_VSPEC_H

#include <pi-version.h>
#include <parint.h>
#include <pi-defs.h>

/*** This gets printed out by print_rparm(): */
#define VERSION_PHRASE       "ParInt1.1 (beta) SEQ Version"

/* This is version-specific because the available run-time options depend upon the
 * specific version used (e.g., global heap size only changeable in global heap version):
 * (NOTE: I am not changing this for the SEQ version for simplicity sake even though there
 * are some fields I do not need.  When I separate out the rparms code from the iparms code
 * then I should do that.)
 */
typedef struct 
{
    bool      do_lb;                 /* TRUE => do load balancing                   */
    bool      ic_is_worker;          /* Intg. controller is also Intg worker?       */
    bool      simple_rgns;           /* Forces init rgn calc to be simple           */
    int       nruns;                 /* Number of identical runs in this test       */
    double    lb_help_ratio;         /* Ratio below which help is refused           */
    int       ns;                    /* # of iterations for IW's between updates    */
    int       max_heap_size;         /* Max size of deap, or 0 if no limit          */
    int       vis_type;              /* Type of user visualization to do (enum type)   */
    short     vis_port;              /* TCP/IP port used for comm. with user for vis.  */
    char      pl_file[PI_MAX_PATH+1]; /* not used in this version */
} pi_rparm_t;

/* These are default values for run-time-changeable parameters: */
#define DEF_DO_LB         TRUE
#define DEF_NUM_RUNS      1
#define DEF_LB_HELP_RATIO 1.2
#define DEF_NS            3
#define DEF_MAX_HEAP_SIZE 0
#define DEF_IC_IS_WORKER  TRUE
#define DEF_SIMPLE_RGNS   FALSE
#define DEF_VIS_TYPE      VIS_OFF

extern void integ_seq_solve(pi_ifcn_t ifcn_ptr, pi_iparm_t *iparm_p, pi_rparm_t *rparm_p,
			    pi_base_t result[], pi_base_t estabs[],
			    bool *fcn_limit_flag, int *fcncount);

/* Currently supported types of user visualization: */
/* (Needed by cmd-line-args for defaulting) */
enum { VIS_OFF = 0, VIS_ON };

#endif /* PI_VSPEC_H */
