/*        File: sparint.c
     Copyright: (C) 1998,1999,2000 by Elise de Doncker, Ajay Gupta, Alan Genz & Rodger Zanny. 
                    All rights reserved. See file COPYRIGHT in top-level directory.
  ParInt Versn: seq (only!)
   Description: Main source file for seq version
           CVS: $Source: /home/other/parint/cvsroot/parint1.1/src/seq/sparint.c,v $
                $Author: parint $
                $Revision: 1.1.1.1 $
                $Date: 2000/06/06 03:13:34 $

                $Log: sparint.c,v $
                Revision 1.1.1.1  2000/06/06 03:13:34  parint
                Initial checkin


       Version: 05/01/99  rrz  Original
		05/11/99  rrz  Expand print_rparms()
		07/19/99  rrz  If init region has zero volume, return a zero
				result immediately
		07/23/99  rrz  --- Code freeze for ParInt1.1 ---
		02/18/00  rrz  Add to print_rparm() for max_heap_size
		03/25/00  rrz  Move rparm_load_defaults() here from
				cmd-line-args.c
*/

#ifndef FORCE_NO_MPI
#error FORCE_NO_MPI must be defined within the makefile!
#endif

#include <stdio.h>
#include <stdarg.h>
#include <errno.h>

#include <pi-vspec.h>
#include <pi-version.h>
#include <pi-defs.h>
#include <parint.h>
#include <pi-ifcn.h>
#include <math-misc.h>
#include <math-intg.h>
#include <math-adapt.h>
#include <math-rgn.h>       /* for region_has_zero_volume() */
#include <cmd-line-args.h>
#include <timer.h>

extern void integ_seq_solve(pi_ifcn_t ifcn_ptr, pi_iparm_t *iparm_p, pi_rparm_t *rparm_p,
			    pi_base_t result[], pi_base_t estabs[], 
			    bool *fcn_limit_flag, int *fcncount);


#define MAX_ERR_LINE   256

extern void rparm_load_defaults(pi_rparm_t *rparm_p)
{
    rparm_p->do_lb         = DEF_DO_LB;
    rparm_p->nruns         = DEF_NUM_RUNS;
    rparm_p->lb_help_ratio = DEF_LB_HELP_RATIO;
    rparm_p->ic_is_worker  = DEF_IC_IS_WORKER;
    rparm_p->simple_rgns   = DEF_SIMPLE_RGNS;
    rparm_p->ns            = DEF_NS;
    rparm_p->max_heap_size = DEF_MAX_HEAP_SIZE;
    rparm_p->vis_type      = DEF_VIS_TYPE;
    rparm_p->vis_port      = (int) 0;
    memset(rparm_p->pl_file, '\0', PI_MAX_PATH + 1);
} /* rparm_load_defaults() */


/* The functions _failed_assertion() and pi_terminate() are normally provided by messaging.c,
 * but we have no such source file in this version
 */
extern void _failed_assertion(char *str, char *file, int line)
{
    fflush(stdout);
    printf("FAILED ASSERTION: %s\n"
	   " ASSERT LOCATION: File %s, Line %d\n", str, file, line);
    fflush(stdout);
    exit(1);
} /* _failed_assertion() */


extern int pi_terminate(int pi_err_num, const char *err_str)
{
    printf("FATAL PARINT ERROR: %s\n", err_str);
    exit(pi_err_num);
} /* pi_terminate() */



/* Terminates a run of ParInt.  Call this in circumstances where some system call failed, and
 * where you want the standard message coressponding to errno to be printed out
 * Allows for formatted message strings! (fancy! ;-) )
 */
extern void pi_sys_terminate(const char *fmt, /* arg1, arg2, */ ...)
{
    int errno_save = errno;
    char buf[MAX_ERR_LINE];
    va_list ap;
    
    va_start(ap, fmt);
    vsprintf(buf, fmt, ap);
    fflush(stdout);
    fprintf(stdout, "FATAL PARINT SYSTEM ERROR:\n");
    fprintf(stdout, "%s: %s\n", buf, strerror(errno_save));
    fflush(stdout);

    va_end(ap);
    exit(PI_ERR_INTERNAL);
} /* pi_sys_terminate() */


/* I had to copy this here from user-fcns.c (in the MPI version) b/c I cannot compile
 * user-fcns.o from this version, and this really does belong there.
 */
static void seq_print_results(pi_base_t result[], pi_base_t estabs[], int nfcns, 
			     pi_status_t *status_p, pi_iparm_t *iparm_p)
{
    int i;

    if (!result || !estabs || !status_p)
    {
	pi_terminate(PI_ERR_PARMS, "NULL argument to seq_print_results()");
	return;
    }

    if (nfcns == 1)
    {
	printf("\nRESULT: ");
#ifdef USING_LONG_DOUBLES
	printf("%.33LG\n", result[0]);
#else
	printf("%.15G\n", result[0]);
#endif
	printf("ESTABS: ");
#ifdef USING_LONG_DOUBLES
	printf("%.33LG\n", estabs[0]);
#else
	printf("%.15G\n", estabs[0]);
#endif
    }
    else
    {
#ifdef USING_LONG_DOUBLES
	printf("\nFn %38s %38s\n", "RESULT", "ESTABS");
#else
	printf("\nFcn# %21s %21s\n", "RESULT", "ESTABS");
#endif	
	for (i = 0; i < nfcns; i++)
	{
#ifdef USING_LONG_DOUBLES
	    printf("%2d %38.33LG %38.33LG\n", i, result[i], estabs[i]);
#else
	    printf("%3d  %21.15G %21.15G\n", i, result[i], estabs[i]);
#endif
	}
    }
    
    {
	int nregions = status_p->fcn_eval_count / neval(iparm_p->ndims, iparm_p->intg_rule);
	
	printf("STATUS: Fcn count: %d; Region count: %d; Fcn count flag: %d\n", 
	       status_p->fcn_eval_count, nregions, status_p->fcn_limit_flag);
	printf("        Time: %.3f; Time/1M: %.5f; Time/Region: %.4g\n", 
	       status_p->total_time, 
	       (status_p->fcn_eval_count > 0
		? (status_p->total_time / status_p->fcn_eval_count) * 1000000
		: 0.0),
	       (nregions > 0
		? status_p->total_time / nregions
		: 0.0));
    }
} /* seq_print_results() */


/*** Note that at some point this needs to go in a version-specific source file: */
extern void print_rparm(pi_rparm_t *rparm_p)
{
    printf("   VERSION: %s\n", VERSION_PHRASE);
    if (rparm_p->max_heap_size > 0)
	printf(" RUN PARMS: #runs %2d; MaxHeapSz: %d\n", rparm_p->nruns, rparm_p->max_heap_size);
    else
	printf(" RUN PARMS: #runs %2d; MaxHeapSz: No limit\n", rparm_p->nruns);
    printf("COMP PARMS: MAX_NFCN %d; MAX_NDIM %d; Assertions? ", PI_MAX_NFCN, PI_MAX_NDIM);
#ifdef USE_MY_ASSERTIONS
    printf("Y");
#else
    printf("N");
#endif    
    printf("; ldbls? ");
#ifdef USING_LONG_DOUBLES
    printf("Y");
#else
    printf("N");
#endif    
    printf("; Measuremnts? ");
#ifdef USE_MEASUREMENTS
    printf("Y\n");
#else
    printf("N\n");
#endif    
} /* print_rparm() */


static int do_single_integral(pi_iparm_t *iparm_p, pi_rparm_t *rparm_p)
{
    pi_base_t result[PI_MAX_NDIM], estabs[PI_MAX_NDIM];
    bool result_will_be_zero;
    pi_status_t status;
    pi_ifcn_t ifcn_p;
    int run;
    
    /* Use bool's for internal parint boolean types, even though we present
       int flags to the user (in pi_status_t): */
    bool fcn_limit_flag;

#ifdef WE_ARE_AT_WMU
	print_rparm(rparm_p);
#endif

    print_iparm(iparm_p);
    
    if (!(ifcn_p = pi_ifcn_lookup_ifcn(iparm_p->ifcn_name)))
	return pi_terminate(PI_ERR_PARMS, "Invalid integral function name");

    timer_start();
    
    result_will_be_zero = region_has_zero_volume(iparm_p->a, iparm_p->b, iparm_p->ndims);
    for (run = 0; run < rparm_p->nruns; ++run)
    {
	if (result_will_be_zero)
	{
	    clear_results(result, estabs, iparm_p->nfcns);
	    status.fcn_limit_flag = 0;
	    status.total_time = 0.0;
	    status.fcn_eval_count = 0;
	}
	else
	{
	    integ_seq_solve(ifcn_p, iparm_p, rparm_p, result, estabs, &fcn_limit_flag,
			    &status.fcn_eval_count);
	    status.fcn_limit_flag = fcn_limit_flag;
	}
    }

    status.total_time = timer_retrieve() / rparm_p->nruns;
    seq_print_results(result, estabs, iparm_p->nfcns, &status, iparm_p);

    return 0;
} /* do_single_integral() */


int main(int argc, char *argv[])
{
    pi_iparm_t iparm;
    pi_rparm_t rparm;
    int run_option;
    
    /* All PE's read this option, but note that only the controller tries to read from
       the script file: */
    char script_file[PI_MAX_PATH + 1];

#if 0
    /* This just to test the cmd line args; when doing this testing, parint can be
       run without any mpirun stuff: */
    test_cmd_line_args(argc, argv);
    exit(0);
#endif

    if (argc > 1)
    {
	/* The user is supplying cmd-line arguments.  All PE's can read them.  The run_option
	   specifies what kind of run this is (help, info, single-integral, or script-file): */
	if (get_cmd_line_args(argc, argv, &run_option, &rparm, &iparm, script_file, 1) != 0)
	{
	    pi_terminate(PI_ERR_PARMS, "Parint executable called with bad parameters");
	    exit(1);
	}
    }
    else
    {
	return pi_terminate(PI_ERR_NO_SUPPORT, "ParInt does not yet support character menus");
    }

    switch (run_option)
    {
    case RUN_DO_HELP:
	print_long_usage();
	break;
    case RUN_DO_INFO_1:
	pi_ifcn_print_list_sm();
	break;
    case RUN_DO_INFO_2:
	pi_ifcn_print_list_lrg();
	break;
    case RUN_DO_INTEGRAL:
	do_single_integral(&iparm, &rparm);
	break;
    case RUN_DO_SCRIPT:
	return pi_terminate(PI_ERR_NO_SUPPORT, "ParInt does not yet support script files\n");
	break;
    default:
	pi_assert(FALSE, "Invalid run option");
	break;
    }
    
    exit(0);

} /* main() */
